/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace CodeMetricsGui
{
    public partial class CDMtrGuiSetting : Form
    {
        private CDMtrGuiSetInfo pLocalSetInfo;

        private string[,] strCompVerArray = {
            {"VC80", "VisualC++ 8.0"},
            {"VC71", "VisualC++ 7.1"},
            {"VC70", "VisualC++ 7.0"},
            {"VC60", "VisualC++ 6.0"},
            {"VC50", "VisualC++ 5.0"},
            {"VC42", "VisualC++ 4.2"},
            {"VC41", "VisualC++ 4.1"},
            {"VC40", "VisualC++ 4.0"},
        };

        private string[,] strJavaCompVerArray = {
            {"JDK_1_5", "JDK1.5"},
            {"JDK_1_4", "JDK1.4"},
        };

        /// <summary>
        /// RXgN^
        /// </summary>
        public CDMtrGuiSetting(CDMtrGuiSetInfo setInfo)
        {
            InitializeComponent();

            this.pLocalSetInfo = setInfo;
            return;
        }

        /// <summary>
        /// RpCwRpC擾(C/C++)
        /// </summary>
        private string cdmtrGetCompVerFromName(string strCompName)
        {
            string strRet = "";

            for (int i = 0; i < this.strCompVerArray.GetLength(0); i++)
            {
                if (this.strCompVerArray[i,1].CompareTo(strCompName) == 0)
                {
                    strRet = this.strCompVerArray[i,0];
                }
            }

            return strRet;
        }

        /// <summary>
        /// RpCwRpC擾(JAVA)
        /// </summary>
        private string cdmtrGetJavaCompVerFromName(string strCompName)
        {
            string strRet = "";

            for (int i = 0; i < this.strJavaCompVerArray.GetLength(0); i++)
            {
                if (this.strJavaCompVerArray[i, 1].CompareTo(strCompName) == 0)
                {
                    strRet = this.strJavaCompVerArray[i, 0];
                }
            }

            return strRet;
        }

        /// <summary>
        /// wRpC񂩂RpC(C/C++)擾
        /// </summary>
        private string cdmtrGetCompNameFromVer(string strCompVer)
        {
            string strRet = "";

            for (int i = 0; i < this.strCompVerArray.GetLength(0); i++)
            {
                if (this.strCompVerArray[i, 0].CompareTo(strCompVer) == 0)
                {
                    strRet = this.strCompVerArray[i, 1];
                }
            }

            return strRet;
        }

        /// <summary>
        /// wRpC񂩂RpC(JAVA)擾
        /// </summary>
        private string cdmtrGetJavaCompNameFromVer(string strCompVer)
        {
            string strRet = "";

            for (int i = 0; i < this.strJavaCompVerArray.GetLength(0); i++)
            {
                if (this.strJavaCompVerArray[i, 0].CompareTo(strCompVer) == 0)
                {
                    strRet = this.strJavaCompVerArray[i, 1];
                }
            }

            return strRet;
        }

        /// <summary>
        /// ݒXV
        /// </summary>
        public void CDMtrSettingLoad(CDMtrGuiSetInfo setInfo)
        {
            this.PropTextBox.Text = setInfo.strPropFile;

            switch (setInfo.iRestyleCheck)
            {
                case (int)CDMtrGui.RESTYLE_MODE.BOTH:
                    this.RestyleRadio.Checked = false;
                    this.RestyleNoneRadio.Checked = false;
                    this.RestyleBothRadio.Checked = true;
                    break;
                case (int)CDMtrGui.RESTYLE_MODE.STD:
                    this.RestyleRadio.Checked = true;
                    this.RestyleNoneRadio.Checked = false;
                    this.RestyleBothRadio.Checked = false;
                    break;
                case (int)CDMtrGui.RESTYLE_MODE.NONE:
                default:
                    this.RestyleRadio.Checked = false;
                    this.RestyleNoneRadio.Checked = true;
                    this.RestyleBothRadio.Checked = false;
                    break;
            }

            this.DetailCheck.Checked = setInfo.bDetailCheck;
            this.DebugCheck.Checked = setInfo.bDebugCheck;

            for (int i = 0; i < setInfo.iDefNum; i++)
            {
                this.DefListBox.Items.Add(setInfo.strDefArray[i]);
            }

            for (int i = 0; i < setInfo.iIncNum; i++)
            {
                this.IncListBox.Items.Add(setInfo.strIncArray[i]);
            }

            for (int i = 0; i < setInfo.iSysIncNum; i++)
            {
                this.SysIncListBox.Items.Add(setInfo.strSysIncArray[i]);
            }

            for (int i = 0; i < setInfo.iOptionClassNum; i++)
            {
                this.OptionListBox.Items.Add(setInfo.strOptionClassArray[i]);
            }
            string[] strCompVerList = setInfo.strCompVer.Split(';');
            if (strCompVerList.GetLength(0) == 2)
            {
                this.CompVerCombo.Text = cdmtrGetCompNameFromVer(strCompVerList[0]);
                this.JavaCompVerCombo.Text = cdmtrGetJavaCompNameFromVer(strCompVerList[1]);
            }
            else if (strCompVerList.GetLength(0) == 1)
            {
                this.CompVerCombo.Text = cdmtrGetCompNameFromVer(strCompVerList[0]);
                if (this.CompVerCombo.Text == "")
                {
                    this.JavaCompVerCombo.Text = cdmtrGetJavaCompNameFromVer(strCompVerList[0]);
                }
            }
            else
            {
                this.CompVerCombo.Text = "";
                this.JavaCompVerCombo.Text = "";
            }


            this.FormatCheckBox.Checked = setInfo.bFormatCheck;
            this.FormatTextBox.Text = setInfo.strFormatTool;

            if (setInfo.strOutPutFormat.CompareTo("xml") == 0)
            {
                this.OutPutCSVRadio.Checked = false;
                this.OutPutXMLRadio.Checked = true;
            }
            else
            {
                this.OutPutXMLRadio.Checked = false;
                this.OutPutCSVRadio.Checked = true;
            }

            return;
        }

        /// <summary>
        /// ݒۑ
        /// </summary>
        private void CDMtrSettingSave()
        {
            pLocalSetInfo.strPropFile = this.PropTextBox.Text;
            pLocalSetInfo.strOptionFile = this.OptionTextBox.Text;
            String strCCompVer = "";
            String strJavaCompVer = "";

            if (this.RestyleBothRadio.Checked)
            {
                pLocalSetInfo.iRestyleCheck = (int)CDMtrGui.RESTYLE_MODE.BOTH;
            }
            else if (this.RestyleRadio.Checked)
            {
                pLocalSetInfo.iRestyleCheck = (int)CDMtrGui.RESTYLE_MODE.STD;
            }
            else
            {
                pLocalSetInfo.iRestyleCheck = (int)CDMtrGui.RESTYLE_MODE.NONE;
            }
            pLocalSetInfo.bDetailCheck = this.DetailCheck.Checked;
            pLocalSetInfo.bDebugCheck = this.DebugCheck.Checked;

            pLocalSetInfo.iDefNum = this.DefListBox.Items.Count;
            pLocalSetInfo.strDefArray = new string[pLocalSetInfo.iDefNum];
            for (int i = 0; i < pLocalSetInfo.iDefNum; i++)
            {
                pLocalSetInfo.strDefArray[i] = this.DefListBox.Items[i].ToString();
            }

            pLocalSetInfo.iIncNum = this.IncListBox.Items.Count;
            pLocalSetInfo.strIncArray = new string[pLocalSetInfo.iIncNum];
            for (int i = 0; i < pLocalSetInfo.iIncNum; i++)
            {
                pLocalSetInfo.strIncArray[i] = this.IncListBox.Items[i].ToString();
            }

            pLocalSetInfo.iSysIncNum = this.SysIncListBox.Items.Count;
            pLocalSetInfo.strSysIncArray = new string[pLocalSetInfo.iSysIncNum];
            for (int i = 0; i < pLocalSetInfo.iSysIncNum; i++)
            {
                pLocalSetInfo.strSysIncArray[i] = this.SysIncListBox.Items[i].ToString();
            }

            pLocalSetInfo.iOptionClassNum = this.OptionListBox.Items.Count;
            pLocalSetInfo.strOptionClassArray = new string[pLocalSetInfo.iOptionClassNum];
            for (int i = 0; i < pLocalSetInfo.iOptionClassNum; i++)
            {
                pLocalSetInfo.strOptionClassArray[i] = this.OptionListBox.Items[i].ToString();
            }

            if (this.CompVerCombo.SelectedItem != null)
            {
                strCCompVer = 
                    cdmtrGetCompVerFromName(this.CompVerCombo.SelectedItem.ToString());
            }

            if (this.JavaCompVerCombo.SelectedItem != null)
            {
                strJavaCompVer = 
                    cdmtrGetJavaCompVerFromName(this.JavaCompVerCombo.SelectedItem.ToString());
            }
           
            if ((strCCompVer != "") && (strJavaCompVer != "")) 
            { 
                pLocalSetInfo.strCompVer = strCCompVer + ";" + strJavaCompVer;
            }
            else if ((strCCompVer != "") && (strJavaCompVer == "")) 
            {
                pLocalSetInfo.strCompVer = strCCompVer;
            }
            else if ((strCCompVer == "") && (strJavaCompVer != "")) 
            {
                pLocalSetInfo.strCompVer = strJavaCompVer;
            }
            else
            {
                pLocalSetInfo.strCompVer = "";
            }

            pLocalSetInfo.bFormatCheck = this.FormatCheckBox.Checked;
            pLocalSetInfo.strFormatTool = this.FormatTextBox.Text;

            if (this.OutPutCSVRadio.Checked)
            {
                pLocalSetInfo.strOutPutFormat = "csv";
            }
            else if (this.OutPutXMLRadio.Checked)
            {
                pLocalSetInfo.strOutPutFormat = "xml";
            }
            else
            {
                pLocalSetInfo.strOutPutFormat = "csv";
            }

            return;
        }


        /// <summary>
        /// ݒt@CQƃ_CAO{^
        /// </summary>
        private void PropFileDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(PropTextBox.Text);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenFile(strSelectPath, CDMtrGui.FILENAME_PROP);
            if (strFilePath != null)
            {
                PropTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// ݒt@CҏW{^
        /// </summary>
        private void PropFileEditButton_Click(object sender, EventArgs e)
        {
            if (PropTextBox.Text != "")
            {
                try
                {
                    System.Diagnostics.Process.Start("notepad", PropTextBox.Text);
                }
                catch (Exception ex)
                {
                    CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.GuiSetting_01 + PropTextBox.Text);
                    CDMtrGuiDebug.write(ex.ToString());
                }
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// tH[}bgc[Qƃ_CAO{^
        /// </summary>
        private void FormatFileDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(FormatTextBox.Text);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenFile(strSelectPath, CDMtrGui.FILENAME_FORMATTOOL);
            if (strFilePath != null)
            {
                FormatTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// `ǉ{^
        /// </summary>
        private void DefAddButton_Click(object sender, EventArgs e)
        {
            if (DefTextBox.Text != "")
            {
                DefListBox.Items.Add(DefTextBox.Text);
                DefTextBox.Clear();
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// `폜{^
        /// </summary>
        private void DefDelButton_Click(object sender, EventArgs e)
        {
            if (DefListBox.SelectedItem != null)
            {
                DefListBox.Items.RemoveAt(DefListBox.SelectedIndex);
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// CN[hpXǉ{^
        /// </summary>
        private void IncAddButton_Click(object sender, EventArgs e)
        {
            if (IncTextBox.Text != "")
            {
                IncListBox.Items.Add(IncTextBox.Text);
                IncTextBox.Clear();
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// CN[hpX폜{^
        /// </summary>
        private void IncDelButton_Click(object sender, EventArgs e)
        {
            if (IncListBox.SelectedItem != null)
            {
                IncListBox.Items.RemoveAt(IncListBox.SelectedIndex);
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// VXeCN[hpXǉ{^
        /// </summary>
        private void SysIncAddButton_Click(object sender, EventArgs e)
        {
            if (SysIncTextBox.Text != "")
            {
                SysIncListBox.Items.Add(SysIncTextBox.Text);
                SysIncTextBox.Clear();
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// VXeCN[hpX폜{^
        /// </summary>
        private void SysIncDelButton_Click(object sender, EventArgs e)
        {
            if (SysIncListBox.SelectedItem != null)
            {
                SysIncListBox.Items.RemoveAt(SysIncListBox.SelectedIndex);
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// CN[hpXQƃ_CAO{^
        /// </summary>
        private void IncFileDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            if (IncTextBox.Text != "")
            {
                strSelectPath = CDMtrGuiPath.CDMtrPathGetDirName(IncTextBox.Text);
            }
            else
            {
                if (IncListBox.Items.Count > 0)
                {
                    strSelectPath = CDMtrGuiPath.CDMtrPathGetDirName(IncListBox.Items[IncListBox.Items.Count - 1].ToString());
                }
                else
                {
                    strSelectPath = CDMtrGuiPath.CDMtrPathGetCurrentDirectory();
                }
            }

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenDir(strSelectPath);
            if (strFilePath != null)
            {
                IncTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// VXeCN[hpXQƃ_CAO{^
        /// </summary>
        private void SysIncFileDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            if (SysIncTextBox.Text != "")
            {
                strSelectPath = CDMtrGuiPath.CDMtrPathGetDirName(SysIncTextBox.Text);
            }
            else
            {
                if (SysIncListBox.Items.Count > 0)
                {
                    strSelectPath = CDMtrGuiPath.CDMtrPathGetDirName(SysIncListBox.Items[SysIncListBox.Items.Count - 1].ToString());
                }
                else
                {
                    strSelectPath = CDMtrGuiPath.CDMtrPathGetCurrentDirectory();
                }
            }

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenDir(strSelectPath);
            if (strFilePath != null)
            {
                SysIncTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// OK{^ɐݒXV
        /// </summary>
        private void SettingOKButton_Click(object sender, EventArgs e)
        {
            CDMtrSettingSave();
            
            this.Close();
            return;
        }

        /// <summary>
        /// LZ{^
        /// </summary>
        private void SettingCancelButton_Click(object sender, EventArgs e)
        {
            this.Close();
            return;
        }

        /// <summary>
        /// `XgNA{^
        /// </summary>
        private void DefClearButton_Click(object sender, EventArgs e)
        {
            DefListBox.Items.Clear();
            return;
        }

        /// <summary>
        /// CN[hpXXgNA{^
        /// </summary>
        private void IncClearButton_Click(object sender, EventArgs e)
        {
            IncListBox.Items.Clear();
            return;
        }

        /// <summary>
        /// VXeCN[hpXXgNA{^
        /// </summary>
        private void SysIncClearButton_Click(object sender, EventArgs e)
        {
            SysIncListBox.Items.Clear();
            return;
        }

        /// <summary>
        /// `XgI
        /// </summary>
        private void DefListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (DefListBox.SelectedItem != null)
            {
                DefTextBox.Text = DefListBox.SelectedItem.ToString();
            }
            return;
        }

        /// <summary>
        /// CN[hpXXgI
        /// </summary>
        private void IncListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (IncListBox.SelectedItem != null)
            {
                IncTextBox.Text = IncListBox.SelectedItem.ToString();
            }
            return;
        }

        /// <summary>
        /// VXeCN[hpXXgI
        /// </summary>
        private void SysIncListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (SysIncListBox.SelectedItem != null)
            {
                SysIncTextBox.Text = SysIncListBox.SelectedItem.ToString();
            }
            return;
        }

        /// <summary>
        /// `Xgł̃L[
        /// </summary>
        private void DefListBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Delete)
            {
                if (DefListBox.SelectedItem != null)
                {
                    DefListBox.Items.RemoveAt(DefListBox.SelectedIndex);
                }
            }
            return;
        }

        /// <summary>
        /// CN[hpXXgł̃L[
        /// </summary>
        private void IncListBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Delete)
            {
                if (IncListBox.SelectedItem != null)
                {
                    IncListBox.Items.RemoveAt(IncListBox.SelectedIndex);
                }
            }
            return;
        }

        /// <summary>
        /// VXeCN[hpXXgł̃L[
        /// </summary>
        private void SysIncListBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Delete)
            {
                if (SysIncListBox.SelectedItem != null)
                {
                    SysIncListBox.Items.RemoveAt(SysIncListBox.SelectedIndex);
                }
            }
            return;
        }

        /// <summary>
        /// v⏕NXfBNgQƃ_CAO\{^
        /// </summary>
        private void OptionDirDialog_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(OptionTextBox.Text, OptionListBox);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenDir(strSelectPath);
            if (strFilePath != null)
            {
                OptionTextBox.Text = strFilePath;
            }
            else
            {
                
            }
        }

        /// <summary>
        /// v⏕NXt@CQƃ_CAO\{^
        /// </summary>
        private void OptionFileDialog_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(OptionTextBox.Text, OptionListBox);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenFile(strSelectPath, null);
            if (strFilePath != null)
            {
                OptionTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// v⏕NXt@Cǉ{^
        /// </summary>
        private void OptionAddButton_Click(object sender, EventArgs e)
        {
            if (OptionTextBox.Text != "")
            {
                OptionListBox.Items.Add(OptionTextBox.Text.Trim());
                OptionTextBox.Clear();
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// v⏕NXt@C폜{^
        /// </summary>
        private void OptionDelButton_Click(object sender, EventArgs e)
        {
            if (OptionListBox.SelectedItem != null)
            {
                OptionListBox.Items.RemoveAt(OptionListBox.SelectedIndex);
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// v⏕NXt@CNA{^
        /// </summary>
        private void OptionClearButton_Click(object sender, EventArgs e)
        {
            OptionListBox.Items.Clear();
            return;
        }

        /// <summary>
        /// hbOCxgŋʂɃR[郁\bh
        /// (LɂɂFormAllowDropvpeBTrueɂ)
        /// </summary>
        private void CmnBoxDragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.All;
            else
                e.Effect = DragDropEffects.None;
        }

        /// <summary>
        /// hbvCxgŋʂɃR[郁\bh
        /// (LɂɂFormAllowDropvpeBTrueɂ)
        /// </summary>
        private void CmnBoxDragDrop(object sender, DragEventArgs e)
        {
            string[] s = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            if (sender.GetType().Equals(typeof(TextBox)))
            {
                if (s.Length > 0) ((TextBox)sender).Text = s[0].Trim();
            }
            else if (sender.GetType().Equals(typeof(ListBox)))
            {
                for (int i = 0; i < s.Length; i++)
                    ((ListBox)sender).Items.Add(s[i].Trim());
            }
        }

        /// <summary>
        /// tH[[h
        /// </summary>
        private void CDMtrGuiSetting_Load(object sender, EventArgs e)
        {

#if OSS
            this.tabControl1.TabPages.Remove(this.tabPage2);
            this.tabControl1.TabPages.Remove(this.tabPageCompOpt);
#else
            
#endif
        }

    }
}