/*
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Rhino code, released
 * May 6, 1999.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1997-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Igor Bukanov, igor@fastmail.fm
 *   Norris Boyd,
 *   NEC Corporation.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 or later (the "GPL"), in which
 * case the provisions of the GPL are applicable instead of those above. If
 * you wish to allow use of your version of this file only under the terms of
 * the GPL and not to allow others to use your version of this file under the
 * MPL, indicate your decision by deleting the provisions above and replacing
 * them with the notice and other provisions required by the GPL. If you do
 * not delete the provisions above, a recipient may use your version of this
 * file under either the MPL or the GPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
package com.nec.jp.ncmeter.cmn.file;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ResourceBundle;

import com.nec.jp.ncmeter.cmn.log.NCMCmnLog;

/**
 * ファイル操作クラス
 */
public class NCMCmnFile {

    private ResourceBundle myResources;

    /**
     * ファイル出力
     */
    private boolean outputFile(String path, String head, String data) {
        boolean res = false;
        FileOutputStream fo = null;
        OutputStreamWriter fw = null;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {

            fo = new FileOutputStream(path, false);
            fw = new OutputStreamWriter(fo, "UTF-8");

            if (head != null) {

                fw.write(head);
            }
            fw.write(data);
            res = true;
        } catch (IOException ex) {
            try {
                if (myResources != null) {
                    String msg05601 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000056_01");
                    NCMCmnLog.outputLogError(msg05601 + ex.getMessage()); // MSG_LOG_ID_JAVA_00000056
                }
            } catch (Exception ex1) {
                NCMCmnLog.outputLogError(ex.getMessage());
            }
        } finally {
            try {
                if (fw != null) {
                    fw.close();
                    fw = null;
                }
            } catch (IOException ex) {
                try {
                    if (myResources != null) {
                        String msg05701 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000057_01");
                        NCMCmnLog.outputLogWarn(msg05701 + ex.getMessage()); // MSG_LOG_ID_JAVA_00000057
                    }
                } catch (Exception ex1) {
                    NCMCmnLog.outputLogError(ex.getMessage());
                }
            }
            try {
                if (fo != null) {
                    fo.close();
                    fo = null;
                }
            } catch (IOException ex) {
                try {
                    if (myResources != null) {
                        String msg05801 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000058_01");
                        NCMCmnLog.outputLogWarn(msg05801 + ex.getMessage()); // MSG_LOG_ID_JAVA_00000058
                    }
                } catch (Exception ex1) {
                    NCMCmnLog.outputLogError(ex.getMessage());
                }
            }

        }
        return res;
    }

    /**
     * ファイル出力
     */
    public boolean outputFile(String dir, String name, String head, String data) {

        String path = new File(dir, name).getPath();
        return outputFile(path, head, data);
    }

}
