/*
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Rhino code, released
 * May 6, 1999.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1997-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Igor Bukanov, igor@fastmail.fm
 *   Norris Boyd,
 *   NEC Corporation.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 or later (the "GPL"), in which
 * case the provisions of the GPL are applicable instead of those above. If
 * you wish to allow use of your version of this file only under the terms of
 * the GPL and not to allow others to use your version of this file under the
 * MPL, indicate your decision by deleting the provisions above and replacing
 * them with the notice and other provisions required by the GPL. If you do
 * not delete the provisions above, a recipient may use your version of this
 * file under either the MPL or the GPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
package com.nec.jp.ncmeter.cmn.log;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * ログ出力
 */
public class NCMCmnLog {
    private static final String logfmt = NCMCmnLogFormatter.class.getName();
    private static final Logger logger = Logger.getLogger(logfmt);

    public enum LEVEL {

        OFF,

        DEBUG,

        INFO,

        WARN,

        ERROR,
    }

    /**
     * ログ設定プロパティファイルのファイル内容
     */
    protected static final String LOGGING_PROPERTIES_DATA = "handlers=java.util.logging.ConsoleHandler\n"
            + "java.util.logging.ConsoleHandler.level=ALL\n"
            + "java.util.logging.ConsoleHandler.formatter=" + logfmt;

    /**
     * ログ設定の初期化
     */
    static {
        InputStream inStream = null;
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            inStream = new ByteArrayInputStream(
                    LOGGING_PROPERTIES_DATA.getBytes("UTF-8"));
            try {
                LogManager.getLogManager().readConfiguration(inStream);
            } catch (IOException e) {
                try {
                    if (myResources != null) {
                        String msg05901 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000059_01");
                        logger.warning(msg05901 + e.toString()); // MSG_LOG_ID_JAVA_00000059
                    }
                } catch (Exception ex) {
                    logger.warning(ex.toString());
                }
            }
            logger.setLevel(Level.SEVERE);
        } catch (UnsupportedEncodingException e) {
            try {
                if (myResources != null) {
                    String msg06001 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000060_01");
                    logger.severe(msg06001 + e.toString()); // MSG_LOG_ID_JAVA_00000060
                }
            } catch (Exception ex) {
                logger.severe(ex.toString());
            }
        } finally {
            try {
                if (inStream != null)
                    inStream.close();
            } catch (IOException e) {
                try {
                    if (myResources != null) {
                        String msg06101 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000061_01");
                        logger.warning(msg06101 + e.toString()); // MSG_LOG_ID_JAVA_00000061
                    }
                } catch (Exception ex) {
                    logger.warning(ex.toString());
                }
            }
        }
    }

    /**
     * 出力ログレベルの設定を行う
     */
    public static void setLogger(LEVEL n) {
        Level l = Level.OFF;
        switch (n) {
        case DEBUG:
            l = Level.FINE;
            break;
        case INFO:
            l = Level.INFO;
            break;
        case WARN:
            l = Level.WARNING;
            break;
        case ERROR:
            l = Level.SEVERE;
            break;
        case OFF:
        default:
            break;
        }
        logger.setLevel(l);
    }

    /**
     * デバッグログ出力
     */
    public static void outputLogDebug(String msg) {
        logger.fine(msg);
    }

    /**
     * 情報ログ出力
     */
    public static void outputLogInfo(String msg) {
        logger.info(msg);
    }

    /**
     * 警告ログ出力
     */
    public static void outputLogWarn(String msg) {
        logger.warning(msg);
    }

    /**
     * エラーログ出力
     */
    public static void outputLogError(String msg) {
        logger.severe(msg);
    }
}
