/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package com.nec.ncmeter;

import java.util.LinkedList;
import java.util.ListIterator;

/**
 * parse時に利用されるLOC計測用クラス
 */
public class NCMJParseMetricsData {

    enum LineStatus {

        LineKindNone,

        LineKindELOC,

        LineKindCLOC,

        LineKindBLOC
    };

    private int lastLine = 0;

    private LineStatus lastStatus = LineStatus.LineKindNone;

    private LinkedList<LineStatus> listLOCKind = new LinkedList<LineStatus>();

    /**
     * 関数の開始、終了行から関数内の行情報を取得する。
     */
    public NCMMethodLOCData getMethodLOC(int iBeginLine, int iEndLine) {
        int floc = 0;
        int eloc = 0;
        int cloc = 0;
        int bloc = 0;
        LineStatus iCurLineKind = LineStatus.LineKindNone;

        int iListIndex = iBeginLine - 1;
        ListIterator<LineStatus> iLOC = listLOCKind.listIterator(iListIndex);
        for (; iListIndex < iEndLine && iLOC.hasNext(); iListIndex++) {
            iCurLineKind = iLOC.next();
            switch (iCurLineKind) {
            case LineKindELOC:
                eloc++;
                break;
            case LineKindCLOC:
                cloc++;
                break;
            case LineKindBLOC:
                bloc++;
                break;
            default:
                break;
            }
        }

        floc = iEndLine - iBeginLine + 1;
        return new NCMMethodLOCData(floc, eloc, cloc, bloc);
    }

    /**
     * ELOCを登録する
     */
    public void countUpEloc(int iBeginLine, int iEndLine) {
        countUpLOC(iBeginLine, iEndLine, LineStatus.LineKindELOC);
        return;
    }

    /**
     * CLOCを登録する
     */
    public void countUpCloc(int iBeginLine, int iEndLine) {
        countUpLOC(iBeginLine, iEndLine, LineStatus.LineKindCLOC);
        return;
    }

    /**
     * ステータスに該当するLOCを登録する
     */
    private void countUpLOC(int iBeginLine, int iEndLine, LineStatus iStatus) {
        int iCurLine = lastLine;

        if (iCurLine == iBeginLine) {

            if (iStatus == LineStatus.LineKindELOC) {
                if (lastStatus == LineStatus.LineKindCLOC) {

                    listLOCKind.removeLast();
                    iCurLine--;
                } else {

                    return;
                }
            } else {

            }
        } else {

            for (; iCurLine < iBeginLine - 1; iCurLine++) {

                listLOCKind.add(LineStatus.LineKindBLOC);
            }
        }

        for (; iCurLine < iEndLine; iCurLine++) {

            listLOCKind.add(iStatus);

            lastStatus = iStatus;
        }

        lastLine = iCurLine;
        return;
    }
}
