/*
 * Copyright (C) 2007 J�lio Vilmar Gesser.
 *
 * This file is part of Java 1.5 parser and Abstract Syntax Tree.
 *
 * Java 1.5 parser and Abstract Syntax Tree is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java 1.5 parser and Abstract Syntax Tree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Java 1.5 parser and Abstract Syntax Tree.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 21/11/2006
 */
 /* Modified by NEC Corporation in 2015. */
package japa.parser.ast.body;

import japa.parser.ast.expr.AnnotationExpr;
import japa.parser.ast.expr.DimExpr;
import japa.parser.ast.expr.Expression;
import japa.parser.ast.type.Type;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * @author Julio Vilmar Gesser
 */
public final class AnnotationMemberDeclaration extends BodyDeclaration {

	private int modifiers;

	private Type type;

	private String name;

	private List<DimExpr> dims = null;

	private Expression defaultValue;

	public AnnotationMemberDeclaration() {
	}

	public AnnotationMemberDeclaration(int modifiers, Type type, String name, List<DimExpr> dims,
			Expression defaultValue) {
		this.modifiers = modifiers;
		this.type = type;
		this.name = name;
		this.dims = dims;
		this.defaultValue = defaultValue;
	}

	public AnnotationMemberDeclaration(JavadocComment javaDoc, int modifiers, List<AnnotationExpr> annotations,
			Type type, String name, List<DimExpr> dims, Expression defaultValue) {
		super(annotations, javaDoc);
		this.modifiers = modifiers;
		this.type = type;
		this.name = name;
		this.dims = dims;
		this.defaultValue = defaultValue;
	}

	public AnnotationMemberDeclaration(int beginLine, int beginColumn, int endLine, int endColumn,
			JavadocComment javaDoc, int modifiers, List<AnnotationExpr> annotations, Type type, String name,
			List<DimExpr> dims, Expression defaultValue) {
		super(beginLine, beginColumn, endLine, endColumn, annotations, javaDoc);
		this.modifiers = modifiers;
		this.type = type;
		this.name = name;
		this.dims = dims;
		this.defaultValue = defaultValue;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public Expression getDefaultValue() {
		return defaultValue;
	}

	/**
	 * Return the modifiers of this member declaration.
	 *
	 * @see ModifierSet
	 * @return modifiers
	 */
	public int getModifiers() {
		return modifiers;
	}

	public String getName() {
		return name;
	}

	public List<DimExpr> getDims() {
		return dims;
	}

	public Type getType() {
		return type;
	}

	public void setDefaultValue(Expression defaultValue) {
		this.defaultValue = defaultValue;
	}

	public void setModifiers(int modifiers) {
		this.modifiers = modifiers;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setDims(List<DimExpr> dims) {
		this.dims = dims;
	}

	public void setType(Type type) {
		this.type = type;
	}
}
