/*
 * Copyright (C) 2007 J�lio Vilmar Gesser.
 *
 * This file is part of Java 1.5 parser and Abstract Syntax Tree.
 *
 * Java 1.5 parser and Abstract Syntax Tree is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java 1.5 parser and Abstract Syntax Tree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Java 1.5 parser and Abstract Syntax Tree.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 03/11/2006
 */
 /* Modified by NEC Corporation in 2015. */
package japa.parser.ast.body;

import japa.parser.ast.Node;
import japa.parser.ast.expr.AnnotationExpr;
import japa.parser.ast.type.Type;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * @author Julio Vilmar Gesser
 */
public final class Parameter extends Node {

	private int modifiers;

	private List<AnnotationExpr> annotations;

	private Type type;

	private List<AnnotationExpr> mh_annotations = null;

	private boolean isVarArgs = false;

	private String name = null;

	private boolean isThis = false;

	private VariableDeclaratorId id;

	public Parameter() {
	}

	public Parameter(Type type, VariableDeclaratorId id) {
		this.type = type;
		this.id = id;
	}

	public Parameter(int modifiers, Type type, VariableDeclaratorId id) {
		this.modifiers = modifiers;
		this.type = type;
		this.id = id;
	}

	public Parameter(int beginLine, int beginColumn, int endLine, int endColumn, int modifiers,
			List<AnnotationExpr> annotations, Type type, boolean isVarArgs, VariableDeclaratorId id) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.modifiers = modifiers;
		this.annotations = annotations;
		this.type = type;
		this.isVarArgs = isVarArgs;
		this.id = id;
	}

	public Parameter(int beginLine, int beginColumn, int endLine, int endColumn, int modifiers,
			List<AnnotationExpr> annotations, Type type, List<AnnotationExpr> mh_annotations, boolean isVarArgs,
			String name, boolean isThis, VariableDeclaratorId id) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.modifiers = modifiers;
		this.annotations = annotations;
		this.type = type;
		this.mh_annotations = mh_annotations;
		this.isVarArgs = isVarArgs;
		this.name = name;
		this.isThis = isThis;
		this.id = id;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public List<AnnotationExpr> getAnnotations() {
		return annotations;
	}

	public VariableDeclaratorId getId() {
		return id;
	}

	public List<AnnotationExpr> getMHAnnotations() {
		return mh_annotations;
	}

	/**
	 * Return the modifiers of this parameter declaration.
	 *
	 * @see ModifierSet
	 * @return modifiers
	 */
	public int getModifiers() {
		return modifiers;
	}

	public Type getType() {
		return type;
	}

	public boolean isVarArgs() {
		return isVarArgs;
	}

	public String getName() {
		return name;
	}

	public boolean isThis() {
		return isThis;
	}

	public void setAnnotations(List<AnnotationExpr> annotations) {
		this.annotations = annotations;
	}

	public void setId(VariableDeclaratorId id) {
		this.id = id;
	}

	public void setMHAnnotations(List<AnnotationExpr> mh_annotations) {
		this.mh_annotations = mh_annotations;
	}

	public void setModifiers(int modifiers) {
		this.modifiers = modifiers;
	}

	public void setType(Type type) {
		this.type = type;
	}

	public void setVarArgs(boolean isVarArgs) {
		this.isVarArgs = isVarArgs;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setThis(boolean isThis) {
		this.isThis = isThis;
	}
}
