/*
 * Copyright (C) 2007 J�lio Vilmar Gesser.
 *
 * This file is part of Java 1.5 parser and Abstract Syntax Tree.
 *
 * Java 1.5 parser and Abstract Syntax Tree is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java 1.5 parser and Abstract Syntax Tree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Java 1.5 parser and Abstract Syntax Tree.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 05/10/2006
 */
 /* Modified by NEC Corporation in 2015. */
package japa.parser.ast.expr;

import japa.parser.ast.type.Type;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * @author Julio Vilmar Gesser
 */
public final class ArrayCreationExpr extends Expression {

	private Type type = null;

	private List<AnnotationExpr> annotations = null;

	private List<DimExpr> dims = null;

	private ArrayInitializerExpr initializer = null;

	private List<Expression> dimensions = null;

	public ArrayCreationExpr() {
	}

	public ArrayCreationExpr(Type type, List<DimExpr> dims, ArrayInitializerExpr initializer) {
		this.type = type;
		this.dims = dims;
		this.initializer = initializer;
		this.annotations = null;
		this.dimensions = null;
	}

	public ArrayCreationExpr(int beginLine, int beginColumn, int endLine, int endColumn, Type type, List<DimExpr> dims,
			ArrayInitializerExpr initializer) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.type = type;
		this.dims = dims;
		this.initializer = initializer;
		this.annotations = null;
		this.dimensions = null;
	}

	public ArrayCreationExpr(Type type, List<AnnotationExpr> annotations, List<Expression> dimensions,
			List<DimExpr> dims) {
		this.type = type;
		this.annotations = annotations;
		this.dimensions = dimensions;
		this.dims = dims;
		this.initializer = null;
	}

	public ArrayCreationExpr(int beginLine, int beginColumn, int endLine, int endColumn, Type type,
			List<AnnotationExpr> annotations, List<Expression> dimensions, List<DimExpr> dims) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.type = type;
		this.annotations = annotations;
		this.dimensions = dimensions;
		this.dims = dims;
		this.initializer = null;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public List<AnnotationExpr> getAnnotations() {
		return annotations;
	}

	public List<DimExpr> getDims() {
		return dims;
	}

	public List<Expression> getDimensions() {
		return dimensions;
	}

	public ArrayInitializerExpr getInitializer() {
		return initializer;
	}

	public Type getType() {
		return type;
	}

	public void setAnnotations(List<AnnotationExpr> annotations) {
		this.annotations = annotations;
	}

	public void setDims(List<DimExpr> dims) {
		this.dims = dims;
	}

	public void setDimensions(List<Expression> dimensions) {
		this.dimensions = dimensions;
	}

	public void setInitializer(ArrayInitializerExpr initializer) {
		this.initializer = initializer;
	}

	public void setType(Type type) {
		this.type = type;
	}

}
