/*
 * Copyright (C) 2007 J�lio Vilmar Gesser.
 *
 * This file is part of Java 1.5 parser and Abstract Syntax Tree.
 *
 * Java 1.5 parser and Abstract Syntax Tree is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java 1.5 parser and Abstract Syntax Tree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Java 1.5 parser and Abstract Syntax Tree.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 05/10/2006
 */
 /* Modified by NEC Corporation in 2015. */
package japa.parser.ast.expr;

import japa.parser.ast.type.ClassOrInterfaceType;
import japa.parser.ast.type.Type;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * @author Julio Vilmar Gesser
 */
public final class CastExpr extends Expression {

	private Type type = null;

	private List<ClassOrInterfaceType> additionalbounds = null;

	private Expression expr = null;

	public CastExpr() {
	}

	public CastExpr(Type type, List<ClassOrInterfaceType> additionalbounds, Expression expr) {
		this.type = type;
		this.additionalbounds = additionalbounds;
		this.expr = expr;
	}

	public CastExpr(int beginLine, int beginColumn, int endLine, int endColumn, Type type,
			List<ClassOrInterfaceType> additionalbounds, Expression expr) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.type = type;
		this.additionalbounds = additionalbounds;
		this.expr = expr;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public Expression getExpr() {
		return expr;
	}

	public List<ClassOrInterfaceType> getAdditionalBounds() {
		return additionalbounds;
	}

	public Type getType() {
		return type;
	}

	public void setExpr(Expression expr) {
		this.expr = expr;
	}

	public void setAdditionalBounds(List<ClassOrInterfaceType> additionalbounds) {
		this.additionalbounds = additionalbounds;
	}

	public void setType(Type type) {
		this.type = type;
	}

}
