/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package japa.parser.ast.expr;

import japa.parser.ast.body.Parameter;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

public final class LambdaParametersExpr extends Expression {

	private List<Parameter> parameters = null;
	private NameExpr names = null;
	private boolean isInterface = false;

	public LambdaParametersExpr() {
	}

	public LambdaParametersExpr(List<Parameter> parameters, NameExpr names, boolean isInterface) {
		this.parameters = parameters;
		this.names = names;
		this.isInterface = isInterface;
	}

	public LambdaParametersExpr(int beginLine, int beginColumn, int endLine, int endColumn, List<Parameter> parameters,
			NameExpr names, boolean isInterface) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.parameters = parameters;
		this.names = names;
		this.isInterface = isInterface;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public List<Parameter> getParameters() {
		return parameters;
	}

	public NameExpr getNames() {
		return names;
	}

	public boolean isInterface() {
		return isInterface;
	}

	public void setParameters(List<Parameter> parameters) {
		this.parameters = parameters;
	}

	public void setNames(NameExpr names) {
		this.names = names;
	}

	public void setInterface(boolean isInterface) {
		this.isInterface = isInterface;
	}

}
