/*
 * Copyright (C) 2007 J�lio Vilmar Gesser.
 *
 * This file is part of Java 1.5 parser and Abstract Syntax Tree.
 *
 * Java 1.5 parser and Abstract Syntax Tree is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java 1.5 parser and Abstract Syntax Tree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Java 1.5 parser and Abstract Syntax Tree.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 07/11/2006
 */
/* Modified by NEC Corporation in 2015. */
package japa.parser.ast.stmt;

import japa.parser.ast.body.VariableDeclaratorId;
import japa.parser.ast.expr.AnnotationExpr;
import japa.parser.ast.expr.Expression;
import japa.parser.ast.type.Type;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * @author Julio Vilmar Gesser
 */
public final class ForeachStmt extends Statement {

	private int modifiers = 0;

	private List<AnnotationExpr> annotations = null;

	private Type type = null;

	private VariableDeclaratorId id = null;

	private Expression iterable;

	private Statement body;

	public ForeachStmt() {
	}

	public ForeachStmt(int modifiers, List<AnnotationExpr> annotations, Type type, VariableDeclaratorId id,
			Expression iterable, Statement body) {
		this.modifiers = modifiers;
		this.annotations = annotations;
		this.type = type;
		this.iterable = iterable;
		this.body = body;
	}

	public ForeachStmt(int beginLine, int beginColumn, int endLine, int endColumn, int modifiers,
			List<AnnotationExpr> annotations, Type type, VariableDeclaratorId id, Expression iterable, Statement body) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.modifiers = modifiers;
		this.annotations = annotations;
		this.type = type;
		this.iterable = iterable;
		this.body = body;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public Statement getBody() {
		return body;
	}

	public Expression getIterable() {
		return iterable;
	}

	public VariableDeclaratorId getId() {
		return id;
	}

	public Type getType() {
		return type;
	}

	public List<AnnotationExpr> getAnnotations() {
		return annotations;
	}

	public int getModifiers() {
		return modifiers;
	}

	public void setBody(Statement body) {
		this.body = body;
	}

	public void setIterable(Expression iterable) {
		this.iterable = iterable;
	}

	public void setId(VariableDeclaratorId id) {
		this.id = id;
	}

	public void setType(Type type) {
		this.type = type;
	}

	public void setAnnotations(List<AnnotationExpr> annotations) {
		this.annotations = annotations;
	}

	public void setModifiers(int modifiers) {
		this.modifiers = modifiers;
	}
}
