/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package japa.parser.ast.type;

import japa.parser.Token;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

public final class ClassType extends Type {

    private Type left;

    private Type right;

    private Token orToken;

    public ClassType() {
    }

    public ClassType(Type left, Type right) {
        this.left = left;
        this.right = right;
    }

    public ClassType(int beginLine, int beginColumn, int endLine,
            int endColumn, Type left, Type right, Token orToken) {
        super(beginLine, beginColumn, endLine, endColumn);
        this.left = left;
        this.right = right;
        this.orToken = orToken;
    }

    @Override
    public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
        return v.visit(this, arg);
    }

    @Override
    public <A> void accept(VoidVisitor<A> v, A arg) {
        v.visit(this, arg);
    }

    public Type getLeft() {
        return left;
    }

    public Type getRight() {
        return right;
    }

    public Token getOrToken() {
        return orToken;
    }

    public void setLeft(Type left) {
        this.left = left;
    }

    public void setRight(Type right) {
        this.right = right;
    }

    public void setPipe(Token orToken) {
        this.orToken = orToken;
    }
}
