/*
 * Copyright (C) 2007 J�lio Vilmar Gesser.
 *
 * This file is part of Java 1.5 parser and Abstract Syntax Tree.
 *
 * Java 1.5 parser and Abstract Syntax Tree is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java 1.5 parser and Abstract Syntax Tree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Java 1.5 parser and Abstract Syntax Tree.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 05/10/2006
 */
/* Modified by NEC Corporation in 2015. */

package japa.parser.ast.type;

import japa.parser.ast.expr.AnnotationExpr;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * @author Julio Vilmar Gesser
 */
public final class PrimitiveType extends Type {

	private List<AnnotationExpr> annotations = null;

	public enum Primitive {
		Boolean, Char, Byte, Short, Int, Long, Float, Double
	}

	private Primitive type;

	public PrimitiveType() {
	}

	public PrimitiveType(Primitive type) {
		this.type = type;
	}

	public PrimitiveType(List<AnnotationExpr> annotations, Primitive type) {
		this.annotations = annotations;
		this.type = type;
	}

	public PrimitiveType(int beginLine, int beginColumn, int endLine, int endColumn, List<AnnotationExpr> annotations,
			Primitive type) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.annotations = annotations;
		this.type = type;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public List<AnnotationExpr> getAnnotations() {
		return annotations;
	}

	public Primitive getType() {
		return type;
	}

	public void setAnnotations(List<AnnotationExpr> annotations) {
		this.annotations = annotations;
	}

	public void setType(Primitive type) {
		this.type = type;
	}

}
