/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS

#include "CDMtrCmn.h"
#include "CDMtrExtCmn.h"
#include "CDMtrLibFileW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrLibSystemW.h"
#include "CDMtrCntXrefDataUtil.h"
#include "CDMtrCntMain.h"
#include <memory.h>
#include <string.h>
#include "CDMtrCntFanIO.h"
#include "CDMtrMessage.h"

/*----------------------------------------------------------------------------*/
/* static variable declaration                                                */
/*----------------------------------------------------------------------------*/
const unsigned int gXrefBitPaternFinProc = XREF_S_CATEGORY_FUNCTION_BIT;
const unsigned int gXrefBitPaternFinGlobal = XREF_VARIABLE_TYPE_G_BIT;

const unsigned int gXrefBitPaternFinParam = XREF_VARIABLE_TYPE_P_BIT;


const unsigned int gXrefBitPaternFoutProc = XREF_S_CATEGORY_FUNCTION_BIT;

const unsigned int gXrefBitPaternFoutGlobal
    = XREF_A_MODIFIED_MODIFIED_BIT | XREF_VARIABLE_TYPE_G_BIT;

const unsigned int gXrefBitPaternFoutParam
    = XREF_A_MODIFIED_MODIFIED_BIT | XREF_VARIABLE_TYPE_P_BIT;


#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntFanIOMain")

/*----------------------------------------------------------------------------*/
/* global function                                                            */
/*----------------------------------------------------------------------------*/
/*!
    @brief      FANIN/FANOUT̃JEgs
*/
int CDMtrCntFanIOMain(_TCHAR *pcXrefFile, fanCounterList *pFanCountList)
{
    fanCounter *pFanCounterList = NULL;
    fanXrefDataList xrefDataList;
    int dataSize = 0, funcListSize = 0;
    int staticDataMax = 0;
    int numF = 0;
    int isStatic = 0, i = 0;
    int xrefDataNum;
    int iRet = RET_NG;
    xrefData **ppStaticDataList = NULL;
    xrefData *pXrefData = NULL;
    xrefData *pSData = NULL;
    xrefData **ppXrefDataList = NULL;
    nameStack foutNameStackOfProcCall;
    nameStack foutNameStackOfGlobalVariablesWrittenTo;
    nameStack foutNameStackOfByRefParamWrittenTo;
    nameStack finNameStackOfCallThisProc;
    nameStack finNameStackOfGlobalVariablesRead;
    nameStack finNameStackOfParametersRead;
    memset(&xrefDataList, 0, sizeof(fanXrefDataList));
    memset(&foutNameStackOfProcCall, 0, sizeof(nameStack));
    memset(&foutNameStackOfGlobalVariablesWrittenTo, 0, sizeof(nameStack));
    memset(&foutNameStackOfByRefParamWrittenTo, 0, sizeof(nameStack));
    memset(&finNameStackOfCallThisProc, 0, sizeof(nameStack));
    memset(&finNameStackOfGlobalVariablesRead, 0, sizeof(nameStack));
    memset(&finNameStackOfParametersRead, 0, sizeof(nameStack));
    xrefDataList.pSrcList = CDMtrCntXrefDataUtilLoadXrefByFileAndMakeHashTable(pcXrefFile, &dataSize);
    if (xrefDataList.pSrcList == NULL) {
        DEBUG_LOG((MSG_LOG_ID_C_00000022)); //* MSG_LOG_ID_C_00000022
        iRet = RET_OK;
        goto FUNC_END;
    }
    xrefDataList.numOfSrcList = dataSize;
    DEBUG_LOG((MSG_LOG_ID_C_00000023, pcXrefFile, dataSize)); //* MSG_LOG_ID_C_00000023
    pFanCounterList = CDMtrCntXrefDataUtilGetFunctionList(&xrefDataList, &funcListSize);
    if ( pFanCounterList == NULL ) {
        DEBUG_LOG((MSG_LOG_ID_C_00000024)); //* MSG_LOG_ID_C_00000024
        goto FUNC_END;
    }
    for (numF = 0; numF < funcListSize; numF++) {
        CDMtrCntXrefDataUtilInitNameStack(&foutNameStackOfProcCall);
        CDMtrCntXrefDataUtilInitNameStack(&foutNameStackOfGlobalVariablesWrittenTo);
        CDMtrCntXrefDataUtilInitNameStack(&foutNameStackOfByRefParamWrittenTo);
        CDMtrCntXrefDataUtilInitNameStack(&finNameStackOfCallThisProc);
        CDMtrCntXrefDataUtilInitNameStack(&finNameStackOfGlobalVariablesRead);
        CDMtrCntXrefDataUtilInitNameStack(&finNameStackOfParametersRead);
        pXrefData = pFanCounterList[numF].pXrefData;
        ppStaticDataList = xrefDataList.ppStaticList;
        staticDataMax = xrefDataList.numOfStaticList;
        if (_tcscmp(pXrefData->d_dspec, XREF_D_DSPEC_STATIC) == 0) {
            isStatic = 1;
        }
        ppXrefDataList = NULL;
        ppXrefDataList = gXrefAfuncHashTable[pXrefData->s_name_hash % HASH_SIZE].ppXrefDataList;
        xrefDataNum = gXrefAfuncHashTable[pXrefData->s_name_hash % HASH_SIZE].xrefDataNum;
        for (i = 0; i < xrefDataNum; i++ ) {
            pSData = ppXrefDataList[i];
            if (pSData->a_func_hash == pXrefData->s_name_hash) {
                if (pSData->bitFlag == gXrefBitPaternFoutProc) {
                    if (
                        !CDMtrCntXrefDataUtilIsStacked(&foutNameStackOfProcCall, pSData->s_name)
                        && (isStatic ?
                            (pXrefData->s_id_hash == pSData->a_id_hash && !_tcscmp(pXrefData->s_id, pSData->a_id)) :
                            !CDMtrCntXrefDataUtilIsSameSid(ppStaticDataList, staticDataMax, pSData->a_id, pSData->a_id_hash))
                    ) {
                        if (_tcscmp(pSData->a_func, pXrefData->s_name) == 0) {
                            pFanCounterList[numF].foutNumOfProcCall++;
                            iRet = CDMtrCntXrefDataUtilPushNameStack(&foutNameStackOfProcCall, pSData->s_name);
                            if (iRet == RET_NG) {
                                goto FUNC_END;
                            }
                        }
                    }
                } else if (pSData->bitFlag == gXrefBitPaternFoutGlobal) {
                    if (
                        !CDMtrCntXrefDataUtilIsStacked(&foutNameStackOfGlobalVariablesWrittenTo, pSData->s_name)
                        && (isStatic ?
                            (pXrefData->s_id_hash == pSData->a_id_hash && !_tcscmp(pXrefData->s_id, pSData->a_id)) :
                            !CDMtrCntXrefDataUtilIsSameSid(ppStaticDataList, staticDataMax, pSData->a_id, pSData->a_id_hash))
                    ) {
                        if (_tcscmp(pSData->a_func, pXrefData->s_name) == 0) {
                            pFanCounterList[numF].foutNumOfGlobalVariablesWrittenTo++;
                            iRet = CDMtrCntXrefDataUtilPushNameStack(&foutNameStackOfGlobalVariablesWrittenTo, pSData->s_name);
                            if (iRet == RET_NG) {
                                goto FUNC_END;
                            }
                        }
                    }
                } else if (pSData->bitFlag == gXrefBitPaternFoutParam) {
                    if (_tcschr(pSData->d_type, _T('*')) != NULL) {
                        if (
                            !CDMtrCntXrefDataUtilIsStacked(&foutNameStackOfByRefParamWrittenTo, pSData->s_name)
                            && (isStatic ?
                                (pXrefData->s_id_hash == pSData->a_id_hash && !_tcscmp(pXrefData->s_id, pSData->a_id)) :
                                !CDMtrCntXrefDataUtilIsSameSid(ppStaticDataList, staticDataMax, pSData->a_id, pSData->a_id_hash))
                        ) {
                            if (_tcscmp(pSData->a_func, pXrefData->s_name) == 0) {
                                pFanCounterList[numF].foutNumOfByRefParamWrittenTo++;
                                iRet = CDMtrCntXrefDataUtilPushNameStack(&foutNameStackOfByRefParamWrittenTo, pSData->s_name);
                                if (iRet == RET_NG) {
                                    goto FUNC_END;
                                }
                            }
                        }
                    }
                } else if (pSData->bitFlag == gXrefBitPaternFinGlobal) {
                    if (
                        !CDMtrCntXrefDataUtilIsStacked(&finNameStackOfGlobalVariablesRead, pSData->s_name)
                        && (isStatic ?
                            (pXrefData->s_id_hash == pSData->a_id_hash && !_tcscmp(pXrefData->s_id, pSData->a_id)) :
                            !CDMtrCntXrefDataUtilIsSameSid(ppStaticDataList, staticDataMax, pSData->a_id, pSData->a_id_hash))
                    ) {
                        if (_tcscmp(pSData->a_func, pXrefData->s_name) == 0) {
                            pFanCounterList[numF].finNumOfGlobalVariablesRead++;
                            iRet = CDMtrCntXrefDataUtilPushNameStack(&finNameStackOfGlobalVariablesRead, pSData->s_name);
                            if (iRet == RET_NG) {
                                goto FUNC_END;
                            }
                        }
                    }
                } else if (pSData->bitFlag == gXrefBitPaternFinParam) {
                    if (
                        !CDMtrCntXrefDataUtilIsStacked(&finNameStackOfParametersRead, pSData->s_name)
                        && (isStatic ?
                            (pXrefData->s_id_hash == pSData->a_id_hash && !_tcscmp(pXrefData->s_id, pSData->a_id)) :
                            !CDMtrCntXrefDataUtilIsSameSid(ppStaticDataList, staticDataMax, pSData->a_id, pSData->a_id_hash))
                    ) {
                        if (_tcscmp(pSData->a_func, pXrefData->s_name) == 0) {
                            pFanCounterList[numF].finNumOfParametersRead++;
                            iRet = CDMtrCntXrefDataUtilPushNameStack(&finNameStackOfParametersRead, pSData->s_name);
                            if (iRet == RET_NG) {
                                goto FUNC_END;
                            }
                        }
                    }
                }
            }
        }
        ppXrefDataList = gXrefSnameHashTable[pXrefData->s_name_hash % HASH_SIZE].ppXrefDataList;
        xrefDataNum = gXrefSnameHashTable[pXrefData->s_name_hash % HASH_SIZE].xrefDataNum;
        for (i = 0; i < xrefDataNum; i++ ) {
            pSData = ppXrefDataList[i];
            if (pSData->s_name_hash == pXrefData->s_name_hash) {
                if (pSData->bitFlag == gXrefBitPaternFinProc ) {
                    if (
                        !CDMtrCntXrefDataUtilIsStacked(&finNameStackOfCallThisProc, pSData->a_id)
                        && (isStatic ?
                            (pXrefData->s_id_hash == pSData->s_id_hash && !_tcscmp(pXrefData->s_id, pSData->s_id)) :
                            !CDMtrCntXrefDataUtilIsSameSid(ppStaticDataList, staticDataMax, pSData->s_id, pSData->s_id_hash))
                    ) {
                        if (_tcscmp(pSData->s_name, pXrefData->s_name) == 0) {
                            pFanCounterList[numF].finNumOfCallThisProc++;
                            iRet = CDMtrCntXrefDataUtilPushNameStack(&finNameStackOfCallThisProc, pSData->a_id);
                            if (iRet == RET_NG) {
                                goto FUNC_END;
                            }
                        }
                    }
                }
            }
        }
    }
    pFanCountList->pFanCounter = pFanCounterList;
    pFanCountList->iListSize = funcListSize;
    iRet = RET_OK;
FUNC_END:
    CDMtrCntXrefDataUtilFreeNameStackData(&foutNameStackOfProcCall);
    CDMtrCntXrefDataUtilFreeNameStackData(&foutNameStackOfGlobalVariablesWrittenTo);
    CDMtrCntXrefDataUtilFreeNameStackData(&foutNameStackOfByRefParamWrittenTo);
    CDMtrCntXrefDataUtilFreeNameStackData(&finNameStackOfCallThisProc);
    CDMtrCntXrefDataUtilFreeNameStackData(&finNameStackOfGlobalVariablesRead);
    CDMtrCntXrefDataUtilFreeNameStackData(&finNameStackOfParametersRead);
    return iRet;
}

#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntFanIOStructFileToFanTempFile")
/*!
    @brief  \gNX̃gNXt@C𒆊ԃt@CɃRs[
*/
int CDMtrCntFanIOStructFileToFanTempFile(_TCHAR *pcInName, int iLangType)
{
    int iRet = RET_NG;
    int iFuncRet;
    FILE *fpIn = NULL;
    _TCHAR *tReadBuffer = NULL;
    _TCHAR *pcRet = NULL;
    long lSize;
    int iLineCount = 0;
    FILE *fpout = NULL;
    int bomOffset = 0;
    if ((fpIn = _tfopen(pcInName, _T("r, ccs=UTF-8"))) == NULL) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000387, pcInName); //* MSG_LOG_ID_C_00000387
        goto FUNC_END;
    }
    if ((fpout = _tfopen(pcTempFio, _T("a+, ccs=UTF-8"))) == NULL) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000388, pcTempFio); //* MSG_LOG_ID_C_00000388
        goto FUNC_END;
    }
    bomOffset = ftell(fpIn);
    if (fseek(fpIn, 0L, SEEK_END) != 0) {
        goto FUNC_END;
    }
    lSize = ftell(fpIn);
    if (lSize <= 0) {
        goto FUNC_END;
    }
    lSize++;
    if (fseek(fpIn, bomOffset, SEEK_SET) != 0) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibUtilMallocW((void **)&tReadBuffer, (lSize + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    while (1) {
        pcRet = _fgetts(tReadBuffer, (int)lSize, fpIn);
        if (pcRet == NULL) {
            break;
        }
        _fputts(tReadBuffer, fpout);
        iLineCount++;
    }
    iRet = RET_OK;
FUNC_END:
    if (fpIn) {
        fclose(fpIn);
        fpIn = NULL;
    }
    if (fpout) {
        fclose(fpout);
        fpout = NULL;
    }
    CDMtrLibUtilFreeW((void **)&tReadBuffer);
    return iRet;
}

#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntFanIOErrFileToFanTempFile")
/*!
    @brief          ͎šʂ𑪒茋ʃt@C(ԃt@C)ɏo

*/
int CDMtrCntFanIOErrFileToFanTempFile(const _TCHAR *pcInName, _TCHAR *pcLangName)
{
    int iRet = RET_NG;
    int iFuncRet;
    unsigned int uiPathLen;
    _TCHAR *pcDirName;
    _TCHAR *pcFileName;
    FILE *fpout = NULL;
    pcDirName = NULL;
    pcFileName = NULL;
    if ((fpout = _tfopen(pcTempFio, _T("a+, ccs=UTF-8"))) == NULL) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000389, pcTempFio); //* MSG_LOG_ID_C_00000389
        goto FUNC_END;
    }
    if (pcInName == NULL) {
        goto FUNC_END;
    }
    uiPathLen = _tcslen(pcInName);
    iFuncRet = CDMtrLibUtilMallocW((void **)&pcDirName, (uiPathLen + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibFileGetDirName(pcInName, pcDirName);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibUtilMallocW((void **)&pcFileName, (uiPathLen + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibFileGetFileName(pcInName, pcFileName);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (pcLangName != NULL) {
        iFuncRet = _ftprintf(fpout, OUTPUT_ERROR_FORMAT,
                             pcDirName,
                             pcFileName,
                             OUTPUT_NOT_TARGET,
                             pcLangName
                            );
    } else {
        iFuncRet = _ftprintf(fpout, OUTPUT_ERROR_FORMAT,
                             pcDirName,
                             pcFileName,
                             OUTPUT_NOT_TARGET,
                             OUTPUT_NOT_TARGET
                            );
    }
    iRet = RET_OK;
FUNC_END:
    if (fpout) {
        fclose(fpout);
        fpout = NULL;
    }
    CDMtrLibUtilFreeW((void **)&pcDirName);
    CDMtrLibUtilFreeW((void **)&pcFileName);
    return iRet;
}
