/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "CDMtrCmn.h"
#include "CDMtrLibCmnW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrCntFirstLine.h"

#define FIRSTLINE_MAX_BUFF 1024
#define FIRSTLINE_MAX_STRING 127
#define IS_CONTROL_CHAR(p) (((*p) <= 0x1f) || ((*p) == 0x7f))
#define IS_CTRL_ESC(p) (0x1b == (*p))

typedef struct tag_FIRSTLINE_INFO_TREE {
    struct tag_FIRSTLINE_INFO_TREE *next;
    _FIRSTLINE_INFO *ptFirstLineInfo;
}
_FIRSTLINE_INFO_TREE;

static _FIRSTLINE_INFO_TREE stFirstLineTreeFirst;
static _FIRSTLINE_INFO_TREE *sptFirstLineTree = NULL;
static int siFirstLineInitialized = CDMTR_FALSE;
static char stFirstLineBuff[FIRSTLINE_MAX_BUFF];
static int siReadBufSize = 0;
static int siOpenedByBinaryMode = 0;
static int cdmtIsCheckBOM(const unsigned char *p);
static void cdmtrReplaceBreakLineToNULL();

/*!
    @brief  擪sc[̏s
*/
void CDMtrCntFirstLineInit(void)
{
    stFirstLineTreeFirst.next = NULL;
    stFirstLineTreeFirst.ptFirstLineInfo = NULL;
    sptFirstLineTree = &stFirstLineTreeFirst;
    return;
}

/*!
    @brief  擪sc[̗̈s
*/
void CDMtrCntFirstLineFreeInfo(void)
{
    for (sptFirstLineTree = stFirstLineTreeFirst.next; sptFirstLineTree != NULL; sptFirstLineTree = stFirstLineTreeFirst.next) {
        stFirstLineTreeFirst.next = sptFirstLineTree->next;
        CDMtrLibCmnFree((void **) & (sptFirstLineTree->ptFirstLineInfo->pcFirstLineID));
        CDMtrLibCmnFree((void **) & (sptFirstLineTree->ptFirstLineInfo->pcFirstLineStr));
        CDMtrLibCmnFree((void **) & (sptFirstLineTree->ptFirstLineInfo));
        CDMtrLibCmnFree((void **)&sptFirstLineTree);
    }
    return;
}


/*!
    @brief      擪s([FIRSTLINE_STR])̑̊mFs
*/
int CDMtrCntFirstLineIsInvalidID(const _TCHAR *pcName)
{
    int iRet;
    _TCHAR *p;
    iRet = RET_TRUE;
    p = (_TCHAR *)pcName;
    if (*p != _T('$')) {
        goto FUNC_END;
    }
    p++;
    if (_T('1') <= *p && *p <= _T('9')) {
        p++;
        if (_T('0') <= *p && *p <= _T('9')) {
            p++;
        }
        if (IS_NULL_CHAR(p)) {
            iRet = RET_FALSE;
        }
    }
FUNC_END:
    return iRet;
}


/*!
    @brief      擪s̓o^s
*/
int CDMtrCntFirstLineRegInfo(const _TCHAR *pcAttr , const _TCHAR *pcValue)
{
    int iRet;
    unsigned int uiLen;
    iRet = RET_NG;
    sptFirstLineTree->next = (_FIRSTLINE_INFO_TREE *)CDMtrLibCmnMalloc(sizeof(_FIRSTLINE_INFO_TREE));
    if (sptFirstLineTree->next == NULL) {
        goto FUNC_END;
    }
    sptFirstLineTree = sptFirstLineTree->next;
    sptFirstLineTree->ptFirstLineInfo = (_FIRSTLINE_INFO *)CDMtrLibCmnMalloc(sizeof(_FIRSTLINE_INFO));
    if (sptFirstLineTree->ptFirstLineInfo == NULL) {
        goto FUNC_END;
    }
    uiLen = _tcslen(pcAttr);
    sptFirstLineTree->ptFirstLineInfo->pcFirstLineID = (_TCHAR *)CDMtrLibCmnMalloc(uiLen + 1);
    if (sptFirstLineTree->ptFirstLineInfo->pcFirstLineID == NULL) {
        goto FUNC_END;
    }
    _tcsncpy(sptFirstLineTree->ptFirstLineInfo->pcFirstLineID, pcAttr, uiLen);
    uiLen = _tcslen(pcValue);
    if (uiLen > FIRSTLINE_MAX_STRING) {
        uiLen = FIRSTLINE_MAX_STRING;
    }
    sptFirstLineTree->ptFirstLineInfo->pcFirstLineStr = (_TCHAR *)CDMtrLibCmnMalloc(uiLen + 1);
    if (sptFirstLineTree->ptFirstLineInfo->pcFirstLineStr == NULL) {
        goto FUNC_END;
    }
    _tcsncpy(sptFirstLineTree->ptFirstLineInfo->pcFirstLineStr, pcValue, uiLen);
    iRet = RET_OK;
FUNC_END:
    return iRet;
}


/*!
    @brief      t@C擪s̓ǂݍ
*/
int CDMtrCntFirstLineStart(const _TCHAR *pcFilePath, int iBinMode)
{
    int iRet = RET_NG;
    FILE *file = NULL;
    if (siFirstLineInitialized) {
        if (!iBinMode && siOpenedByBinaryMode) {
            cdmtrReplaceBreakLineToNULL();
        }
        iRet = RET_OK;
        goto FUNC_END;
    }
    if ( (file = _tfopen(pcFilePath, _T("rb")) ) != NULL ) {
        memset(stFirstLineBuff, 0x00, (FIRSTLINE_MAX_BUFF) * sizeof(char));
        siReadBufSize = fread((void *)stFirstLineBuff, sizeof(char),
                              FIRSTLINE_MAX_BUFF, file);
        if (ferror(file)) {
            siReadBufSize = 0;
            goto FUNC_END;
        }
    } else {
        goto FUNC_END;
    }
    siFirstLineInitialized = CDMTR_TRUE;
    siOpenedByBinaryMode = CDMTR_TRUE;
    iRet = RET_OK;
FUNC_END:
    if (file != NULL) {
        fclose(file);
    }
    return iRet;
}


/*!
    @brief  t@C擪s̏
*/
void CDMtrCntFirstLineClear()
{
    siFirstLineInitialized = CDMTR_FALSE;
    siReadBufSize = 0;
}


/*!
    @brief      擪s̓KmFs
*/
int CDMtrCntFirstLineCheck(const _TCHAR *pcFirstLineID)
{
    int iRet;
    _FIRSTLINE_INFO_TREE *ptTree;
    _TCHAR *pcFirstLine = NULL;
    int iFuncRet = 0;
    _TCHAR *wTmpBuffer = NULL;
    char *p = NULL;
    iRet = RET_FALSE;
    if (pcFirstLineID == NULL) {
        goto FUNC_END;
    }
    if (!siFirstLineInitialized) {
        goto FUNC_END;
    } else if (siOpenedByBinaryMode) {
        cdmtrReplaceBreakLineToNULL();
    }
    ptTree = stFirstLineTreeFirst.next;
    while (ptTree != NULL) {
        if (_tcscmp(pcFirstLineID, ptTree->ptFirstLineInfo->pcFirstLineID) == 0) {
            pcFirstLine = ptTree->ptFirstLineInfo->pcFirstLineStr;
            break;
        }
        ptTree = ptTree->next;
    }
    if (pcFirstLine != NULL) {
        p = stFirstLineBuff;
        iFuncRet = CDMtrLibUtilMBCStoUNICODEW(&wTmpBuffer, &p, ENCODING_UTF_8);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        if (CDMtrLibUtilStrCaseStr(wTmpBuffer, pcFirstLine) != NULL) {
            iRet = RET_TRUE;
        }
    }
FUNC_END:
    CDMtrLibUtilFreeW((void **)&wTmpBuffer);
    return iRet;
}

/*!
    @brief  oCit@C`FbN
*/
int CDMtrCntFirstLineCheckBinary()
{
    int iRet;
    int iBinCharNum;
    int iMaxBinCharNum;
    int i;
    unsigned char *p;
    iRet = RET_FALSE;
    iBinCharNum = 0;
    iMaxBinCharNum = siReadBufSize / 10;
    if (!siFirstLineInitialized || !siOpenedByBinaryMode) {
        goto FUNC_END;
    }
    p = (unsigned char *)stFirstLineBuff;
    if (siReadBufSize == 0) {
        goto FUNC_END;
    }
    if (cdmtIsCheckBOM(p)) {
        goto FUNC_END;
    }
    for (i = 0; i < siReadBufSize; i++, p++) {
        if (IS_NULL_CHAR(p)) {
            iRet = RET_TRUE;
            break;
        } else if (IS_BREAK_LINE(p) || IS_CTRL_TAB(p) || IS_CTRL_ESC(p)) {
        } else if (IS_CONTROL_CHAR(p)) {
            iBinCharNum++;
            if (iBinCharNum >= iMaxBinCharNum) {
                iRet = RET_TRUE;
                break;
            }
        } else {
        }
    }
FUNC_END:
    return iRet;
}


/*!
    @brief      BOM̗L`FbNs
*/
static int cdmtIsCheckBOM ( const unsigned char *p )
{
    int iRet = RET_FALSE;
    unsigned short bom = 0;
    bom = (*p << 8) | *(p + 1);
    switch (bom) {
    case 0xFEFF:
    case 0xFFFE:
        iRet = RET_TRUE;
        break;
    default:
        if (siReadBufSize >= 3) {
            if (p[0] == 0xEF && p[1] == 0xBB && p[2] == 0xBF) {
                iRet = RET_TRUE;
            }
        }
        break;
    }
    return iRet;
}


/*!
    @brief  t@C擪sobt@NULLI[t
*/
static void cdmtrReplaceBreakLineToNULL()
{
    int i;
    unsigned char *p;
    if (!siFirstLineInitialized || !siOpenedByBinaryMode) {
        return ;
    }
    p = (unsigned char *)stFirstLineBuff;
    for (i = 0; i < siReadBufSize; i++, p++) {
        if (IS_BREAK_LINE(p)) {
            *p = _T('\0');
            break;
        }
    }
    stFirstLineBuff[FIRSTLINE_MAX_BUFF - 1] = _T('\0');
    siOpenedByBinaryMode = CDMTR_FALSE;
    return ;
}
