/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "CDMtrCmn.h"
#include "CDMtrExtCmn.h"
#include "CDMtrLibCmnW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrLibFileW.h"
#include "CDMtrCntLang.h"
#include "CDMtrCntFirstLine.h"
#include "CDMtrMessage.h"

#include <tchar.h>

#define COMMA _T(",")

#define SEP_FIRSTLINE_ID_IN_LANG _T(";")

typedef struct tag_LANG_INFO_TREE {
    struct tag_LANG_INFO_TREE *next;
    _LANG_INFO *ptLangInfo;
}
_LANG_INFO_TREE;

typedef struct tag_LANG_DEFAULT_TABLE {
    int iLangType;
    _TCHAR *pcLangName;
    _TCHAR *pcFileNames;
} _LANG_DEFAULT_TABLE;


static _LANG_DEFAULT_TABLE stLangDefaultTable[] = {
    {LANG_TYPE_CPP, _T("CPP"), _T("*.cpp")},
    {LANG_TYPE_CPPH, _T("CPPH"), _T("*.h")},
    {LANG_TYPE_C, _T("C"), _T("*.c")},
    {LANG_TYPE_CH, _T("CH"), _T("*.h")},
    {LANG_TYPE_JAVA, _T("JAVA"), _T("*.java")},
    {LANG_TYPE_CLASS, _T("CLASS"), _T("*.class")},
    {LANG_TYPE_JAR, _T("JAR"), _T("*.jar")},
    {LANG_TYPE_CS, _T("CS"), _T("*.cs")},
    {LANG_TYPE_VB, _T("VB"), _T("*.vb")},
    {LANG_TYPE_JS, _T("JS"), _T("*.js")},
    {LANG_TYPE_TXT, _T("TXT"), _T("")},
};


static _LANG_INFO_TREE stLangTreeFirst;

static _LANG_INFO_TREE *sptLangTree = NULL;


static int cdmtrCntLangMatchTargetFiles(const _TCHAR *pcFileName, _LANG_VALUE_TREE *ptTargetFiles);
static int cdmtrCntLangMatchTargetFirstLines(const _TCHAR *pcFileName, _LANG_VALUE_TREE *ptTargetFiles);
static int cdmtrCntLangMatchTargetBinary(const _TCHAR *pcFileName);
static void cdmtrCntLangValueFree(_LANG_VALUE_TREE **ptTargetTree) ;
static int CDMtrCntLangRegSplitTargets(_TCHAR *pcTargetStart, _TCHAR *pcTargetEnd, _LANG_VALUE_TREE **ptTargetTree);
static _TCHAR *cdmtrCntLangSplit(_TCHAR *pcString, _TCHAR *pcTargetEnd, _TCHAR **pcRetTop, _TCHAR **pcRetEnd);


/*!
    @brief  ʃc[̏s
*/
void CDMtrCntLangInitInfo(void)
{
    stLangTreeFirst.next = NULL;
    stLangTreeFirst.ptLangInfo = NULL;
    sptLangTree = &stLangTreeFirst;
    return;
}


/*!
    @brief      ftHgʂ̓o^s
*/
int CDMtrCntLangRegDefault(void)
{
    int iRet;
    int i;
    iRet = RET_OK;
    for (i = 0; i < (sizeof(stLangDefaultTable) / sizeof(_LANG_DEFAULT_TABLE)); i++) {
        iRet = CDMtrCntLangRegInfo(stLangDefaultTable[i].iLangType,
                                   stLangDefaultTable[i].pcLangName,
                                   stLangDefaultTable[i].pcFileNames);
        if (iRet != RET_OK) {
            break;
        }
    }
    return iRet;
}


/*!
    @brief          ʂ̓o^s
*/
int CDMtrCntLangRegInfo(int iLangType, const _TCHAR *pcLangName, _TCHAR *pcFileNames)
{
    int iRet, iFuncRet;
    unsigned int uiLen;
    _TCHAR *pcSemi;
    iRet = RET_NG;
    sptLangTree->next = (_LANG_INFO_TREE *)CDMtrLibCmnMalloc(sizeof(_LANG_INFO_TREE));
    if (sptLangTree->next == NULL) {
        goto FUNC_END;
    }
    sptLangTree = sptLangTree->next;
    sptLangTree->ptLangInfo = (_LANG_INFO *)CDMtrLibCmnMalloc(sizeof(_LANG_INFO));
    if (sptLangTree->ptLangInfo == NULL) {
        goto FUNC_END;
    }
    sptLangTree->ptLangInfo->iLangType = iLangType;
    uiLen = _tcslen(pcLangName);
    sptLangTree->ptLangInfo->pcLangName = (_TCHAR *)CDMtrLibCmnMalloc(uiLen + 1);
    if (sptLangTree->ptLangInfo->pcLangName == NULL) {
        goto FUNC_END;
    }
    _tcsncpy(sptLangTree->ptLangInfo->pcLangName, pcLangName, uiLen);
    pcSemi = _tcsstr(pcFileNames, SEP_FIRSTLINE_ID_IN_LANG);
    if (pcSemi == NULL) {
        iFuncRet = CDMtrCntLangRegSplitTargets(pcFileNames, pcFileNames + _tcslen(pcFileNames),
                                               &(sptLangTree->ptLangInfo->ptFileNames));
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    } else {
        iFuncRet = CDMtrCntLangRegSplitTargets(pcFileNames, pcSemi,
                                               &(sptLangTree->ptLangInfo->ptFileNames));
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        iFuncRet = CDMtrCntLangRegSplitTargets(pcSemi + 1, pcFileNames + _tcslen(pcFileNames),
                                               &(sptLangTree->ptLangInfo->ptFirstLineID));
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}


/*!
    @brief          o^ς݌ʂmFs (ݖgp)
*/
int CDMtrCntLangExistRegName(const _TCHAR *pcLangName)
{
    int iRet;
    _LANG_INFO_TREE *ptTree;
    _TCHAR *pcLowName;
    _TCHAR *pcRegName;
    pcLowName = NULL;
    pcRegName = NULL;
    iRet = RET_FALSE;
    if (pcLangName == NULL) {
        goto FUNC_END;
    }
    pcLowName = CDMtrLibUtilCopyString(pcLangName);
    if (pcLowName == NULL) {
        goto FUNC_END;
    }
    CDMtrLibUtilSetLowString(pcLowName);
    ptTree = stLangTreeFirst.next;
    while (ptTree != NULL) {
        pcRegName = CDMtrLibUtilCopyString(ptTree->ptLangInfo->pcLangName);
        if (pcRegName == NULL) {
            break;
        }
        CDMtrLibUtilSetLowString(pcRegName);
        if (_tcscmp(pcRegName, pcLowName) == 0) {
            iRet = RET_TRUE;
            break;
        }
        ptTree = ptTree->next;
        CDMtrLibCmnFree((void **)&pcRegName);
    }
FUNC_END:
    CDMtrLibCmnFree((void **)&pcLowName);
    CDMtrLibCmnFree((void **)&pcRegName);
    return iRet;
}


/*!
    @brief      ʃc[̗̈s
*/
void CDMtrCntLangFreeInfo(void)
{
    for (sptLangTree = stLangTreeFirst.next; sptLangTree != NULL; sptLangTree = stLangTreeFirst.next) {
        stLangTreeFirst.next = sptLangTree->next;
        CDMtrLibCmnFree((void **) & (sptLangTree->ptLangInfo->pcLangName));
        cdmtrCntLangValueFree(&(sptLangTree->ptLangInfo->ptFileNames));
        cdmtrCntLangValueFree(&(sptLangTree->ptLangInfo->ptFirstLineID));
        CDMtrLibCmnFree((void **) & (sptLangTree->ptLangInfo));
        CDMtrLibCmnFree((void **) & (sptLangTree));
    }
    return;
}


/*!
    @brief          ʖ猾ʂ擾
*/
int CDMtrCntLangGetTypeFromLangName(const _TCHAR *pcLangName)
{
    int iLangType;
    int i;
    _TCHAR *pcStr;
    pcStr = NULL;
    iLangType = LANG_TYPE_ETC;
    if (pcLangName == NULL) {
        goto FUNC_END;
    }
    pcStr = CDMtrLibUtilCopyString(pcLangName);
    if (pcStr == NULL) {
        goto FUNC_END;
    }
    CDMtrLibUtilSetUpperString(pcStr);
    for (i = 0; i < (sizeof(stLangDefaultTable) / sizeof(_LANG_DEFAULT_TABLE)); i++) {
        if (_tcscmp(pcStr, stLangDefaultTable[i].pcLangName) == 0) {
            iLangType = stLangDefaultTable[i].iLangType;
            break;
        }
    }
FUNC_END:
    if (pcStr != NULL) {
        free(pcStr);
    }
    return iLangType;
}


/*!
    @brief          t@C猾ʂ擾
*/
int CDMtrCntLangGetTypeFromFileName(int iExceptBinaryMode, const _TCHAR *pcFileName, _TCHAR **ppcLangName)
{
    int iFuncRet;
    int iLangType;
    int iMatch;
    _LANG_INFO_TREE *ptTree;
    _TCHAR *pcFName = NULL;
    int uiLenName;
    int iBinCheckedFlg = CDMTR_FALSE;
    iLangType = LANG_TYPE_UNKNOW;
    if (pcFileName == NULL || ppcLangName == NULL) {
        goto FUNC_END;
    }
    *ppcLangName = NULL;
    uiLenName = _tcslen(pcFileName);
    pcFName = (_TCHAR *)CDMtrLibCmnMalloc(uiLenName + 1);
    if (pcFName == NULL) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibFileGetFileName(pcFileName, pcFName);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000064); //* MSG_LOG_ID_C_00000064
    }
    ptTree = stLangTreeFirst.next;
    for (; ptTree != NULL; ptTree = ptTree->next) {
        iMatch = cdmtrCntLangMatchTargetFiles(pcFName, ptTree->ptLangInfo->ptFileNames);
        if (!iMatch) {
            continue;
        }
        if (ptTree->ptLangInfo->iLangType == LANG_TYPE_ETC) {
            if (iExceptBinaryMode && !iBinCheckedFlg) {
                iMatch = cdmtrCntLangMatchTargetBinary(pcFileName);
                if (iMatch) {
                    iLangType = LANG_TYPE_BIN;
                    break;
                }
                iBinCheckedFlg = CDMTR_TRUE;
            }
        }
        if (ptTree->ptLangInfo->ptFirstLineID != NULL) {
            iMatch = cdmtrCntLangMatchTargetFirstLines(pcFileName,
                     ptTree->ptLangInfo->ptFirstLineID);
            if (!iMatch) {
                continue;
            }
        }
        *ppcLangName = CDMtrLibUtilCopyString(ptTree->ptLangInfo->pcLangName);
        if (*ppcLangName == NULL) {
            goto FUNC_END;
        }
        iLangType = ptTree->ptLangInfo->iLangType;
        break;
    }
    if (iDebug == DEBUG_ON) {
        if (ptTree == NULL) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000065); //* MSG_LOG_ID_C_00000065
        } else {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000066, *ppcLangName); //* MSG_LOG_ID_C_00000066
        }
    }
FUNC_END:
    CDMtrLibCmnFree((void **)&pcFName);
    CDMtrCntFirstLineClear();
    return iLangType;
}


/*!
    @brief          ʂɑ΂ΏۏɈv邩mFs
*/
static int cdmtrCntLangMatchTargetFiles(const _TCHAR *pcFileName, _LANG_VALUE_TREE *ptTargetFiles)
{
    int iRet;
    _LANG_VALUE_TREE *ptTree;
    iRet = RET_FALSE;
    ptTree = ptTargetFiles;
    while (ptTree != NULL) {
        if (CDMtrLibUtilMatchPattern(pcFileName, ptTree->pcValue)) {
            iRet = RET_TRUE;
            break;
        }
        ptTree = ptTree->next;
    }
    return iRet;
}

/*!
    @brief          擪sɈv邩mFs
*/
static int cdmtrCntLangMatchTargetFirstLines(const _TCHAR *pcFileName, _LANG_VALUE_TREE *ptTargetFiles)
{
    int iRet;
    int iFuncRet;
    _LANG_VALUE_TREE *ptTree;
    iRet = RET_FALSE;
    ptTree = ptTargetFiles;
    iFuncRet = CDMtrCntFirstLineStart(pcFileName, CDMTR_FALSE);
    if (iFuncRet != RET_OK) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000067, CDMtrLibFileRemoveLongPathMarkW(pcFileName)); //* MSG_LOG_ID_C_00000067
        goto FUNC_END;
    }
    while (ptTree != NULL) {
        if (CDMtrCntFirstLineCheck(ptTree->pcValue)) {
            iRet = RET_TRUE;
            break;
        }
        ptTree = ptTree->next;
    }
FUNC_END:
    return iRet;
}

/*!
    @brief          oCit@C`FbN
*/
static int cdmtrCntLangMatchTargetBinary(const _TCHAR *pcFileName)
{
    int iRet;
    int iFuncRet;
    iRet = RET_FALSE;
    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000069); //* MSG_LOG_ID_C_00000069
    }
    iFuncRet = CDMtrCntFirstLineStart(pcFileName, CDMTR_TRUE);
    if (iFuncRet != RET_OK) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000070, CDMtrLibFileRemoveLongPathMarkW(pcFileName)); //* MSG_LOG_ID_C_00000070
        goto FUNC_END;
    }
    iFuncRet = CDMtrCntFirstLineCheckBinary();
    if (iFuncRet) {
        iRet = RET_TRUE;
        if (iDebug == DEBUG_ON) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000072); //* MSG_LOG_ID_C_00000072
        }
    }
FUNC_END:
    return iRet;
}


/*!
    @brief      ʂɑ΂Ώۏ̗̈s
*/
static void cdmtrCntLangValueFree(_LANG_VALUE_TREE **ptTargetTree)
{
    _LANG_VALUE_TREE *ptTree, *ptTreeNext;
    ptTree = *ptTargetTree;
    while (ptTree != NULL) {
        ptTreeNext = ptTree->next;
        CDMtrLibCmnFree((void **) & (ptTree->pcValue));
        CDMtrLibCmnFree((void **) & (ptTree));
        ptTree = ptTreeNext;
    }
    *ptTargetTree = NULL;
    return;
}



/*!
    @brief          Ώۏ̐ݒs
*/
static int CDMtrCntLangRegSplitTargets(_TCHAR *pcTargetStart, _TCHAR *pcTargetEnd, _LANG_VALUE_TREE **ptTargetTree)
{
    int iRet;
    _TCHAR *pcStrTop, *pcStrEnd, *pcPos;
    unsigned int uiLen;
    _LANG_VALUE_TREE *ptTreeFirst, *ptTree;
    iRet = RET_NG;
    ptTreeFirst = *ptTargetTree;
    ptTree = ptTreeFirst;
    pcPos = (_TCHAR *)pcTargetStart;
    while (pcTargetEnd > pcPos) {
        pcPos = cdmtrCntLangSplit(pcPos, pcTargetEnd, &pcStrTop, &pcStrEnd);
        uiLen = pcStrEnd - pcStrTop;
        if (uiLen > 0) {
            if (ptTreeFirst == NULL) {
                ptTreeFirst = (_LANG_VALUE_TREE *)CDMtrLibCmnMalloc(sizeof(_LANG_VALUE_TREE));
                if (ptTreeFirst == NULL) {
                    goto FUNC_END;
                }
                ptTree = ptTreeFirst;
            } else {
                while (ptTree->next != NULL) {
                    ptTree = ptTree->next;
                }
                ptTree->next = (_LANG_VALUE_TREE *)CDMtrLibCmnMalloc(sizeof(_LANG_VALUE_TREE));
                if (ptTree->next == NULL) {
                    goto FUNC_END;
                }
                ptTree = ptTree->next;
            }
            ptTree->pcValue = (_TCHAR *)CDMtrLibCmnMalloc(uiLen + 1);
            if (ptTree->pcValue == NULL) {
                goto FUNC_END;
            }
            _tcsncpy(ptTree->pcValue, pcStrTop, uiLen);
        }
        pcPos++;
    }
    iRet = RET_OK;
    *ptTargetTree = ptTreeFirst;
FUNC_END:
    if (iRet != RET_OK) {
        cdmtrCntLangValueFree(&ptTreeFirst);
    }
    return iRet;
}


/*!
    @brief          J}܂͕񖖔̃|C^擾
*/
static _TCHAR *cdmtrCntLangSplit(_TCHAR *pcString, _TCHAR *pcTargetEnd, _TCHAR **pcRetTop, _TCHAR **pcRetEnd)
{
    _TCHAR *pcTop, *pcEnd, *pcPos;
    pcPos = pcString;
    SKIP_WHITE_SPACE(&pcPos);
    pcTop = pcPos;
    while (pcTargetEnd > pcPos) {
        if (*pcPos == _T(',')) {
            break;
        }
        pcPos++;
    }
    pcEnd = pcPos;
    SKIPBACK_WHITE_SPACE(pcTop, &pcEnd);
    if (pcTop == pcEnd) {
    } else {
    }
    *pcRetTop = pcTop;
    *pcRetEnd = pcEnd;
    return pcPos;
}
