/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "CDMtrExtCmn.h"
#include "CDMtrExtMain.h"
#include "CDMtrExtProp.h"

#include "CDMtrLibCmnW.h"
#include "CDMtrLibFileW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrLibPropW.h"

#define PROP_EXT_SEP ','
#define PROP_COMMENT_DELIM ' '

#define DEFAULT_LANG_NAME   _T("c,cpp,ch,cpph,java,cs")
#define DEFAULT_VB_NAME     _T("vb")
#define DEFAULT_TXT_NAME    _T("txt")

#define PROP_GROUP_COMMENT _T("comment")

typedef struct tag_LANG_TYPE {
    int iLangType;
    _TCHAR *pcLangName;
} _LANG_TYPE;


static _LANG_TYPE stLangTable[] = {
    {LANG_TYPE_CPP, _T("cpp")},
    {LANG_TYPE_CPPH, _T("cpph")},
    {LANG_TYPE_C, _T("c")},
    {LANG_TYPE_CH, _T("ch")},
    {LANG_TYPE_JAVA, _T("java")},
    {LANG_TYPE_CS, _T("cs")},
    {LANG_TYPE_VB, _T("vb")},
    {LANG_TYPE_TXT, _T("txt")},
    {LANG_TYPE_BIN, _T("bin")}
};

static int cdmtrExtPropIsTargetExt(const _TCHAR *pcLangType, const _TCHAR *pcExt);
static int cdmtrExtPropSetPropValComment(const _TCHAR *pcVal, _COMMENT *ptComment);


/*!
    @brief          ʂftHǧʂmF
*/
int CDMtrExtPropIsDefaultType(const _TCHAR *pcLangType)
{
    int iRet;
    iRet = cdmtrExtPropIsTargetExt(DEFAULT_LANG_NAME, pcLangType);
    return iRet;
}

/*!
    @brief          ʂVBʂmF
*/
int CDMtrExtPropIsVBType(const _TCHAR *pcLangType)
{
    int iRet;
    iRet = cdmtrExtPropIsTargetExt(DEFAULT_VB_NAME, pcLangType);
    return iRet;
}


/*!
    @brief          ʂeLXǧʂmF
*/
int CDMtrExtPropIsTxtType(const _TCHAR *pcLangType)
{
    int iRet;
    iRet = cdmtrExtPropIsTargetExt(DEFAULT_TXT_NAME, pcLangType);
    return iRet;
}


/*!
    @brief          ݒt@CAʖ̃Rg擾
*/
int CDMtrExtPropGetPropComment(const _TCHAR *pcFileName, const _TCHAR *pcLangName, _COMMENT *ptComment)
{
    int iRet;
    int iFuncRet;
    int i;
    _PropInfoList *ptPropList;
    _TCHAR *pcLowName;
    _TCHAR *pcAttrName;
    iRet = RET_NG;
    pcLowName = NULL;
    pcAttrName = NULL;
    ptPropList = NULL;
    if (pcLangName == NULL || ptComment == NULL) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibPropGetList(pcFileName, PROP_GROUP_COMMENT, &ptPropList);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    pcLowName = CDMtrLibUtilCopyString(pcLangName);
    if (pcLowName == NULL) {
        goto FUNC_END;
    }
    CDMtrLibUtilSetLowString(pcLowName);
    for (i = 0; i < ptPropList->iNum; i++) {
        pcAttrName = CDMtrLibUtilCopyString(ptPropList->ptPropInfo[i]->pcAttr);
        if (pcAttrName == NULL) {
            break;
        }
        CDMtrLibUtilSetLowString(pcAttrName);
        if (_tcscmp(pcAttrName, pcLowName) == 0) {
            iFuncRet = cdmtrExtPropSetPropValComment(ptPropList->ptPropInfo[i]->pcVal, ptComment);
            if (iFuncRet != RET_OK) {
                break;
            }
        }
        CDMtrLibCmnFree((void **)&pcAttrName);
    }
    iRet = RET_OK;
FUNC_END:
    CDMtrLibPropFreeList(&ptPropList);
    CDMtrLibCmnFree((void **)&pcLowName);
    CDMtrLibCmnFree((void **)&pcAttrName);
    return iRet;
}



/*!
    @brief      ʎ擾
*/
int CDMtrExtPropGetType(const _TCHAR *pcLangName)
{
    int iLangType;
    int i;
    unsigned int uiLen;
    _TCHAR *pcStr;
    _TCHAR *p;
    pcStr = NULL;
    iLangType = LANG_TYPE_ETC;
    if (pcLangName == NULL) {
        goto FUNC_END;
    }
    uiLen = _tcslen(pcLangName);
    pcStr = (_TCHAR *)malloc((uiLen + 1) * sizeof(_TCHAR));
    if (pcStr == NULL) {
        goto FUNC_END;
    }
    _tcsncpy(pcStr, pcLangName, uiLen);
    pcStr[uiLen] = _T('\0');
    p = pcStr;
    while (*p != _T('\0')) {
        *p = towlower(*p);
        p++;
    }
    for (i = 0; i < (sizeof(stLangTable) / sizeof(_LANG_TYPE)); i++) {
        if (_tcscmp(pcStr, stLangTable[i].pcLangName) == 0) {
            iLangType = stLangTable[i].iLangType;
            break;
        }
    }
FUNC_END:
    if (pcStr != NULL) {
        free(pcStr);
    }
    return iLangType;
}

/*!
    @brief          t@C̊gqgqXg̊gqmF
*/
static int cdmtrExtPropIsTargetExt(const _TCHAR *pcLangList, const _TCHAR *pcLangName)
{
    int iRet, iFuncRet;
    _TCHAR *p, *startp, *endp;
    unsigned int uiTargetLen, uiLangLen;
    _TCHAR *pcStr;
    iRet = RET_FALSE;
    pcStr = NULL;
    uiLangLen = _tcslen(pcLangName);
    pcStr = (_TCHAR *)malloc((uiLangLen + 1) * sizeof(_TCHAR));
    if (pcStr == NULL) {
        goto FUNC_END;
    }
    _tcsncpy(pcStr, pcLangName, uiLangLen);
    pcStr[uiLangLen] = '\0';
    p = pcStr;
    while (*p != '\0') {
        *p = towlower(*p);
        p++;
    }
    p = (_TCHAR *)pcLangList;
    while (!IS_NULL_CHAR(p)) {
        if (IS_WHITE_SPACE(p)) {
            p++;
            continue;
        }
        iFuncRet = CDMtrLibUtilGetStringNextDelimSP(&p, &startp, &endp, PROP_EXT_SEP);
        if (iFuncRet != RET_OK) {
            if (_tcscmp(p, pcStr) == 0) {
                iRet = RET_TRUE;
            }
            break;
        }
        uiTargetLen = endp - startp;
        if (uiLangLen == uiTargetLen) {
            if (_tcsncmp(startp, pcStr, uiTargetLen) == 0) {
                iRet = RET_TRUE;
                break;
            }
        }
        p++;
    }
FUNC_END:
    if (pcStr != NULL) {
        free(pcStr);
    }
    return iRet;
}


/*!
    @brief          w肳ꂽRg̗̈ɒlݒ肷
*/
static int cdmtrExtPropSetPropValComment(const _TCHAR *pcVal, _COMMENT *ptComment)
{
    int iRet;
    unsigned int uiValLen;
    _TCHAR *p;
    _TCHAR *pCmntS_top, *pCmntS_end;
    _TCHAR *pCmntE_top, *pCmntE_end;
    int i;
    int iType;
    iRet = RET_NG;
    if (_tcslen(pcVal) == 0) {
        iRet = RET_OK;
        goto FUNC_END;
    }
    iType = COMMENT_TYPE_NOSET;
    for (i = 0; i < CMNT_NUM; i++) {
        if (ptComment->tCmnt[i].iType != COMMENT_TYPE_NOSET) {
            continue;
        } else {
            iType = COMMENT_TYPE_LINE;
            break;
        }
    }
    if (iType == COMMENT_TYPE_NOSET) {
        goto FUNC_END;
    }
    p = (_TCHAR *)pcVal;
    pCmntS_top = p;
    pCmntS_end = p + _tcslen(p);
    pCmntE_top = NULL;
    pCmntE_end = NULL;
    while (!IS_NULL_CHAR(p)) {
        if (*p == PROP_COMMENT_DELIM) {
            iType = COMMENT_TYPE_BLOCK;
            pCmntS_end = p - 1;
            SKIP_WHITE_SPACE(&p);
            pCmntE_top = p;
            pCmntE_end = (_TCHAR *)pcVal + _tcslen(pcVal);
            break;
        }
        p++;
    }
    switch (iType) {
    case COMMENT_TYPE_BLOCK:
        uiValLen = pCmntE_end - pCmntE_top + 1;
        if (uiValLen > CMNT_SIZE) {
            uiValLen = CMNT_SIZE;
        }
        _tcsncpy(ptComment->tCmnt[i].cCmntE, pCmntE_top, uiValLen);
        ptComment->tCmnt[i].cCmntE[uiValLen] = '\0';
    case COMMENT_TYPE_LINE:
        uiValLen = pCmntS_end - pCmntS_top + 1;
        if (uiValLen > CMNT_SIZE) {
            uiValLen = CMNT_SIZE;
        }
        _tcsncpy(ptComment->tCmnt[i].cCmntS, pCmntS_top, uiValLen);
        ptComment->tCmnt[i].cCmntS[uiValLen] = '\0';
        break;
    default:
        goto FUNC_END;
        break;
    }
    ptComment->tCmnt[i].iType = iType;
    ptComment->uiNum++;
    iRet = RET_OK;
FUNC_END:
    return iRet;
}
