/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Windows.h>
#include <shlwapi.h>
#include <mbstring.h>

#include "CDMtrExtCmn.h"
#include "CDMtrLibCmn.h"
#include "CDMtrLibFile.h"
#include "CDMtrMessage.h"

#define FILE_TARGET_TYPE "*.*"


#define FILE_SEPALATE '\\'

#define FILE_SEPALATE_EXT '.'
typedef struct tag_FileTree {
    struct tag_FileTree *next;
    char *fname;
}
_FileTree;

int iShowDetails ;

int iDebug ;

int iNumOfUnrecognized ;


static int cdmtrLibFileIsAbsolutePath(const char *pcpath);
static int cdmtrLibFileGetDir(const char *pcDirpath, int *piFilenum, _FileTree **ppftree);
static char *cdmtrLibFileGetRealPath(const char *pcPath);
static char *cdmtrLibFileAddPathName(char *pcPath, const char *pcAdd);
static char *cdmtrLibFileDelPathName(char *pcPath);
static int cdmtrLibFileDeleteSubDir(const char *pcDirpath);
static char *cdmtrLibFileSearchSepalate(const char *pcStr);

/*!
    @brief          pXgq擾
*/
int CDMtrLibFileGetFileExtension(const char *pcFilePath, char *pcExt)
{
    int iRet, iFuncRet;
    char *p;
    char *pcFileName;
    int iLen;
    iRet = RET_NG;
    pcFileName = NULL;
    if (pcFilePath == NULL || pcExt == NULL) {
        goto FUNC_END;
    }
    memset(pcExt, 0x00, EXT_SIZE + 1);
    iLen = strlen(pcFilePath);
    pcFileName = (char *)malloc(iLen + 1);
    if (pcFileName == NULL) {
        goto FUNC_END;
    }
    memset(pcFileName, 0x00, iLen + 1);
    iFuncRet = CDMtrLibFileGetFileName(pcFilePath, pcFileName);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    p = strrchr(pcFileName, FILE_SEPALATE_EXT);
    if (p != NULL) {
        p++;
        strncpy(pcExt, p, EXT_SIZE);
        iRet = RET_OK;
    }
FUNC_END:
    if (pcFileName != NULL) {
        free(pcFileName);
    }
    return iRet;
}


/*!
    @brief          pXfBNg擾
*/
int CDMtrLibFileGetDirName(const char *pcpath, char *pcdir)
{
    int iRet;
    char *p;
    int iLenDir;
    iRet = RET_NG;
    if (pcpath == NULL || pcdir == NULL) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsDir(pcpath)) {
        iLenDir = strlen(pcpath);
        strncpy(pcdir, pcpath, iLenDir);
        pcdir[iLenDir] = '\0';
    } else {
        p = cdmtrLibFileSearchSepalate(pcpath);
        if (p == NULL) {
            *pcdir = '\0';
        } else if (p == pcpath) {
            *pcdir = FILE_SEPALATE;
            *(pcdir + 1) = '\0';
        } else {
            iLenDir = p - pcpath;
            if (iLenDir > FILE_PATH_SIZE) {
                goto FUNC_END;
            }
            memcpy(pcdir, pcpath, iLenDir);
            pcdir[iLenDir] = '\0';
        }
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}


/*!
    @brief          pXt@C擾
*/
int CDMtrLibFileGetFileName(const char *pcpath, char *pcfile)
{
    int iRet;
    char *p;
    int iLenFile;
    iRet = RET_NG;
    if (pcpath == NULL || pcfile == NULL) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsDir(pcpath)) {
        pcfile = '\0';
    } else {
        p = cdmtrLibFileSearchSepalate(pcpath);
        if (p == NULL) {
            iLenFile = strlen(pcpath);
            p = (char *)pcpath;
        } else {
            iLenFile = strlen(pcpath) - (p - pcpath);
            p++;
        }
        if (iLenFile > FILE_PATH_SIZE) {
            goto FUNC_END;
        }
        memcpy(pcfile, p, iLenFile);
        pcfile[iLenFile] = '\0';
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          fBNgƃt@C
*/
int CDMtrLibFileSetDirFileName(const char *pcDir, const char *pcFile, char **ppcPath)
{
    int iRet;
    unsigned int uiDirPath, uiPathLen;
    char *p;
    char *pcFilePath;
    iRet = RET_NG;
    if (pcDir == NULL || pcFile == NULL || ppcPath == NULL) {
        goto FUNC_END;
    }
    *ppcPath = NULL;
    uiDirPath = strlen(pcDir);
    uiPathLen = uiDirPath;
    uiPathLen++;
    uiPathLen += strlen(pcFile);
    p = (char *)malloc(uiPathLen + 1);
    if (p == NULL) {
        goto FUNC_END;
    }
    memset(p, 0x00, uiPathLen + 1);
    pcFilePath = (char *)pcFile;
    if (*pcFilePath == FILE_SEPALATE) {
        pcFilePath++;
    }
    if (PathCombine(p, pcDir, pcFilePath) == NULL ) {
        goto FUNC_END;
    }
    *ppcPath = p;
    iRet = RET_OK;
FUNC_END:
    return iRet;
}



/*!
    @brief          w肳ꂽpX΃pXɕϊ
*/
int CDMtrLibFileGetAbsolutePath(const char *pcfile, char **ppcpath)
{
    int iRet;
    int iLen, iFileLen;
    char *p;
    char *pcAbsolutePath;
    char *pcRetPath;
    char *pcDrivePath = NULL;
    char *pcinPath;
    iRet = RET_NG;
    p = NULL;
    pcAbsolutePath = NULL;
    pcinPath = NULL;
    if (pcfile == NULL || ppcpath == NULL) {
        goto FUNC_END;
    }
    *ppcpath = NULL;
    iLen = strlen(pcfile);
    pcinPath = (char *)malloc(iLen + 1);
    if (pcinPath == NULL) {
        goto FUNC_END;
    }
    strncpy(pcinPath, pcfile, iLen);
    pcinPath[iLen] = '\0';
    if (iLen >= 3) {
        if (pcinPath[iLen - 1] == ' ') {
            if (pcinPath[0] != '"' && pcinPath[iLen - 2] == '"') {
                pcinPath[iLen - 2] = '\\';
                pcinPath[iLen - 1] = '\0';
            }
        } else if (pcinPath[0] != '"' && pcinPath[iLen - 1] == '"') {
            pcinPath[iLen - 1] = '\\';
        }
    }
    if (cdmtrLibFileIsAbsolutePath(pcinPath)) {
        if (*pcinPath == FILE_SEPALATE) {
            char dir[_MAX_DIR];
            char drive[_MAX_DRIVE];
            char fname[_MAX_FNAME];
            char ext[_MAX_EXT];
            char *p;
            pcDrivePath = (char *)malloc(FILE_PATH_SIZE + 1);
            if (pcDrivePath == NULL) {
                goto FUNC_END;
            }
            memset(pcDrivePath, 0x00, FILE_PATH_SIZE + 1);
            if (GetCurrentDirectory(FILE_PATH_SIZE, pcDrivePath) == 0) {
                goto FUNC_END;
            }
            _splitpath(pcDrivePath, drive, dir, fname, ext);
            iLen = strlen(pcinPath);
            iLen += strlen(drive);
            pcAbsolutePath = (char *)malloc(iLen + 1);
            if (pcAbsolutePath == NULL) {
                goto FUNC_END;
            }
            p = pcAbsolutePath;
            strncpy(p, drive, strlen(drive));
            p += strlen(drive);
            strncpy(p, pcinPath, strlen(pcinPath));
            p[strlen(pcinPath)] = '\0';
        } else {
            iLen = strlen(pcinPath);
            pcAbsolutePath = (char *)malloc(iLen + 1);
            if (pcAbsolutePath == NULL) {
                goto FUNC_END;
            }
            strncpy(pcAbsolutePath, pcinPath, iLen);
            pcAbsolutePath[iLen] = '\0';
        }
    } else {
        pcAbsolutePath = (char *)malloc(FILE_PATH_SIZE + 1);
        if (pcAbsolutePath == NULL) {
            goto FUNC_END;
        }
        memset(pcAbsolutePath, 0x00, FILE_PATH_SIZE + 1);
        if (GetCurrentDirectory(FILE_PATH_SIZE, pcAbsolutePath) == 0) {
            goto FUNC_END;
        }
        iLen = strlen(pcAbsolutePath) + strlen(pcinPath) + 1;
        if (iLen > FILE_PATH_SIZE) {
            goto FUNC_END;
        }
        p = pcAbsolutePath;
        p += strlen(pcAbsolutePath);
        *p = FILE_SEPALATE;
        p++;
        iFileLen = strlen(pcinPath);
        strncpy(p, pcinPath, iFileLen);
        p += iFileLen;
        *p = '\0';
    }
    pcRetPath = cdmtrLibFileGetRealPath(pcAbsolutePath);
    if (pcRetPath == NULL) {
        goto FUNC_END;
    }
    *ppcpath = pcRetPath;
    iRet = RET_OK;
FUNC_END:
    if (pcAbsolutePath != NULL) {
        free(pcAbsolutePath);
    }
    if (pcDrivePath != NULL) {
        free(pcDrivePath);
    }
    if (pcinPath != NULL) {
        free(pcinPath);
    }
    return iRet;
}

/*!
    @brief          ΃pXmF
*/
static int cdmtrLibFileIsAbsolutePath(const char *pcpath)
{
    int iRet;
    iRet = RET_FALSE;
    if (!PathIsRelative(pcpath)) {
        iRet = RET_TRUE;
    }
    return iRet;
}


/*!
    @brief          KpXԋp
*/
static char *cdmtrLibFileGetRealPath(const char *pcPath)
{
    char *pcRetPath = NULL;
    unsigned int iLenDir = 0;
    if (NULL == pcPath) {
        return NULL;
    }
    if ('\0' == *pcPath) {
        return NULL;
    }
    pcRetPath = (char *)malloc(FILE_PATH_SIZE + 1);
    if (pcRetPath == NULL) {
        return NULL;
    }
    memset(pcRetPath, 0x00, FILE_PATH_SIZE + 1);
    if (!PathCanonicalize(pcRetPath, pcPath)) {
        memset(pcRetPath, 0x00, FILE_PATH_SIZE + 1);
        iLenDir = strlen(pcPath);
        strncpy(pcRetPath, pcPath, iLenDir);
        pcRetPath[iLenDir] = '\0';
    }
    return pcRetPath;
}


/*!
    @brief              œnꂽpXɁAVtH_ǉ
*/
static char *cdmtrLibFileAddPathName(char *pcPath, const char *pcAdd)
{
    char *pcRetPath = NULL;
    int iLen = 0;
    if (NULL == pcPath) {
        return NULL;
    }
    if (NULL == pcAdd) {
        iLen = strlen(pcPath);
        pcRetPath = (char *)malloc(sizeof(char) * (iLen + 1));
        if (NULL == pcRetPath) {
            return NULL;
        }
        memset(pcRetPath, '\0', iLen + 1);
        memcpy(pcRetPath, pcPath, iLen);
        return pcRetPath;
    }
    iLen = strlen(pcPath) + 1 + strlen(pcAdd);
    pcRetPath = (char *)malloc(sizeof(char) * (iLen + 1));
    if (NULL == pcRetPath) {
        return NULL;
    }
    memset(pcRetPath, '\0', iLen + 1);
    iLen = strlen(pcPath);
    memcpy(pcRetPath, pcPath, iLen);
    *(pcRetPath + iLen) = FILE_SEPALATE;
    memcpy(pcRetPath + iLen + 1, pcAdd, strlen(pcAdd));
    return pcRetPath;
}

/*!
    @brief          w肵fBNg݂邩mF
*/
int CDMtrLibFileIsDir(const char *pcdir)
{
    int iRet;
    iRet = RET_FALSE;
    if (pcdir == NULL) {
        goto FUNC_END;
    }
    if (PathIsDirectory(pcdir)) {
        iRet = RET_TRUE;
    }
FUNC_END:
    return iRet;
}

/*!
    @brief          w肵t@C݂邩mF
*/
int CDMtrLibFileIsFile(const char *pcfile)
{
    int iRet;
    iRet = RET_FALSE;
    if (pcfile == NULL) {
        goto FUNC_END;
    }
    if (PathFileExists( pcfile ) && !PathIsDirectory( pcfile )) {
        iRet = RET_TRUE;
    }
FUNC_END:
    return iRet;
}


/*!
    @brief          wfBNgȉ̃t@Cꗗ擾
*/
int CDMtrLibFileGetFileList(const char *pcdir, _FileList **ppflist)
{
    _FileList *pfiles;
    _FileTree tree_first;
    _FileTree *pftree;
    int iFileNum;
    int iRet, iFuncRet;
    int i;
    iRet = RET_NG;
    tree_first.next = NULL;
    pftree = &tree_first;
    if (pcdir == NULL || ppflist == NULL) {
        goto FUNC_END;
    }
    *ppflist = NULL;
    iFileNum = 0;
    if (strlen(pcdir) <= 0) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsFile(pcdir)) {
        int iPathLen;
        pftree->next = (_FileTree *)malloc(sizeof(_FileTree));
        if (pftree->next == NULL) {
            goto FUNC_END;
        }
        memset(pftree->next, 0x00, sizeof(_FileTree));
        pftree = pftree->next;
        iPathLen = strlen(pcdir);
        pftree->fname = malloc(iPathLen + 1);
        if (pftree->fname == NULL) {
            goto FUNC_END;
        }
        iFileNum++;
        strncpy(pftree->fname, pcdir, iPathLen);
        pftree->fname[iPathLen] = '\0';
    } else {
        iFuncRet = cdmtrLibFileGetDir(pcdir, &iFileNum, &pftree);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    if (iFileNum > 0) {
        pfiles = (_FileList *)malloc(sizeof(_FileList) + (iFileNum - 1) * sizeof (char *));
    } else {
        pfiles = (_FileList *)malloc(sizeof(_FileList) + sizeof (char *));
    }
    if (pfiles == NULL) {
        goto FUNC_END;
    }
    memset(pfiles, 0x00, sizeof(_FileList) + (iFileNum - 1) * sizeof (char *));
    pfiles->count = iFileNum;
    pftree = tree_first.next;
    for (i = 0; i < iFileNum; i++) {
        pfiles->filename[i] = pftree->fname;
        pftree = pftree->next;
    }
    iRet = RET_OK;
    *ppflist = pfiles;
FUNC_END:
    for (pftree = tree_first.next; pftree != NULL; pftree = tree_first.next) {
        tree_first.next = pftree->next;
        free(pftree);
    }
    return iRet;
}

/*!
    @brief              t@CXg̗̈
*/
void CDMtrLibFileFreeFileList(_FileList **ppflist)
{
    int iCount;
    if (ppflist == NULL) {
        goto FUNC_END;
    }
    if (*ppflist == NULL) {
        goto FUNC_END;
    }
    if ((*ppflist)->count > 0) {
        iCount = (*ppflist)->count - 1;
        for (; iCount >= 0 ; iCount--) {
            if ((*ppflist)->filename[iCount] != NULL) {
                free((*ppflist)->filename[iCount]);
            }
            free(*ppflist);
        }
    }
    *ppflist = NULL;
FUNC_END:
    return;
}

/*!
    @brief          fBNg̃t@CꗗXg擾
*/
static int cdmtrLibFileGetDir(const char *pcDirpath, int *piFilenum, _FileTree **ppftree)
{
    int iRet, iFuncRet;
    char *pcTargetPath;
    _FileTree *pFileTree;
    int iPathLen;
    char *pcTargetDir;
    char *pcSeachPath;
    WIN32_FIND_DATA fd;
    HANDLE hFind;
    BOOL bFindRes;
    DWORD dwLastError = 0;
    bFindRes = RET_TRUE;
    iRet = RET_NG;
    pcTargetPath = NULL;
    pcTargetDir = NULL;
    pcSeachPath = NULL;
    hFind = INVALID_HANDLE_VALUE;
    pFileTree = *ppftree;
    pcTargetPath = (char *)malloc(FILE_PATH_SIZE + 1);
    if (pcTargetPath == NULL) {
        goto FUNC_END;
    }
    memset(pcTargetPath, 0x00, FILE_PATH_SIZE + 1);
    pcTargetDir = (char *)malloc(FILE_PATH_SIZE + 1);
    if (pcTargetDir == NULL) {
        goto FUNC_END;
    }
    memset(pcTargetDir, 0x00, FILE_PATH_SIZE + 1);
    pcSeachPath = (char *)malloc(FILE_PATH_SIZE + 1);
    if (pcSeachPath == NULL) {
        goto FUNC_END;
    }
    memset(pcSeachPath, 0x00, FILE_PATH_SIZE + 1);
    iPathLen = strlen(pcDirpath);
    if (0 >= iPathLen || iPathLen > FILE_PATH_SIZE) {
        goto FUNC_END;
    }
    strncpy(pcSeachPath, pcDirpath, iPathLen);
    pcSeachPath[iPathLen] = '\0';
    if (CDMtrLibFileIsDir(pcDirpath)) {
        strncpy(pcTargetDir, pcDirpath, iPathLen);
        pcTargetDir[iPathLen] = '\0';
        if (PathCombine(pcSeachPath, pcDirpath, FILE_TARGET_TYPE) == NULL) {
            goto FUNC_END;
        }
    } else {
        iFuncRet = CDMtrLibFileGetDirName(pcSeachPath, pcTargetDir);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    hFind = FindFirstFile(pcSeachPath, &fd);
    if (hFind == INVALID_HANDLE_VALUE) {
        dwLastError = GetLastError();
        if ( iShowDetails == DETAILS_SHOW || iDebug == DEBUG_ON || iNumOfUnrecognized == 0) {
            if (dwLastError == ERROR_PATH_NOT_FOUND) {
                CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000497, dwLastError, pcSeachPath); //* MSG_LOG_ID_C_00000497
                iNumOfUnrecognized++;
            } else if (dwLastError != ERROR_FILE_NOT_FOUND) {
                CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000498, dwLastError, pcTargetDir); //* MSG_LOG_ID_C_00000498
                iNumOfUnrecognized++;
            }
        }
        iRet = RET_OK;
        goto FUNC_END;
    } else {
        do {
            if (strcmp(fd.cFileName, ".") == 0 || strcmp(fd.cFileName, "..") == 0) {
            } else if (strstr(fd.cFileName, "?") != NULL) {
                if ( iShowDetails == DETAILS_SHOW || iDebug == DEBUG_ON || iNumOfUnrecognized == 0) {
                    CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000499, ERROR_PATH_NOT_FOUND, pcTargetDir , fd.cFileName); //* MSG_LOG_ID_C_00000499
                }
                iNumOfUnrecognized++;
            } else {
                memset(pcTargetPath, 0x00, FILE_PATH_SIZE + 1);
                if (PathCombine(pcTargetPath, pcTargetDir, fd.cFileName) == NULL) {
                    goto FUNC_END;
                }
                if ( (fd.dwFileAttributes & 0x10) != 0 ) {
                    iFuncRet = cdmtrLibFileGetDir(pcTargetPath, piFilenum, &pFileTree);
                    if (iFuncRet != RET_OK) {
                        goto FUNC_END;
                    }
                } else {
                    (*piFilenum)++;
                    pFileTree->next = (_FileTree *)malloc(sizeof(_FileTree));
                    if (pFileTree->next == NULL) {
                        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000506); //* MSG_LOG_ID_C_00000506
                        goto FUNC_END;
                    }
                    memset(pFileTree->next, 0x00, sizeof(_FileTree));
                    pFileTree = pFileTree->next;
                    iPathLen = strlen(pcTargetPath);
                    pFileTree->fname = (char *)malloc(iPathLen + 1);
                    if (pFileTree->fname == NULL) {
                        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000507); //* MSG_LOG_ID_C_00000507
                        goto FUNC_END;
                    }
                    strncpy(pFileTree->fname, pcTargetPath, iPathLen);
                    pFileTree->fname[iPathLen] = '\0';
                }
            }
            while (!(bFindRes = FindNextFile(hFind, &fd))) {
                dwLastError = GetLastError();
                if (ERROR_NO_MORE_FILES == dwLastError) {
                    break;
                } else {
                    if ( iShowDetails == DETAILS_SHOW || iDebug == DEBUG_ON || iNumOfUnrecognized == 0) {
                        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000500, dwLastError, pcTargetDir , fd.cFileName); //* MSG_LOG_ID_C_00000500
                    }
                    iNumOfUnrecognized++;
                    bFindRes = RET_TRUE;
                }
            }
        } while (bFindRes);
    }
    iRet = RET_OK;
FUNC_END:
    if (pcTargetPath != NULL) {
        free(pcTargetPath);
    }
    if (pcTargetDir != NULL) {
        free(pcTargetDir);
    }
    if (pcSeachPath != NULL) {
        free(pcSeachPath);
    }
    if (hFind != INVALID_HANDLE_VALUE) {
        FindClose(hFind);
    }
    *ppftree = pFileTree;
    return iRet;
}

/*!
    @brief          fBNg쐬
*/
int CDMtrLibFileCreateDir(const char *pcDirpath)
{
    int iRet;
    iRet = RET_NG;
    if (pcDirpath == NULL) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsDir(pcDirpath)) {
        iRet = RET_OK;
        goto FUNC_END;
    }
    if (!CreateDirectory(pcDirpath, NULL)) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000508, pcDirpath); //* MSG_LOG_ID_C_00000508
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000509, errno); //* MSG_LOG_ID_C_00000509
        goto FUNC_END;
    }
    iRet = RET_OK;
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000510, pcDirpath); //* MSG_LOG_ID_C_00000510
FUNC_END:
    return iRet;
}

/*!
    @brief          PfBNg폜
*/
int CDMtrLibFileDeleteDir(const char *pcDirpath)
{
    int iRet;
    iRet = RET_NG;
    if (pcDirpath == NULL) {
        goto FUNC_END;
    }
    if (!RemoveDirectory(pcDirpath)) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000511, pcDirpath); //* MSG_LOG_ID_C_00000511
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000512, errno); //* MSG_LOG_ID_C_00000512
        goto FUNC_END;
    } else {
        iRet = RET_OK;
    }
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000513, pcDirpath); //* MSG_LOG_ID_C_00000513
FUNC_END:
    return iRet;
}

/*!
    @brief          fBNgẑׂẴt@C/fBNg폜
*/
int CDMtrLibFileDeleteDirAll(const char *pcDirpath, int iDelFlg)
{
    int iRet, iFuncRet;
    iRet = RET_NG;
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000514, pcDirpath); //* MSG_LOG_ID_C_00000514
    iFuncRet = cdmtrLibFileDeleteSubDir(pcDirpath);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (iDelFlg != CDMTR_FALSE) {
        iFuncRet = CDMtrLibFileDeleteDir(pcDirpath);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000515); //* MSG_LOG_ID_C_00000515
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          fBNĝׂẴt@C/fBNg폜
*/
static int cdmtrLibFileDeleteSubDir(const char *pcDirpath)
{
    int iRet;
    char *pcTargetPath;
    WIN32_FIND_DATA fd;
    HANDLE hFind;
    iRet = RET_NG;
    pcTargetPath = NULL;
    hFind = INVALID_HANDLE_VALUE;
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000516, pcDirpath); //* MSG_LOG_ID_C_00000516
    pcTargetPath = (char *)malloc(FILE_PATH_SIZE + 1);
    if (pcTargetPath == NULL) {
        goto FUNC_END;
    }
    memset(pcTargetPath, 0x00, FILE_PATH_SIZE + 1);
    if (PathCombine(pcTargetPath, pcDirpath, FILE_TARGET_TYPE) == NULL) {
        goto FUNC_END;
    }
    hFind = FindFirstFile(pcTargetPath, &fd);
    if (hFind == INVALID_HANDLE_VALUE) {
        goto FUNC_END;
    } else {
        do {
            if (strcmp(fd.cFileName, ".") == 0 || strcmp(fd.cFileName, "..") == 0) {
                continue;
            }
            memset(pcTargetPath, 0x00, FILE_PATH_SIZE + 1);
            if (PathCombine(pcTargetPath, pcDirpath, fd.cFileName) == NULL) {
                goto FUNC_END;
            }
            if ( (fd.dwFileAttributes & 0x10) != 0 ) {
                iRet = cdmtrLibFileDeleteSubDir(pcTargetPath);
                if (iRet != RET_OK) {
                    goto FUNC_END;
                }
                iRet = RemoveDirectory(pcTargetPath);
                if (iRet != RET_OK) {
                    goto FUNC_END;
                }
            } else {
                iRet = CDMtrLibFileDelete(pcTargetPath);
                if (iRet != RET_OK) {
                    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000517, pcTargetPath); //* MSG_LOG_ID_C_00000517
                    goto FUNC_END;
                }
            }
        } while (FindNextFile(hFind, &fd));
    }
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000524, pcDirpath); //* MSG_LOG_ID_C_00000524
    iRet = RET_OK;
FUNC_END:
    if (pcTargetPath != NULL) {
        free(pcTargetPath);
    }
    if (hFind != INVALID_HANDLE_VALUE) {
        FindClose(hFind);
    }
    return iRet;
}


/*!
    @brief          Pt@C폜
*/
int CDMtrLibFileDelete(const char *pcTargetfile)
{
    int iRet;
    iRet = RET_NG;
    if (pcTargetfile == NULL) {
        goto FUNC_END;
    }
    if (!CDMtrLibFileIsFile(pcTargetfile)) {
        iRet = RET_OK;
        goto FUNC_END;
    }
    if (!DeleteFile(pcTargetfile)) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000525, pcTargetfile); //* MSG_LOG_ID_C_00000525
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000526, errno); //* MSG_LOG_ID_C_00000526
        goto FUNC_END;
    }
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000527, pcTargetfile); //* MSG_LOG_ID_C_00000527
    iRet = RET_OK;
FUNC_END:
    return iRet;
}


/*!
    @brief          fBNgAt@Cl[
*/
int CDMtrLibFileRename(const char *pcOldPath, const char *pcNewPath)
{
    int iRet;
    iRet = RET_NG;
    if (pcOldPath == NULL || pcNewPath == NULL) {
        goto FUNC_END;
    }
    if (rename(pcOldPath, pcNewPath) != 0) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000528, pcOldPath, pcNewPath); //* MSG_LOG_ID_C_00000528
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000529, errno); //* MSG_LOG_ID_C_00000529
        goto FUNC_END;
    }
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000530, pcOldPath, pcNewPath); //* MSG_LOG_ID_C_00000530
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          e|tH_Ƀe|t@C𐶐
*/
int CDMtrLibFileCreateTempFile(char **ppcpath)
{
    int iRet;
    char *pcTempDir;
    char *pcTempPath;
    iRet = RET_NG;
    pcTempDir = NULL;
    pcTempPath = NULL;
    if (ppcpath == NULL) {
        goto FUNC_END;
    }
    *ppcpath = NULL;
    pcTempDir = (char *)malloc(FILE_PATH_SIZE + 1);
    if (pcTempDir == NULL) {
        goto FUNC_END;
    }
    memset(pcTempDir, 0x00, FILE_PATH_SIZE + 1);
    if ((GetTempPath(FILE_PATH_SIZE + 1, pcTempDir)) == 0) {
        goto FUNC_END;
    }
    pcTempPath = (char *)malloc(FILE_PATH_SIZE + 1);
    if (pcTempPath == NULL) {
        goto FUNC_END;
    }
    memset(pcTempPath, 0x00, FILE_PATH_SIZE + 1);
    if (GetTempFileName(pcTempDir,
                        "cdm",
                        0,
                        pcTempPath) == 0) {
        free(pcTempPath);
        goto FUNC_END;
    }
    CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000531, pcTempPath); //* MSG_LOG_ID_C_00000531
    *ppcpath = pcTempPath;
    iRet = RET_OK;
FUNC_END:
    if (pcTempDir != NULL) {
        free(pcTempDir);
    }
    return iRet;
}


/*!
    @brief          Zp[ǧ
*/
static char *cdmtrLibFileSearchSepalate(const char *pcStr)
{
    char *pPos = NULL;
    char *pRet = NULL;
    if (NULL == pcStr) {
        return (char *)pcStr;
    }
    pPos = PathFindFileName(pcStr);
    if (pPos != NULL) {
        if ((char *)pcStr < pPos) {
            if (*(pPos - 1) == FILE_SEPALATE) {
                pRet = pPos - 1;
            }
        }
    }
    return pRet;
}

