/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef CDMMTRLIBUTIL_W_H_
#define CDMMTRLIBUTIL_W_H_

#include <tchar.h>
#include <stdarg.h>

#define CHAR_NULL           _T('\0')
#define CHAR_CR             _T('\r')
#define CHAR_LF             _T('\n')
#define CHAR_BACKSLASH      _T('\\')
#define CHAR_SINGLEQUOTE    _T('\'')
#define CHAR_DOUBLEQUOTE    _T('\"')
#define CHAR_COMMA          _T(',')
#define CHAR_EQUAL          _T('=')
#define CHAR_HYPHEN         _T('-')
#define CHAR_SPACE          _T(' ')
#define CHAR_TAB            _T('\t')

#define STR_UNDERBAR        _T("_")


#define IS_NULL_CHAR(p) (*(p) == '\0')

#define IS_LF(p) (*(p) == 0x0a)

#define IS_CR(p) (*(p) == 0x0d)

#define IS_BREAK_LINE(p) (IS_LF(p) || IS_CR(p))

#define IS_HT(p) (*(p) == 0x09)

#define IS_VT(p) (*(p) == 0x0b)

#define IS_CTRL_TAB(p) (IS_HT(p) || IS_VT(p))

#define IS_EQUAL(p) (*(p) == '=')

#define IS_BACKSLASH(p) (*(p) == '\\')

#define IS_SINGLEQUOTE(p) (*(p) == '\'')

#define IS_DOUBLEQUOTE(p) (*(p) == '"')


#define IS_WHITE_SPACE(p) ((*(p) == ' ') || (*(p) == '\t'))

#define SKIP_WHITE_SPACE(pp) {\
        while(IS_WHITE_SPACE(*(pp))){\
            (*(pp))++;\
        }\
    }

#define SKIPBACK_WHITE_SPACE(pStart, ppEnd) {\
        while(*(ppEnd) > pStart){\
            (*(ppEnd))--;\
            if(!IS_WHITE_SPACE(*(ppEnd))){\
                (*(ppEnd))++;\
                break;\
            }\
        }\
    }

#define REGEX_ASTARISK "*"
#define REGEX_ASTARISK_CHAR '*'
#define REGEX_ESCAPE_CHAR '\\'

typedef enum {
    SEARCH_COMPLETE = 0,
    SEARCH_PREFIX,
    SEARCH_SUFFIX,
    SEARCH_PART
} SEARCH_TYPE;

#define ENCODING_EUC_JP             _T("EUC-JP")
#define ENCODING_SHIFT_JIS          _T("Shift_JIS")
#define ENCODING_UTF_8              _T("UTF-8")
#define ENCODING_SYSTEM             _T("SYSTEM")


#if defined(UNICODE)
#define CDMtrLibUtilGetStringNextDelimSP    CDMtrLibUtilGetStringNextDelimSPW
#define CDMtrExtPropGetAttributePos         CDMtrExtPropGetAttributePosW
#define CDMtrLibUtilSetString               CDMtrLibUtilSetStringW
#define CDMtrLibUtilCopyString              CDMtrLibUtilCopyStringW
#define CDMtrLibUtilCompString              CDMtrLibUtilCompStringW
#define CDMtrLibUtilStrCaseCmp              CDMtrLibUtilStrCaseCmpW
#define CDMtrLibUtilStrCaseStr              CDMtrLibUtilStrCaseStrW
#define CDMtrLibUtilMatchPattern            CDMtrLibUtilMatchPatternW
#define CDMtrLibUtilIsStringFalse           CDMtrLibUtilIsStringFalseW
#define CDMtrLibUtilSetLowString            CDMtrLibUtilSetLowStringW
#define CDMtrLibUtilSetUpperString          CDMtrLibUtilSetUpperStringW
#define CDMtrLibUtilIsAlpha                 CDMtrLibUtilIsAlphaW
#define CDMtrLibUtilIsDigit                 CDMtrLibUtilIsDigitW
#define CDMtrLibUtilMBCStoUNICODE			CDMtrLibUtilMBCStoUNICODEW
#define CDMtrLibUtilUNICODEtoMBCS			CDMtrLibUtilUNICODEtoMBCSW
#define CDMtrLibUtilCheckSupportCharset		CDMtrLibUtilCheckSupportCharsetW
#define CDMtrLibUtilSprintf                 CDMtrLibUtilConvertStringW

#define CDMtrLibUtilTolower                 CDMtrLibUtilTolowerW
#define CDMtrLibUtilToupper                 CDMtrLibUtilToupperW
#define CDMtrLibUtilGetNumSize              CDMtrLibUtilGetNumSizeW
#define CDMtrLibUtilUNICODEtoBYTE           CDMtrLibUtilUNICODEtoBYTEW
#define CDMtrLibUtilCheckOutputCharset      CDMtrLibUtilCheckOutputCharsetW
#endif




int CDMtrLibUtilGetStringNextDelimSPW(_TCHAR **string, _TCHAR **startp, _TCHAR **endp, _TCHAR delimiter);
int CDMtrExtPropGetAttributePosW(_TCHAR **currentp, _TCHAR **name_top, _TCHAR **name_end,
                                 _TCHAR **val_top, _TCHAR **val_end);
int CDMtrLibUtilSetStringW(_TCHAR **dst, const _TCHAR *val_top, const _TCHAR *val_end);
_TCHAR *CDMtrLibUtilCopyStringW(const _TCHAR *pcStr);
int CDMtrLibUtilCompStringW(const _TCHAR *pcDst, const _TCHAR *pcSrc);
int CDMtrLibUtilStrCaseCmpW(const _TCHAR *s1, const _TCHAR *s2, int n);
_TCHAR *CDMtrLibUtilStrCaseStrW(const _TCHAR *string, const _TCHAR *pattern);
int CDMtrLibUtilMatchPatternW(const _TCHAR *pcStr, const _TCHAR *pcReg);
int CDMtrLibUtilIsStringFalseW(const _TCHAR *val_top, const _TCHAR *val_end);
int CDMtrLibUtilSetLowStringW(_TCHAR *dst);
int CDMtrLibUtilSetUpperStringW(_TCHAR *dst);
int CDMtrLibUtilIsAlphaW(_TCHAR c);
int CDMtrLibUtilIsDigitW(_TCHAR c);
int CDMtrLibUtilMBCStoUNICODEW(wchar_t **new_char, const char **org_str, _TCHAR *charSet);
int CDMtrLibUtilUNICODEtoMBCSW(char **new_char, const wchar_t **org_str, _TCHAR *charSet);
int CDMtrLibUtilCheckSupportCharsetW(_TCHAR *cpCharset, unsigned int *uipCodePage);

int CDMtrLibUtilCreateStringW(_TCHAR **psBuffer, ...);
const _TCHAR *CDMtrLibUtilCheckPrefixW(const _TCHAR *psStr, const _TCHAR *psPrefix);
char *CDMtrLibUtilConvertStringW(const wchar_t *charset, const wchar_t *pFormat, ...);

void CDMtrLibUtilTolowerW(_TCHAR *pcStr);
void CDMtrLibUtilToupperW(_TCHAR *pcStr);
int CDMtrLibUtilGetNumSizeW(long lNum);
int CDMtrLibUtilUNICODEtoBYTEW(wchar_t **new_char, wchar_t **org_str);
_TCHAR *CDMtrLibUtilCheckOutputCharsetW(_TCHAR *pcCharset);

int CDMtrLibUtilMallocW(void **pcBuffer, size_t szLen);
void CDMtrLibUtilFreeW(void **pcBuffer);
int CDMtrLibUtilCopyStringToCmdOptBuffW(_TCHAR **pcBuffer, size_t *szBufSize, ...);

#endif
