/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include <windows.h>
#include <tchar.h>
#include <Psapi.h>

#include "CDMtrLibUtilW.h"
#include "CDMtrCmn.h"
#include "CDMtrMessage.h"

#define LOG_FILENAME_FORMAT MSG_LOG_ID_C_00000538 //* MSG_LOG_ID_C_00000538
 
static wchar_t *spcTargetFilePath = NULL;
 
static wchar_t *spcLogCharset = ENCODING_SYSTEM;
 
 
static void cdmtrLibCmnPrintLogW(const wchar_t *pFormat, va_list list);
 
 
/*!
    @brief          mۏ
*/
void *CDMtrLibCmnMallocW(size_t szLen)
{
    wchar_t *pcBuffer;
    pcBuffer = (wchar_t *)malloc(szLen * sizeof(wchar_t));
    if (pcBuffer == NULL) {
        goto FUNC_END;
    }
    memset(pcBuffer, 0, szLen * sizeof(wchar_t));
FUNC_END:
    return (void *)pcBuffer;
}


/*!
    @brief          
*/
void CDMtrLibCmnFreeW(void **ppcBuffer)
{
    if (ppcBuffer == NULL) {
        goto FUNC_END;
    }
    if (*ppcBuffer != NULL) {
        free(*ppcBuffer);
        *ppcBuffer = NULL;
    }
FUNC_END:
    return;
}


/*!
    @brief          Oɏo͂Ώۃt@CpXݒ
*/
void CDMtrLibCmnSetTargetNameW(const wchar_t *pcFilePath)
{
    spcTargetFilePath = (wchar_t *)pcFilePath;
    return;
}


/*!
    @brief  Oɏo͂Ώۃt@CpX
*/
void CDMtrLibCmnClearTargetNameW(void)
{
    spcTargetFilePath = NULL;
    return;
}


/*!
    @brief          Oo
*/
void CDMtrLibCmnLogOutW(
    const wchar_t *pFormat,
    ...)
{
    va_list ap;
    if (pFormat == NULL) {
        return;
    }
    va_start(ap, pFormat);
    cdmtrLibCmnPrintLogW(pFormat, ap);
    va_end(ap);
    return;
}


/*!
    @brief          G[Oo
*/
void CDMtrLibCmnErrLogOutW(
    const wchar_t *pFormat,
    ...)
{
    va_list ap;
    if (pFormat == NULL) {
        return;
    }
    if (spcTargetFilePath != NULL) {
        CDMtrLibCmnLogOutW(LOG_FILENAME_FORMAT, spcTargetFilePath); //* MSG_LOG_ID_C_00000536
    } else {
    }
    va_start(ap, pFormat);
    cdmtrLibCmnPrintLogW(pFormat, ap);
    va_end(ap);
    return;
}


/*!
    @brief          fobOOo
*/
void CDMtrLibCmnDebugLogOutW(int iOutFlg,
                             const wchar_t *pFormat,
                             ...)
{
    va_list ap;
    if ( iOutFlg == 0 || pFormat == NULL ) {
        return;
    }
    va_start(ap, pFormat);
    cdmtrLibCmnPrintLogW(pFormat, ap);
    va_end(ap);
    return;
}


/*!
    @brief          g[XOo
*/
void CDMtrLibCmnTrcLogOutW(
    const wchar_t *pFormat,
    ...)
{
#ifdef CDMTR_DEBUG
    va_list ap;
    va_start(ap, pFormat);
    cdmtrLibCmnPrintLogW(pFormat, ap);
    va_end(ap);
#endif
    return;
}


/*!
    @brief          Oo͕Zbgݒ
*/
void CDMtrLibCmnSetLogCharsetW(const wchar_t *pcCharset)
{
    spcLogCharset = (wchar_t *)pcCharset;
    return;
}


/*!
	@brief      Oo͋ʏ
*/
static void cdmtrLibCmnPrintLogW(const wchar_t *pFormat, va_list list)
{
    int wLength = 0;
    int wBufSize = 0;
    wchar_t *wcBuf = NULL;
    char *cBuf = NULL;
    int iFuncRet = RET_NG;
    wLength = _vscwprintf(pFormat, list);
    wBufSize = (wLength + 1) * sizeof(wchar_t);
    wcBuf = (wchar_t *)malloc(wBufSize);
    if (NULL == wcBuf) {
        goto FUNC_END;
    }
    memset(wcBuf, 0x00, wBufSize);
    vswprintf(wcBuf, wBufSize, pFormat, list);
    iFuncRet = CDMtrLibUtilUNICODEtoMBCSW(&cBuf, &wcBuf, spcLogCharset);
    if (iFuncRet == RET_NG) {
        goto FUNC_END;
    }
    fprintf(stdout, cBuf);
FUNC_END:
    if (cBuf != NULL) {
        free((void *)cBuf);
    }
    if (wcBuf != NULL) {
        free((void *)wcBuf);
    }
    return;
}

/*!
    @brief          
*/
void CDMtrLibCmnProcessMemoryW(_TCHAR *mdl, _TCHAR *cmt, const HANDLE hProcess, const DWORD dwProcessID)
{
    PROCESS_MEMORY_COUNTERS pmc = { 0 };
    HANDLE hPs = { 0 };
    if (hProcess == NULL && dwProcessID > 0) {
        hPs = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, dwProcessID);
    } else if (hProcess != NULL) {
        hPs = hProcess ;
    }
    if (hPs != NULL) {
        if ( GetProcessMemoryInfo(hPs, &pmc, sizeof(pmc)) ) {
            CDMtrLibCmnLogOutW( _T("[MEMORY],%lu, 0x%p,%10lu,%10lu,%10lu,%10lu,%10lu,%10lu,%10lu,%10lu,%10lu,%s,%s\n"), //* MSG_LOG_ID_C_00000537
                                (DWORD)dwProcessID,
                                hPs,
                                pmc.PeakWorkingSetSize,
                                pmc.WorkingSetSize,
                                pmc.QuotaPeakPagedPoolUsage,
                                pmc.QuotaPagedPoolUsage,
                                pmc.QuotaPeakNonPagedPoolUsage,
                                pmc.QuotaNonPagedPoolUsage,
                                pmc.PeakPagefileUsage,
                                pmc.PagefileUsage,
                                pmc.PageFaultCount,
                                mdl,
                                cmt );
        }
        if (hProcess == NULL && dwProcessID > 0) {
            CloseHandle( hPs );
        }
    }
    return;
}
