/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "CDMtrCmn.h"
#include "CDMtrExtCmn.h"
#include "CDMtrLibCmnW.h"
#include "CDMtrLibPropW.h"
#include "CDMtrLibUtilW.h"

#define PROP_GROUP_START           '['

#define PROP_GROUP_END             ']'

#define PROP_COMMENT_CHAR          '#'

typedef struct tag_PropInfoTreeW {
    struct tag_PropInfoTreeW *next;
    _PropInfoW *ptInfo;
} _PropInfoTreeW;

static _TCHAR stBuff[BUFF_LINE_LENGTH];


static int cdmtrPropIsGroupNameW(const _TCHAR *pcGroupName, _TCHAR *pcCurrent);
static int cdmtrPropIsGroupStartW(_TCHAR *pcCurrent);
static int cdmtrLibPropGetTreeW(FILE *pfProp, _PropInfoTreeW **pptTree);


/*!
    @brief          O[vɑݒ擾
*/
int CDMtrLibPropGetListW(const _TCHAR *pcFileName, const _TCHAR *pcGroupName, _PropInfoListW **pptPropList)
{
    int iRet;
    int iCount;
    int i;
    int iGroupFlag;
    _TCHAR *currentp;
    FILE *pfProp;
    _PropInfoTreeW tree_first;
    _PropInfoTreeW *ptTree;
    _PropInfoListW *ptInfo;
    iRet = RET_NG;
    ptInfo = NULL;
    iCount = 0;
    iGroupFlag = RET_FALSE;
    pfProp = NULL;
    tree_first.next = NULL;
    tree_first.ptInfo = NULL;
    ptTree = &tree_first;
    if (pcFileName == NULL) {
        goto FUNC_END;
    }
    if (pcGroupName == NULL) {
        goto FUNC_END;
    }
    if (pptPropList == NULL) {
        goto FUNC_END;
    }
    *pptPropList = NULL;
    pfProp = _tfopen(pcFileName, _T("r, ccs=UTF-8"));
    if (pfProp == NULL) {
        goto FUNC_END;
    }
    while (_fgetts(stBuff, BUFF_LINE_LENGTH, pfProp) != NULL) {
        currentp = stBuff;
        SKIP_WHITE_SPACE(&currentp);
        if (IS_NULL_CHAR(currentp) || IS_BREAK_LINE(currentp)) {
            continue;
        }
        if (cdmtrPropIsGroupNameW(pcGroupName, currentp)) {
            iCount = cdmtrLibPropGetTreeW(pfProp, &ptTree);
            iGroupFlag = RET_TRUE;
            break;
        }
    }
    if (!iGroupFlag) {
        goto FUNC_END;
    }
    ptInfo = (_PropInfoListW *)CDMtrLibCmnMallocW(sizeof(_PropInfoListW) + (iCount - 1) * sizeof (_PropInfoW *));
    if (ptInfo == NULL) {
        goto FUNC_END;
    }
    ptInfo->iNum = iCount;
    ptTree = tree_first.next;
    for (i = 0; i < iCount; i++) {
        ptInfo->ptPropInfo[i] = ptTree->ptInfo;
        ptTree = ptTree->next;
    }
    iRet = RET_OK;
    *pptPropList = ptInfo;
FUNC_END:
    for (ptTree = tree_first.next; ptTree != NULL; ptTree = tree_first.next) {
        tree_first.next = ptTree->next;
        CDMtrLibCmnFreeW((void **)&ptTree);
    }
    if (pfProp != NULL) {
        fclose(pfProp);
        pfProp = NULL;
    }
    return iRet;
}


/*!
    @brief          w肳ꂽݒ񃊃Xg̃̈
*/
void CDMtrLibPropFreeListW(_PropInfoListW **pptPropList)
{
    int iCount;
    if (pptPropList == NULL) {
        goto FUNC_END;
    }
    if (*pptPropList == NULL) {
        goto FUNC_END;
    }
    iCount = (*pptPropList)->iNum - 1;
    for (; iCount >= 0 ; iCount--) {
        if ((*pptPropList)->ptPropInfo[iCount] != NULL) {
            CDMtrLibCmnFreeW((void **) & ((*pptPropList)->ptPropInfo[iCount]->pcAttr));
            CDMtrLibCmnFreeW((void **) & ((*pptPropList)->ptPropInfo[iCount]->pcVal));
            CDMtrLibCmnFreeW((void **) & ((*pptPropList)->ptPropInfo[iCount]));
        }
    }
    CDMtrLibCmnFreeW((void **)pptPropList);
    *pptPropList = NULL;
FUNC_END:
    return;
}


/*!
    @brief          區ږv邩
*/
static int cdmtrPropIsGroupNameW(const _TCHAR *pcGroupName, _TCHAR *pcCurrent)
{
    int iRet, iFuncRet;
    _TCHAR *p, *startp, *endp;
    _TCHAR *pcName;
    iRet = RET_FALSE;
    p = pcCurrent;
    pcName = NULL;
    if (*p != PROP_GROUP_START) {
        goto FUNC_END;
    }
    p++;
    SKIP_WHITE_SPACE(&p);
    startp = p;
    endp = p;
    while (!IS_NULL_CHAR(p)) {
        if (IS_WHITE_SPACE(p) || *p == PROP_GROUP_END) {
            break;
        }
        endp = p;
        p++;
    }
    SKIP_WHITE_SPACE(&p);
    if (*p != PROP_GROUP_END) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibUtilSetStringW(&pcName, startp, endp);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (CDMtrLibUtilCompStringW(pcGroupName, pcName)) {
        iRet = RET_TRUE;
    }
FUNC_END:
    CDMtrLibCmnFreeW((void **)&pcName);
    return iRet;
}


/*!
    @brief          區ڊJns𔻒f
*/
static int cdmtrPropIsGroupStartW(_TCHAR *pcCurrent)
{
    int iRet;
    _TCHAR *p;
    iRet = RET_FALSE;
    p = pcCurrent;
    if (*p != PROP_GROUP_START) {
        goto FUNC_END;
    }
    p++;
    SKIP_WHITE_SPACE(&p);
    while (!IS_NULL_CHAR(p)) {
        if (IS_WHITE_SPACE(p) || *p == PROP_GROUP_END) {
            break;
        }
        p++;
    }
    SKIP_WHITE_SPACE(&p);
    if (*p == PROP_GROUP_END) {
        iRet = RET_TRUE;
    }
FUNC_END:
    return iRet;
}


/*!
    @brief          ƒl̃c[擾
*/
static int cdmtrLibPropGetTreeW(FILE *pfProp, _PropInfoTreeW **pptTree)
{
    int iFuncRet;
    int iNum;
    _TCHAR *currentp;
    _TCHAR *name_top, *name_end, *val_top, *val_end;
    _TCHAR *pcAttr, *pcVal;
    _PropInfoTreeW *ptTree;
    iNum = 0;
    ptTree = *pptTree;
    pcAttr = NULL;
    pcVal = NULL;
    while (_fgetts(stBuff, BUFF_LINE_LENGTH, pfProp) != NULL) {
        currentp = stBuff;
        SKIP_WHITE_SPACE(&currentp);
        if (IS_NULL_CHAR(currentp) || IS_BREAK_LINE(currentp)) {
            continue;
        }
        if (*currentp == PROP_COMMENT_CHAR) {
            continue;
        }
        if (cdmtrPropIsGroupStartW(currentp)) {
            break;
        }
        iFuncRet = CDMtrExtPropGetAttributePosW(&currentp, &name_top, &name_end, &val_top, &val_end);
        if (iFuncRet != RET_OK) {
            continue;
        }
        if (name_top == NULL || name_end == NULL) {
            continue;
        }
        iFuncRet = CDMtrLibUtilSetStringW(&pcAttr, name_top, name_end);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        if (val_top == NULL || val_end == NULL) {
            pcVal = (_TCHAR *)CDMtrLibCmnMallocW(1);
            if (pcVal == NULL) {
                goto FUNC_END;
            }
            *pcVal = '\0';
        } else {
            iFuncRet = CDMtrLibUtilSetStringW(&pcVal, val_top, val_end);
            if (iFuncRet != RET_OK) {
                goto FUNC_END;
            }
        }
        ptTree->next = (_PropInfoTreeW *)CDMtrLibCmnMallocW(sizeof(_PropInfoTreeW));
        if (ptTree->next == NULL) {
            goto FUNC_END;
        }
        iNum++;
        ptTree = ptTree->next;
        ptTree->ptInfo = (_PropInfoW *)CDMtrLibCmnMallocW(sizeof(_PropInfoW));
        if (ptTree->ptInfo == NULL) {
            goto FUNC_END;
        }
        memset(ptTree->ptInfo, 0x00, sizeof(_PropInfoW));
        ptTree->ptInfo->pcAttr = pcAttr;
        ptTree->ptInfo->pcVal = pcVal;
        pcAttr = NULL;
        pcVal = NULL;
    }
FUNC_END:
    CDMtrLibCmnFreeW((void **)&pcAttr);
    CDMtrLibCmnFreeW((void **)&pcVal);
    return iNum;
}
