/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "CDMtrSpsIface.h"
#include "CDMtrSps.h"
#include "CDMtrSpsCmn.h"
#include "CDMtrCmn.h"

#include "CDMtrLibCmn.h"
#include "CDMtrLibOpt.h"
#include "CDMtrLibUtil.h"
#include "CDMtrSparsMessage.h"

#include<windows.h>


#define MAX_FILE_NAME   (2048)
#define MAX_BUF_ARRAY   (2048)
#define MAX_NUMARY      (512)

#define FLAG_ON     CDMTRSPS_FLAG_ON
#define FLAG_OFF    CDMTRSPS_FLAG_OFF

#define TOKEN_IF            "if"
#define TOKEN_IFDEF         "ifdef"
#define TOKEN_IFNDEF        "ifndef"
#define TOKEN_ELSE          "else"
#define TOKEN_ELSE_SPACE    "else "
#define TOKEN_ELSE_BRACE    "else { "
#define TOKEN_ELSE_IF       "else if"
#define TOKEN_ELIF          "elif"
#define TOKEN_ENUM          "enum"
#define TOKEN_ENDIF         "endif"
#define TOKEN_SWITCH        "switch"
#define TOKEN_CASE          "case"
#define TOKEN_DEFAULT       "default"
#define TOKEN_BREAK         "break"
#define TOKEN_FOR           "for"
#define TOKEN_WHILE         "while"
#define TOKEN_DO            "do"
#define TOKEN_STRUCT        "struct"
#define TOKEN_BRACE_OPEN    "{"
#define TOKEN_BRACE_CLOSE   "}"
#define TOKEN_SEMICOLON     ";"

#define OPTION_LANG_NAME               (1)
#define OPTION_LANG_TYPE               (2)
#define OPTION_FILE_PATH               (3)
#define OPTION_FILE_NAME               (4)
#define OPTION_METRIC_FILE_PATH        (5)
#define OPTION_METRIC_SKIP             (6)
#define OPTION_ERROR_OUTPUT            (7)
#define OPTION_FUNCTION_OPT            (8)
#define OPTION_QRT_FLG                 (9)
#define OPTION_DEBUG2                  (10)
#define OPTION_OUT_FORMAT              (11)

#define OPTION_STR_LANG_NAME           "-lang_name"
#define OPTION_STR_LANG_TYPE           "-lang_type"
#define OPTION_STR_FILE_PATH           "-file_path"
#define OPTION_STR_FILE_NAME           "-file_name"
#define OPTION_STR_METRIC_FILE_PATH    "-metric_file"
#define OPTION_STR_METRIC_SKIP         "-metric_skip"
#define OPTION_STR_ERROR_OUTPUT        "-error_output"
#define OPTION_STR_FUNCTION_OPT        "-func_opt"
#define OPTION_STR_QRT_FLG             "-qrt_flg"
#define OPTION_STR_DEBUG2              "-debug2"
#define OPTION_STR_OUT_FORMAT          "-out_format"

#define FUNC_OPTION_NONE _T("none")
#define FUNC_OPTION_PARAM _T("param")
#define OUT_FORMAT_FUNC _T("func")

static OptionInfo stOptionArray[] = {
    { OPTION_LANG_NAME,        OPTION_STR_LANG_NAME,        OPT_HAS_PARAM },
    { OPTION_LANG_TYPE,        OPTION_STR_LANG_TYPE,        OPT_HAS_PARAM },
    { OPTION_FILE_PATH,        OPTION_STR_FILE_PATH,        OPT_HAS_PARAM },
    { OPTION_FILE_NAME,        OPTION_STR_FILE_NAME,        OPT_HAS_PARAM },
    { OPTION_METRIC_FILE_PATH, OPTION_STR_METRIC_FILE_PATH, OPT_HAS_PARAM },
    { OPTION_METRIC_SKIP,      OPTION_STR_METRIC_SKIP,      OPT_HAS_PARAM },
    { OPTION_ERROR_OUTPUT,     OPTION_STR_ERROR_OUTPUT,     OPT_HAS_PARAM },
    { OPTION_FUNCTION_OPT,     OPTION_STR_FUNCTION_OPT,     OPT_HAS_PARAM },
    { OPTION_QRT_FLG,          OPTION_STR_QRT_FLG,          OPT_HAS_PARAM },
    { OPTION_DEBUG2,           OPTION_STR_DEBUG2,           OPT_HAS_NOPARAM },
    { OPTION_OUT_FORMAT,       OPTION_STR_OUT_FORMAT,       OPT_HAS_PARAM }
};


PARAM_INFO_T    strParamInfo;

static char SourceFile[MAX_FILE_NAME];

int iSpsDebug;


struct ParsesEventFunc M_method;


struct ParsesSettingFunc S_method;
typedef struct {
    int     iSpsInfo_Cnt;
    int     iSWITCH_NEST_Cnt;
    int     iSWITCH_NEST_Flag;
    int     iBraceCloseLineNo;
    char   *LineStrBuf;
    int     LineNoBuf;

    int     LineNoBuf4CLOC;

    int     ELOC_Cnt_LineNo;
    int     CLOC_Cnt_LineNo;
    int     SKIP_Cnt_LineNo;
    int     ELSE_ELOC_Cnt_LineNo;
    int     PREP_IF_ZERO_Flag ;
    int     FOR_Flag;
    int     SWITCH_Flag;
    int     CASE_Flag;
    int     DO_WHILE_Flag;
    int     Func_decide_Flag;
    int     Func_start_Flag;
    int     LineBreak_Flag;
    struct  BlankBreak {
        int line_from;
        int line_to;
    } bb;
    struct  EnumInfo {
        int line_from;
        int line_to;
    } enum_info;
    struct  StructInfo {
        int line_from;
        int line_to;
    } struct_info;
} SpsDataInfo;
SpsDataInfo sps_data_info;

typedef struct {
    char *sps_LangName;
    char *sps_DirPath;
    char *sps_FName;
    char *sps_pcTemp;
    int  sps_SkipFlag;
    char *sps_FuncOpt;
    int  sps_QrtFlag;
    int  iLangType;
} SpsFileInfo;

SpsFileInfo sps_fileinfo;

typedef struct {
    char method_name[MAX_NUMARY];

    char param_name[MAX_BUF_ARRAY];

    int sps_line_from;
    int sps_line_to;
    int sps_floc;
    int sps_eloc;
    int sps_cloc;
    int sps_bloc;
    int sps_skip;
    int sps_cyclo;
    int sps_cyclowc;
    int sps_branchs;
    int sps_cndops;
    int sps_nests;
    int sps_evalr;
    int sps_evals;
    int sps_avals;
    int sps_parms;
} SpsCntInfo;

static SpsCntInfo *sps_info = NULL;

int     iSpsInfo_Cnt_Max = 0;


void eventTokenString(int tokrn_id, int line_from, int colum_from, int line_to, int colum_to, char *token_text);
void eventLineBreak(int break_id, int line_from, int colum_from, int line_to, int colum_to);
void eventLineSkip(int line_from, int colum_from, int line_to, int colum_to, char *skipline);
void eventComment(int cmt_type, int cmtno, int linenum, int line_from, int colum_from, int line_to, int colum_to, char *cmtline);
void eventMacro(int pre_type, int preno, int linenum, int line_from, int colum_from, int line_to, int colum_to, int pre_nest, int pre_pars, int *macronest, int cntmacronest, int iSkipLine, char *keyname, char *keyparam);
void eventFileEof(int linenum);

void eventStatement(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, char *statement_text);
void eventFuncCandidate(int line_from, int line_to, char *funcname, int cntparam, char *funcline);
void eventFuncToStatement(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, char *funcname, char *statement_text);
void eventFunction(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, char *funcname, char *param_name, int cntparam, char *funcline, int ifunc_name_line);
void eventFuncEnd(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, int cntmaxnests);
void eventBranchCandidate(int branch_type, int line_from, int line_to, char *statement_text);
void eventBranchStatement(int branch_type, int **StackTokenInfo, int iStackCnt, int line_from, int line_to, int cntcondition, char *statement_text);
void eventBraceNest(int brace_type, int **StackTokenInfo, int iStackCnt, int line_from, int line_to, int cntinner, int cntouter, int cntmaxinner, int cntmaxouter, char *token_text);
void eventParenNest(int paren_type, int line_from, int line_to, int cntfunc, int cntbranch, char *token_text);
int cdmtrSpsMain(int debug_mode, char *filepath, char *pcLangName, char *pcDirPath, char *pcFName, char *pcTemp, int iSkipFlg, char *strFuncOpt, int QrtFlg, int iLangType, int iOutFmtFlg);
int cdmtrSpsCalc();
void cdmtrSpsCalcStruct(int iCnt, FILE *pFile);
void cdmtrSpsCalcFunc(int iCnt, FILE *pFile);

void cdmtrSpsEnumStructLineInfoMemory(int line_to, int line_from, char *linestr);
void CDMtrSpsCountElocLine(int **StackTokenInfo, int iStackCnt, int line_to, int iFuncCandinate, int iFuncNameLineNo);

int GetTargetLangType(void);
int GetFunctionStartFlag(void);

extern void call_setParamState();
int giQrt_Out_Flg_ext;
extern void GetStackTokens(int ** *, int *);


/*!
    @brief      C/C++ȈՃp[T C֐
*/
int wmain(int argc, wchar_t *argv[])
{
    int iRet = RET_OK;
    char *pcParamStr = NULL;
    int iChkRes = 0;
    FILE *errfile = NULL;
    int i = 0;
    char **cArgv = NULL;
    int i_result = 0;
    int utf8_flg = CDMTR_FALSE;
    char *codepage = ENCODING_SYSTEM;
    if (argc > 0) {
        cArgv = (char **)malloc(sizeof(char *) * argc);
        if (cArgv == NULL) {
            iRet = RET_NG;
            goto Func_End;
        }
        memset(cArgv, 0x00, sizeof(char *) * argc);
    } else {
        iRet = RET_NG;
        goto Func_End;
    }
    for (i = 0; i < argc; i++) {
        if (utf8_flg == CDMTR_TRUE) {
            codepage = ENCODING_UTF_8;
        } else {
            codepage = ENCODING_SYSTEM;
        }
        i_result = CDMtrLibUtilUNICODEtoMBCS(&cArgv[i], &(argv[i]), codepage);
        if (i_result != RET_OK) {
            iRet = RET_NG;
            goto Func_End;
        }
        if (wcscmp(argv[i], L"--file_path") == 0 ||
                wcscmp(argv[i], L"--file_name") == 0) {
            utf8_flg = CDMTR_TRUE;
        } else {
            utf8_flg = CDMTR_FALSE;
        }
    }
    memset(&strParamInfo, 0x00, sizeof(PARAM_INFO_T));
    strParamInfo.iSkipFlg = CDMTR_TRUE;
    strParamInfo.iQrtFlg = CDMTR_TRUE;
    strParamInfo.iOutFmtFlg = CDMTR_FALSE;
    strParamInfo.pcFuncOpt = (_TCHAR *)malloc( (_tcslen(FUNC_OPTION_PARAM) + 1) * sizeof(_TCHAR) );
    if (strParamInfo.pcFuncOpt == NULL) {
        goto Func_End;
    }
    _tcscpy(strParamInfo.pcFuncOpt, FUNC_OPTION_PARAM);
    CDMtrLibOptInitGetParameter(stOptionArray, sizeof(stOptionArray) / sizeof(OptionInfo));
    while (OPTION_END != (iChkRes = CDMtrLibOptGetParameter(argc, cArgv, &pcParamStr))) {
        switch (iChkRes) {
        case OPTION_ERROR:
        case OPTION_NOINIT:
            iRet = RET_NG;
            goto Func_End;
        case OPTION_NOOPTION:
            strParamInfo.pcFilepath = pcParamStr;
            break;
        case OPTION_LANG_NAME:
            strParamInfo.pcLangName = pcParamStr;
            break;
        case OPTION_LANG_TYPE:
            strParamInfo.iLangType = atoi(pcParamStr);
            break;
        case OPTION_FILE_PATH:
            strParamInfo.pcDirPath = pcParamStr;
            break;
        case OPTION_FILE_NAME:
            strParamInfo.pcFName = pcParamStr;
            break;
        case OPTION_METRIC_FILE_PATH:
            strParamInfo.pcTemp = pcParamStr;
            break;
        case OPTION_METRIC_SKIP:
            if (_tcsicmp(pcParamStr, _T("old")) == 0) {
                strParamInfo.iSkipFlg = CDMTR_FALSE;
            } else {
                strParamInfo.iSkipFlg = CDMTR_TRUE;
            }
            break;
        case OPTION_ERROR_OUTPUT:
            strParamInfo.pcErrorOutput = pcParamStr;
            break;
        case OPTION_FUNCTION_OPT:
            if (_tcsicmp(pcParamStr, FUNC_OPTION_NONE) == 0) {
                free(strParamInfo.pcFuncOpt);
                strParamInfo.pcFuncOpt = (_TCHAR *)malloc( (_tcslen(FUNC_OPTION_NONE) + 1) * sizeof(_TCHAR) );
                if (strParamInfo.pcFuncOpt == NULL) {
                    goto Func_End;
                }
                _tcscpy(strParamInfo.pcFuncOpt, FUNC_OPTION_NONE);
            } else {
            }
            break;
        case OPTION_QRT_FLG:
            if (_tcsicmp(pcParamStr, _T("none")) == 0 ) {
                strParamInfo.iQrtFlg = CDMTR_FALSE;
            } else {
                strParamInfo.iQrtFlg = CDMTR_TRUE;
            }
            break;
        case OPTION_DEBUG2:
            strParamInfo.debug_mode = FLAG_ON;
            break;
        case OPTION_OUT_FORMAT:
            if (_tcsicmp(pcParamStr, OUT_FORMAT_FUNC) == 0 ) {
                strParamInfo.iOutFmtFlg = CDMTR_TRUE;
            } else {
                strParamInfo.iOutFmtFlg = CDMTR_FALSE;
            }
            break;
        default:
            break;
        }
    }
    CDMtrLibOptFinishGetParameter();
    if ((strParamInfo.pcErrorOutput == NULL) ||
            (strlen(strParamInfo.pcErrorOutput) == 0)) {
        strParamInfo.errfile = stdout;
    } else {
        errfile = fopen(strParamInfo.pcErrorOutput, "w");
        if (errfile != NULL) {
            strParamInfo.errfile = errfile;
        } else {
            strParamInfo.errfile = stdout;
        }
    }
    if ((strParamInfo.pcLangName == NULL)
            || (strParamInfo.pcDirPath == NULL)
            || (strParamInfo.pcFName == NULL)
            || (strParamInfo.pcTemp == NULL)
            || (strParamInfo.pcFuncOpt == NULL)
            || (strlen(strParamInfo.pcLangName) == 0)
            || (strlen(strParamInfo.pcDirPath) == 0)
            || (strlen(strParamInfo.pcFName) == 0)
            || (strlen(strParamInfo.pcTemp) == 0)
            || (strlen(strParamInfo.pcFuncOpt) == 0)
       ) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000603); //* MSG_LOG_ID_C_00000603
        iRet = RET_NG;
        goto Func_End;
    }
    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000604); //* MSG_LOG_ID_C_00000604
    i_result = cdmtrSpsMain(
                   strParamInfo.debug_mode,
                   strParamInfo.pcFilepath,
                   strParamInfo.pcLangName,
                   strParamInfo.pcDirPath,
                   strParamInfo.pcFName,
                   strParamInfo.pcTemp,
                   strParamInfo.iSkipFlg,
                   strParamInfo.pcFuncOpt,
                   strParamInfo.iQrtFlg,
                   strParamInfo.iLangType,
                   strParamInfo.iOutFmtFlg);
    if (i_result != RET_OK) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000605); //* MSG_LOG_ID_C_00000605
        iRet = RET_NG;
        goto Func_End;
    }
    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000606); //* MSG_LOG_ID_C_00000606
Func_End:
    if (errfile != NULL) {
        fclose(errfile);
    }
    free(strParamInfo.pcFilepath);
    free(strParamInfo.pcLangName);
    free(strParamInfo.pcDirPath);
    free(strParamInfo.pcFName);
    free(strParamInfo.pcTemp);
    free(strParamInfo.pcFuncOpt);
    if (cArgv != NULL) {
        for (i = 0; i < argc; i++) {
            if (cArgv[i] != NULL) {
                free(cArgv[i]);
                cArgv[i] = NULL;
            }
        }
        free(cArgv);
        cArgv = NULL;
    }
    return iRet;
}

/*!
    @brief          ȈՃp[T[C֐
*/
int cdmtrSpsMain(int debug_mode, char *filepath, char *pcLangName, char *pcDirPath, char *pcFName, char *pcTemp, int iSkipFlg, char *strFuncOpt, int iQrtFlg, int iLangType, int iOutFmtFlg)
{
    int iRet = SPSIF_RET_NG;
    iSpsInfo_Cnt_Max = MAX_BUF_ARRAY;
    sps_info = (SpsCntInfo *)malloc(sizeof(SpsCntInfo) * iSpsInfo_Cnt_Max);
    if (sps_info == NULL) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000607); //* MSG_LOG_ID_C_00000607
        goto Func_End;
    }
    memset(sps_info, 0x00, sizeof(SpsCntInfo) * iSpsInfo_Cnt_Max);
    memset(&sps_fileinfo, 0x00, sizeof(sps_fileinfo));
    memset(&sps_data_info, 0x00, sizeof(sps_data_info));
    memset(&M_method, 0x00, sizeof(M_method));
    M_method.pefTokenString = eventTokenString;
    M_method.pefLineBreak = eventLineBreak;
    M_method.pefLineSkip = eventLineSkip;
    M_method.pefComment = eventComment;
    M_method.pefMacro = eventMacro;
    M_method.pefFileEof = eventFileEof;
    M_method.pefStatement = eventStatement;
    M_method.pefFuncCandidate = eventFuncCandidate;
    M_method.pefFuncToStatement = eventFuncToStatement;
    M_method.pefFunction = eventFunction;
    M_method.pefFuncEnd = eventFuncEnd;
    M_method.pefBranchCandidate = eventBranchCandidate;
    M_method.pefBranchStatement = eventBranchStatement;
    M_method.pefBraceNest = eventBraceNest;
    M_method.pefParenNest = eventParenNest;
    memset(&S_method, 0x00, sizeof(S_method));
    S_method.psfGetFunctionStartFlag = GetFunctionStartFlag;
    S_method.psfGetTargetLangType = GetTargetLangType;
    iSpsDebug = debug_mode;
    giQrt_Out_Flg_ext = iQrtFlg;
    if (filepath == NULL) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000608); //* MSG_LOG_ID_C_00000608
        return SPSIF_RET_NG;
    } else {
        if (strlen(filepath) >= MAX_FILE_NAME) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000609); //* MSG_LOG_ID_C_00000609
            iRet = SPSIF_RET_NG;
            goto Func_End;
        }
        cdmtrSpsStrcpyToArray(SourceFile, filepath, MAX_FILE_NAME);
    }
    sps_fileinfo.sps_LangName = (char *)pcLangName;
    sps_fileinfo.sps_DirPath = (char *)pcDirPath;
    sps_fileinfo.sps_FName = (char *)pcFName;
    sps_fileinfo.sps_pcTemp = (char *)pcTemp;
    sps_fileinfo.sps_SkipFlag = iSkipFlg;
    sps_fileinfo.sps_FuncOpt = strFuncOpt;
    sps_fileinfo.sps_QrtFlag = iQrtFlg;
    sps_fileinfo.iLangType = iLangType;
    cdmtrSpsIfaceEventFuncSet(&M_method);
    cdmtrSpsIfaceSettingFuncSet(&S_method);
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000610); //* MSG_LOG_ID_C_00000610
    }
    cdmtrSpsIfaceInit();
    if (cdmtrSpsIfaceParses(SourceFile) != SPSIF_RET_OK) {
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000611); //* MSG_LOG_ID_C_00000611
        }
        iRet = SPSIF_RET_NG;
        goto Func_End;
    } else {
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000612); //* MSG_LOG_ID_C_00000612
        }
        if (cdmtrSpsCalc() == SPSIF_RET_OK) {
            iRet = SPSIF_RET_OK;
        } else {
            iRet = SPSIF_RET_NG;
        }
    }
Func_End:
    cdmtrSpsIfaceInit();
    if (sps_info != NULL) {
        free(sps_info);
        sps_info = NULL;
        iSpsInfo_Cnt_Max = 0;
    }
    memset(&sps_fileinfo, 0x00, sizeof(sps_fileinfo));
    memset(&sps_data_info, 0x00, sizeof(sps_data_info));
    return iRet;
}

/*!
    @brief          vʂ̂܂Ƃߏ
*/
int cdmtrSpsCalc()
{
    FILE *fp = NULL;
    if ((fp = fopen(sps_fileinfo.sps_pcTemp, "w")) == NULL) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000613, sps_fileinfo.sps_pcTemp); //* MSG_LOG_ID_C_00000613
        return SPSIF_RET_NG;
    }
    if (strParamInfo.iOutFmtFlg == CDMTR_TRUE) {
        cdmtrSpsCalcFunc(sps_data_info.iSpsInfo_Cnt, fp);
    } else {
        cdmtrSpsCalcStruct(sps_data_info.iSpsInfo_Cnt, fp);
    }
    fclose(fp);
    return SPSIF_RET_OK;
}

/*!
    @brief          R[h\gNXvʂ̂܂Ƃߏ
*/
void cdmtrSpsCalcStruct(int iCnt, FILE *pFile)
{
    int ii;
    if (iCnt != 0) {
        for (ii = 0; ii < iCnt; ii++) {
            sps_info[ii].sps_floc = sps_info[ii].sps_eloc + sps_info[ii].sps_cloc + sps_info[ii].sps_bloc + sps_info[ii].sps_skip;
            if (strlen(sps_fileinfo.sps_DirPath) > 0) {
                fprintf(pFile, "\"%s\",", sps_fileinfo.sps_DirPath);
            } else {
                fprintf(pFile, ",");
            }
            if (strlen(sps_fileinfo.sps_FName) > 0) {
                fprintf(pFile, "\"%s\",", sps_fileinfo.sps_FName);
            } else {
                fprintf(pFile, ",");
            }
            if (strlen(sps_info[ii].method_name) > 0) {
                if (sps_fileinfo.sps_FuncOpt != NULL &&
                        strcmp(sps_fileinfo.sps_FuncOpt, "none") == 0) {
                    fprintf(pFile, "\"%s\",", sps_info[ii].method_name);
                } else {
                    fprintf(pFile, "\"%s%s\",", sps_info[ii].method_name, sps_info[ii].param_name);
                }
            } else {
                fprintf(pFile, ",");
            }
            fprintf(pFile, "%s,", sps_fileinfo.sps_LangName);
            fprintf(pFile, "%d,", sps_info[ii].sps_floc);
            fprintf(pFile, "%d,", sps_info[ii].sps_eloc);
            fprintf(pFile, "%d,", sps_info[ii].sps_cloc);
            fprintf(pFile, "%d,", sps_info[ii].sps_bloc);
            fprintf(pFile, "%d,", sps_info[ii].sps_cyclo + 1);
            fprintf(pFile, "%d,", sps_info[ii].sps_cyclowc + 1);
            fprintf(pFile, "%d,", sps_info[ii].sps_branchs + 1);
            fprintf(pFile, "-,");
            fprintf(pFile, "%d,", sps_info[ii].sps_nests);
            fprintf(pFile, "-,");
            fprintf(pFile, "-,");
            fprintf(pFile, "-,");
            fprintf(pFile, "-");
            fprintf(pFile, "\n");
            if (iSpsDebug == FLAG_ON) {
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000614, ii + 1, sps_data_info.iSpsInfo_Cnt); //* MSG_LOG_ID_C_00000614
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000615, sps_fileinfo.sps_DirPath); //* MSG_LOG_ID_C_00000615
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000616, sps_fileinfo.sps_FName); //* MSG_LOG_ID_C_00000616
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000617, sps_fileinfo.sps_LangName); //* MSG_LOG_ID_C_00000617
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000618, sps_info[ii].method_name); //* MSG_LOG_ID_C_00000618
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000619, sps_info[ii].sps_line_from); //* MSG_LOG_ID_C_00000619
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000620, sps_info[ii].sps_line_to); //* MSG_LOG_ID_C_00000620
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000621, sps_info[ii].sps_floc); //* MSG_LOG_ID_C_00000621
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000622, sps_info[ii].sps_eloc); //* MSG_LOG_ID_C_00000622
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000623, sps_info[ii].sps_cloc); //* MSG_LOG_ID_C_00000623
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000624, sps_info[ii].sps_bloc); //* MSG_LOG_ID_C_00000624
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000625, sps_info[ii].sps_skip); //* MSG_LOG_ID_C_00000625
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000626, sps_info[ii].sps_cyclo + 1); //* MSG_LOG_ID_C_00000626
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000627, sps_info[ii].sps_cyclowc + 1); //* MSG_LOG_ID_C_00000627
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000628, sps_info[ii].sps_branchs + 1); //* MSG_LOG_ID_C_00000628
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000629); //* MSG_LOG_ID_C_00000629
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000630, sps_info[ii].sps_nests); //* MSG_LOG_ID_C_00000630
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000631, sps_info[ii].sps_evalr); //* MSG_LOG_ID_C_00000631
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000632, sps_info[ii].sps_evals); //* MSG_LOG_ID_C_00000632
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000633, sps_info[ii].sps_avals); //* MSG_LOG_ID_C_00000633
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000634, sps_info[ii].sps_parms); //* MSG_LOG_ID_C_00000634
                fprintf(strParamInfo.errfile, "\n"); //* MSG_LOG_ID_C_00000635
            }
        }
    } else {
        if (strlen(sps_fileinfo.sps_DirPath) > 0) {
            fprintf(pFile, "\"%s\",", sps_fileinfo.sps_DirPath);
        } else {
            fprintf(pFile, ",");
        }
        if (strlen(sps_fileinfo.sps_FName) > 0) {
            fprintf(pFile, "\"%s\",", sps_fileinfo.sps_FName);
        } else {
            fprintf(pFile, ",");
        }
        fprintf(pFile, "-,");
        fprintf(pFile, "%s,", sps_fileinfo.sps_LangName);
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-,");
        fprintf(pFile, "-");
        fprintf(pFile, "\n");
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000636); //* MSG_LOG_ID_C_00000636
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000637, sps_fileinfo.sps_DirPath); //* MSG_LOG_ID_C_00000637
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000638, sps_fileinfo.sps_FName); //* MSG_LOG_ID_C_00000638
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000639, sps_fileinfo.sps_LangName); //* MSG_LOG_ID_C_00000639
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000640); //* MSG_LOG_ID_C_00000640
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000641); //* MSG_LOG_ID_C_00000641
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000642); //* MSG_LOG_ID_C_00000642
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000643); //* MSG_LOG_ID_C_00000643
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000644); //* MSG_LOG_ID_C_00000644
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000645); //* MSG_LOG_ID_C_00000645
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000646); //* MSG_LOG_ID_C_00000646
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000647); //* MSG_LOG_ID_C_00000647
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000648); //* MSG_LOG_ID_C_00000648
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000649); //* MSG_LOG_ID_C_00000649
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000650); //* MSG_LOG_ID_C_00000650
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000651); //* MSG_LOG_ID_C_00000651
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000652); //* MSG_LOG_ID_C_00000652
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000653); //* MSG_LOG_ID_C_00000653
            fprintf(strParamInfo.errfile, "\n"); //* MSG_LOG_ID_C_00000654
        }
    }
}

/*!
    @brief          ֐Pʂ̍gNXp̊֐o͏ (V1.11 PARSER VKǉ֐j
*/
void cdmtrSpsCalcFunc(int iCnt, FILE *pFile)
{
    int ii;
    if (iCnt != 0) {
        for (ii = 0; ii < iCnt; ii++) {
            if (strlen(sps_info[ii].method_name) <= 0) {
                continue;
            }
            fprintf(pFile, "%d,", ii + 1);
            fprintf(pFile, "%d,", sps_info[ii].sps_line_from);
            fprintf(pFile, "%d,", sps_info[ii].sps_line_to);
            if (strlen(sps_info[ii].method_name) > 0) {
                fprintf(pFile, "\"%s\",", sps_info[ii].method_name);
            } else {
                fprintf(pFile, "\"\",");
            }
            if (strlen(sps_info[ii].param_name) > 0) {
                fprintf(pFile, "\"%s\"", sps_info[ii].param_name);
            } else {
                fprintf(pFile, "\"()\"");
            }
            fprintf(pFile, "\n");
            if (iSpsDebug == FLAG_ON) {
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000655, ii + 1, sps_data_info.iSpsInfo_Cnt); //* MSG_LOG_ID_C_00000655
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000656, sps_fileinfo.sps_DirPath); //* MSG_LOG_ID_C_00000656
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000657, sps_fileinfo.sps_FName); //* MSG_LOG_ID_C_00000657
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000658, sps_fileinfo.sps_LangName); //* MSG_LOG_ID_C_00000658
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000659, sps_info[ii].method_name); //* MSG_LOG_ID_C_00000659
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000660, sps_info[ii].sps_line_from); //* MSG_LOG_ID_C_00000660
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000661, sps_info[ii].sps_line_to); //* MSG_LOG_ID_C_00000661
                fprintf(strParamInfo.errfile, "\n"); //* MSG_LOG_ID_C_00000662
            }
        }
    } else {
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000663); //* MSG_LOG_ID_C_00000663
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000664, sps_fileinfo.sps_DirPath); //* MSG_LOG_ID_C_00000664
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000665, sps_fileinfo.sps_FName); //* MSG_LOG_ID_C_00000665
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000666, sps_fileinfo.sps_LangName); //* MSG_LOG_ID_C_00000666
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000667, "-"); //* MSG_LOG_ID_C_00000667
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000668); //* MSG_LOG_ID_C_00000668
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000669); //* MSG_LOG_ID_C_00000669
            fprintf(strParamInfo.errfile, "\n"); //* MSG_LOG_ID_C_00000670
        }
    }
}

/*!
    @brief          EnumyStructƏ񋤒ʏ
*/
void cdmtrSpsEnumStructLineInfoMemory(int line_to, int line_from, char *linestr)
{
    if (strncmp(linestr, TOKEN_ENUM, strlen(TOKEN_ENUM)) == 0) {
        sps_data_info.enum_info.line_from = line_from;
        sps_data_info.enum_info.line_to   = line_to;
    } else if (strncmp(linestr, TOKEN_STRUCT, strlen(TOKEN_STRUCT)) == 0) {
        sps_data_info.struct_info.line_from = line_from;
        sps_data_info.struct_info.line_to   = line_to;
    }
}

/*!
    @brief          TOKENPʂɃR[BiLexer Callj
*/
void eventTokenString(int token_id, int line_from, int colum_from, int line_to, int colum_to, char *token_text)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000671, line_from, colum_from, line_to, colum_to, token_id, token_text); //* MSG_LOG_ID_C_00000671
    }
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    if (sps_data_info.SWITCH_Flag == FLAG_ON) {
        if ((token_id == SPSIF_CASE)) {
            ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_branchs;
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000672, sps_info[sps_data_info.iSpsInfo_Cnt].sps_branchs); //* MSG_LOG_ID_C_00000672
#endif
            sps_info[sps_data_info.iSpsInfo_Cnt].sps_cyclowc++;
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000673,//* MSG_LOG_ID_C_00000673
                    sps_info[sps_data_info.iSpsInfo_Cnt].sps_cyclowc);
#endif
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000674, sps_data_info.LineNoBuf4CLOC); //* MSG_LOG_ID_C_00000674
#endif
            sps_data_info.LineNoBuf4CLOC = line_to;
            sps_data_info.CASE_Flag = FLAG_ON;
        } else if (token_id == SPSIF_DEFAULT) {
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000675, __LINE__, //* MSG_LOG_ID_C_00000675
                    sps_data_info.LineNoBuf4CLOC);
#endif
            sps_data_info.LineNoBuf4CLOC = line_to;
            sps_data_info.CASE_Flag = FLAG_ON;
        }
        if (token_id == SPSIF_BREAK) {
            sps_data_info.CASE_Flag = FLAG_OFF;
        }
    }
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief          sPʂɃR[BiLexer Callj
*/
void eventLineBreak(int break_id, int line_from, int colum_from, int line_to, int colum_to)
{
    switch (break_id) {
    case 0 :
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000676, line_from, colum_from, line_to, colum_to); //* MSG_LOG_ID_C_00000676
        }
        sps_data_info.LineBreak_Flag = FLAG_ON;
        break;
    case 1 :
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000677, line_from, colum_from, line_to, colum_to); //* MSG_LOG_ID_C_00000677
        }
        if (sps_data_info.Func_start_Flag == FLAG_ON) {
            ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_bloc;
            memset(&sps_data_info.bb, 0x00, sizeof(sps_data_info.bb));
            sps_data_info.bb.line_from = line_from;
            sps_data_info.bb.line_to   = line_to;
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000678, sps_data_info.bb.line_from, sps_data_info.bb.line_to); //* MSG_LOG_ID_C_00000678
#endif
        }
        break;
    case 2 :
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000679, line_from, colum_from, line_to, colum_to); //* MSG_LOG_ID_C_00000679
        }
        call_setParamState();
        break;
    }
    sps_data_info.LineNoBuf = sps_data_info.LineNoBuf4CLOC = line_to;
}

/*!
    @brief          }NXLbvsPʂɃR[BiLexer Callj
*/
void eventLineSkip(int line_from, int colum_from, int line_to, int colum_to, char *skipline)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000680, line_from, colum_from, line_to, colum_to, skipline); //* MSG_LOG_ID_C_00000680
    }
    if (sps_data_info.Func_start_Flag != FLAG_ON) {
        return;
    }
    if (sps_fileinfo.sps_SkipFlag == FLAG_OFF) {
        sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc += (line_to - line_from) + 1 ;
        sps_data_info.ELOC_Cnt_LineNo = line_to;
    } else {
        sps_info[sps_data_info.iSpsInfo_Cnt].sps_skip += (line_to - line_from) + 1 ;
        sps_data_info.SKIP_Cnt_LineNo = line_to;
    }
    sps_data_info.LineStrBuf = (char *)skipline;
    sps_data_info.LineBreak_Flag = FLAG_OFF;
    sps_data_info.LineNoBuf = sps_data_info.LineNoBuf4CLOC = line_to;
}

/*!
    @brief          RgPʂɃR[BiLexer Callj
*/
void eventComment(int cmt_type, int cmtno, int linenum, int line_from, int colum_from, int line_to, int colum_to, char *cmtline)
{
    int i;
    int cloc_plus_num;
    int **ppStackTokenInfo = NULL;
    int iStackTokenCnt = 0;
    int iTokenFlag = FLAG_OFF;
    if (cmt_type == 0) {
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000681 //* MSG_LOG_ID_C_00000681
                    , cmt_type , cmtno, linenum, line_from, colum_from, line_to, colum_to, cmtline);
        }
    } else {
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000682 //* MSG_LOG_ID_C_00000682
                    , cmt_type , cmtno, linenum, line_from, colum_from, line_to, colum_to, cmtline);
        }
    }
    GetStackTokens(&ppStackTokenInfo, &iStackTokenCnt);
    if (ppStackTokenInfo != NULL && iStackTokenCnt > 0 ) {
        iTokenFlag = FLAG_ON;
    }
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    if (sps_data_info.CLOC_Cnt_LineNo == line_from) {
        return;
    }
    cloc_plus_num = 0;
    if (linenum > 1) {
        if (0 < (line_to - sps_data_info.LineNoBuf4CLOC)) {
            for (i = 1; i <= (line_to - sps_data_info.LineNoBuf4CLOC); i++) {
                if (sps_data_info.ELOC_Cnt_LineNo < sps_data_info.LineNoBuf4CLOC + i) {
                    if (iTokenFlag == FLAG_OFF || ppStackTokenInfo[iStackTokenCnt - 1][0] < sps_data_info.LineNoBuf4CLOC + i) {
                        ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_cloc;
                        ++cloc_plus_num;
                        sps_data_info.CLOC_Cnt_LineNo = line_to;
#ifdef DEBUG
                        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000683, sps_info[sps_data_info.iSpsInfo_Cnt].sps_cloc); //* MSG_LOG_ID_C_00000683
#endif
                    }
                }
            }
        } else {
            if ((sps_data_info.LineNoBuf4CLOC < line_from) &&
                    (sps_data_info.ELOC_Cnt_LineNo < line_from)) {
                if (iTokenFlag == FLAG_OFF || ppStackTokenInfo[iStackTokenCnt - 1][0] < line_from) {
                    sps_info[sps_data_info.iSpsInfo_Cnt].sps_cloc += linenum;
                    cloc_plus_num = linenum;
                    sps_data_info.CLOC_Cnt_LineNo = line_to;
#ifdef DEBUG
                    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000684, sps_data_info.LineNoBuf4CLOC, line_from); //* MSG_LOG_ID_C_00000684
                    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000685, sps_info[sps_data_info.iSpsInfo_Cnt].sps_cloc); //* MSG_LOG_ID_C_00000685
#endif
                }
            }
        }
    } else {
        if ((sps_data_info.LineNoBuf4CLOC < line_from) &&
                (sps_data_info.ELOC_Cnt_LineNo < line_from)) {
            if (iTokenFlag == FLAG_OFF || ppStackTokenInfo[iStackTokenCnt - 1][0] < line_from) {
                ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_cloc;
                ++cloc_plus_num;
                sps_data_info.CLOC_Cnt_LineNo = line_to;
#ifdef DEBUG
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000686, sps_data_info.LineNoBuf4CLOC, line_from); //* MSG_LOG_ID_C_00000686
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000687, sps_info[sps_data_info.iSpsInfo_Cnt].sps_cloc); //* MSG_LOG_ID_C_00000687
#endif
            }
        }
    }
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000688 //* MSG_LOG_ID_C_00000688
                , cmt_type , sps_info[sps_data_info.iSpsInfo_Cnt].sps_cloc);
    }
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief          }NLqPʂɃR[BiLexer Callj
*/
void eventMacro(int pre_type, int preno, int linenum, int line_from, int colum_from,
                int line_to, int colum_to, int pre_nest, int pre_pars, int *macronest, int cntmacronest, int iSkipLine, char *keyname, char *keyparam)
{
    if (iSpsDebug == FLAG_ON) {
        //* MSG_LOG_ID_C_00000689
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000689 \
                , preno, linenum, line_from, colum_from, line_to, colum_to, pre_nest, pre_pars, pre_type, keyname, keyparam);
    }
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    switch (pre_type) {
    case SPSIF_TYPE_IF0:
    case SPSIF_TYPE_IFNDEF:
    case SPSIF_TYPE_ELIF:
    case SPSIF_TYPE_ELSE:
    case SPSIF_TYPE_IF:
    case SPSIF_TYPE_IFDEF:
        if (pre_pars == SPSIF_MACRO_TRUE) {
        } else {
            sps_data_info.PREP_IF_ZERO_Flag = FLAG_ON;
            if (iSkipLine != line_from) {
                sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc += (line_to - line_from) + 1 ;
                break;
            }
        }
    default:
        if ( (cntmacronest > 0 ) && (macronest[cntmacronest - 1] != SPSIF_MACRO_TRUE )) {
            if (sps_fileinfo.sps_SkipFlag == FLAG_OFF) {
                sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc += (line_to - line_from) + 1 ;
            } else {
                sps_info[sps_data_info.iSpsInfo_Cnt].sps_skip += (line_to - line_from) + 1 ;
            }
        } else {
            sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc += (line_to - line_from) + 1 ;
            break;
        }
    }
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000690 //* MSG_LOG_ID_C_00000690
                , line_from , line_to, sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc, sps_info[sps_data_info.iSpsInfo_Cnt].sps_skip);
    }
    sps_data_info.LineBreak_Flag = FLAG_OFF;
    sps_data_info.LineNoBuf = sps_data_info.LineNoBuf4CLOC = line_to;
}

/*!
    @brief          ͏IɃR[BiLexer Callj
*/
void eventFileEof(int linenum)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000691, linenum); //* MSG_LOG_ID_C_00000691
    }
}

/*!
    @brief          Xe[ggsPʂɃR[BiParses Callj
*/
void eventStatement(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, char *statement_text)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000692, line_from, line_to, statement_text); //* MSG_LOG_ID_C_00000692
    }
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    CDMtrSpsCountElocLine(StackTokenInfo, iStackCnt, line_to, FLAG_OFF, 0);
    sps_data_info.LineStrBuf = (char *)statement_text;
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief  ֐sPʂɃR[BiParses Callj

*/
void eventFuncCandidate(int line_from, int line_to, char *funcname, int cntparam, char *funcline)
{
    if ((funcname == NULL) || (funcline == NULL)) {
        return;
    }
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000693 //* MSG_LOG_ID_C_00000693
                , line_from, line_to, funcname, cntparam, funcline);
    }
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief  ֐₩Xe[ggڍsŃR[BiParses Callj
*/
void eventFuncToStatement(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, char *funcname, char *statement_text)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000694 //* MSG_LOG_ID_C_00000694
                , line_from, line_to, funcname, statement_text);
    }
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    CDMtrSpsCountElocLine(StackTokenInfo, iStackCnt, line_to, FLAG_OFF, 0);
    sps_data_info.LineStrBuf = (char *)statement_text;
    sps_data_info.Func_decide_Flag = FLAG_OFF;
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief  ֐sPʂɃR[BiParses Callj
*/
void eventFunction(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, char *funcname, char *param_name, int cntparam, char *funcline, int ifunc_name_line)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, "\n"); //* MSG_LOG_ID_C_00000695
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000696 //* MSG_LOG_ID_C_00000696
                , line_from, line_to, funcname, cntparam, funcline);
    }
    cdmtrSpsStrcpyToArray(sps_info[sps_data_info.iSpsInfo_Cnt].method_name, funcname,
                          MAX_NUMARY);
    cdmtrSpsStrcpyToArray(sps_info[sps_data_info.iSpsInfo_Cnt].param_name, param_name,
                          MAX_BUF_ARRAY);
    sps_info[sps_data_info.iSpsInfo_Cnt].sps_line_from = line_from ;
    CDMtrSpsCountElocLine(StackTokenInfo, iStackCnt, line_to, FLAG_ON, ifunc_name_line);
    sps_data_info.Func_decide_Flag = FLAG_OFF;
    sps_data_info.Func_start_Flag = FLAG_ON;
    sps_data_info.LineBreak_Flag = FLAG_OFF;
    sps_data_info.LineNoBuf = sps_data_info.LineNoBuf4CLOC = line_to;
}

/*!
    @brief  ֐IɃR[BiParses Callj
*/
void eventFuncEnd(int **StackTokenInfo, int iStackCnt, int line_from, int line_to, int cntmaxnests)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000697, line_from, line_to, cntmaxnests); //* MSG_LOG_ID_C_00000697
        fprintf(strParamInfo.errfile, "\n"); //* MSG_LOG_ID_C_00000698
    }
    sps_info[sps_data_info.iSpsInfo_Cnt].sps_line_to = line_to ;
    CDMtrSpsCountElocLine(StackTokenInfo, iStackCnt, line_to, FLAG_OFF, 0);
    sps_data_info.Func_start_Flag = FLAG_OFF;
    sps_data_info.SWITCH_Flag = FLAG_OFF;
    sps_info[sps_data_info.iSpsInfo_Cnt].sps_nests = cntmaxnests;
    ++sps_data_info.iSpsInfo_Cnt;
#ifdef DEBUG
    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000699, sps_data_info.iSpsInfo_Cnt); //* MSG_LOG_ID_C_00000699
#endif
    sps_data_info.LineBreak_Flag = FLAG_OFF;
    if (sps_data_info.iSpsInfo_Cnt >= iSpsInfo_Cnt_Max) {
        SpsCntInfo *pTmp = NULL;
        pTmp = (SpsCntInfo *) realloc(sps_info, sizeof(SpsCntInfo) * (iSpsInfo_Cnt_Max * 2));
        if (pTmp == NULL) {
            --sps_data_info.iSpsInfo_Cnt;
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000700, //* MSG_LOG_ID_C_00000700
                    sps_info[sps_data_info.iSpsInfo_Cnt].method_name);
            memset(sps_info + sps_data_info.iSpsInfo_Cnt, 0x00, sizeof(SpsCntInfo));
            return;
        }
        memset(pTmp + iSpsInfo_Cnt_Max,
               0x00, sizeof(SpsCntInfo) * iSpsInfo_Cnt_Max);
        sps_info = pTmp;
        iSpsInfo_Cnt_Max = iSpsInfo_Cnt_Max * 2;
    }
}

/*!
    @brief  򖽗ߌPʂɃR[BiParses Callj
*/
void eventBranchCandidate(int branch_type, int line_from, int line_to, char *statement_text)
{
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000701 //* MSG_LOG_ID_C_00000701
                , branch_type, line_from, line_to, statement_text);
    }
    if ((branch_type == KEYWOR_CASE_IF) ||
            (branch_type == KEYWOR_CASE_FOR) ||
            (branch_type == KEYWOR_CASE_WHILE) ||
            (branch_type == KEYWOR_CASE_SWITCH)) {
        ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_cyclo;
#ifdef DEBUG
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000702, sps_info[sps_data_info.iSpsInfo_Cnt].sps_cyclo); //* MSG_LOG_ID_C_00000702
#endif
        if (branch_type == KEYWOR_CASE_SWITCH) {
            sps_data_info.SWITCH_Flag = FLAG_ON;
        } else {
            sps_info[sps_data_info.iSpsInfo_Cnt].sps_cyclowc++;
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000703, __LINE__, //* MSG_LOG_ID_C_00000703
                    sps_info[sps_data_info.iSpsInfo_Cnt].sps_cyclowc);
#endif
        }
    }
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief          򖽗ߒPʂɃR[BiParses Callj
*/
void eventBranchStatement(int branch_type, int **StackTokenInfo, int iStackCnt, int line_from, int line_to, int cntcondition, char *statement_text)
{
    int i;
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000704 //* MSG_LOG_ID_C_00000704
                , branch_type, line_from, line_to, cntcondition, statement_text);
    }
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    if ((branch_type == KEYWOR_CASE_IF) ||
            (branch_type == KEYWOR_CASE_WHILE) ||
            (branch_type == KEYWOR_CASE_FOR) ||
            (branch_type == KEYWOR_CASE_SWITCH)) {
        for (i = 0; i < cntcondition; i++) {
            ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_branchs;
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000705, sps_info[sps_data_info.iSpsInfo_Cnt].sps_branchs); //* MSG_LOG_ID_C_00000705
#endif
        }
    }
    if (sps_data_info.iBraceCloseLineNo == line_from) {
        if (branch_type == KEYWOR_CASE_WHILE) {
            sps_data_info.DO_WHILE_Flag = FLAG_OFF;
#ifdef DEBUG
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000706, statement_text); //* MSG_LOG_ID_C_00000706
#endif
        }
    } else {
        CDMtrSpsCountElocLine(StackTokenInfo, iStackCnt, line_to, FLAG_OFF, 0);
        sps_data_info.LineNoBuf = sps_data_info.LineNoBuf4CLOC = line_to;
    }
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief lXgɃR[BiParses Callj
*/
void eventBraceNest(int brace_type, int **StackTokenInfo, int iStackCnt, int line_from, int line_to, int cntinner, int cntouter, int cntmaxinner, int cntmaxouter, char *token_text)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000707 //* MSG_LOG_ID_C_00000707
                , brace_type, line_from, line_to, cntinner, cntmaxinner, cntouter, cntmaxouter, token_text);
    }
    if (sps_data_info.Func_start_Flag == FLAG_OFF) {
        return;
    }
    if (strncmp(token_text, TOKEN_DO, strlen(TOKEN_DO)) == 0) {
        sps_data_info.DO_WHILE_Flag = FLAG_ON;
    }
    if (sps_data_info.SWITCH_Flag == FLAG_ON) {
        if (strncmp(token_text, TOKEN_BRACE_OPEN, strlen(TOKEN_BRACE_OPEN)) == 0) {
            if (sps_data_info.iSWITCH_NEST_Flag == FLAG_OFF) {
                sps_data_info.iSWITCH_NEST_Flag = FLAG_ON;
                sps_data_info.iSWITCH_NEST_Cnt = cntinner - 1;
            }
        } else if (strncmp(token_text, TOKEN_BRACE_CLOSE, strlen(TOKEN_BRACE_CLOSE)) == 0) {
            if (sps_data_info.iSWITCH_NEST_Cnt == cntinner) {
                sps_data_info.iSWITCH_NEST_Flag = FLAG_OFF;
                sps_data_info.iSWITCH_NEST_Cnt = 0;
                sps_data_info.SWITCH_Flag = FLAG_OFF;
            }
        }
    }
    if (sps_data_info.ELOC_Cnt_LineNo < line_to) {
        CDMtrSpsCountElocLine(StackTokenInfo, iStackCnt, line_to, FLAG_OFF, 0);
    }
    sps_data_info.LineStrBuf = (char *)token_text;
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief  LqшLqł̃lXgɃR[BiParses Callj
*/
void eventParenNest(int paren_type, int line_from, int line_to, int cntfunc, int cntbranch, char *token_text)
{
    if (iSpsDebug == FLAG_ON) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000708 //* MSG_LOG_ID_C_00000708
                , paren_type, line_from, line_to, cntfunc, cntbranch, token_text);
    }
    sps_data_info.LineBreak_Flag = FLAG_OFF;
}

/*!
    @brief      Eloc̃JEgɃR[
*/
void CDMtrSpsCountElocLine(int **StackTokenInfo, int iStackCnt, int line_to, int iFuncCandinate, int iFuncNameLineNo)
{
    int iLineCntEnd;
    int line_from;
    int i;
    int iCPPMetrics;
    int iFindBraceNest;
    int iFindColon;
    int iFuncNameLine;
    int iflgSkipToken = FLAG_OFF;
    iLineCntEnd = sps_data_info.ELOC_Cnt_LineNo;
    iCPPMetrics = FLAG_OFF;
    iFindBraceNest = FLAG_OFF;
    iFindColon = FLAG_OFF;
    iFuncNameLine = StackTokenInfo[iFuncNameLineNo][0];
    if (sps_fileinfo.iLangType == SPS_LANG_TYPE_CPP) {
        iCPPMetrics = FLAG_ON;
    }
    iflgSkipToken = FLAG_OFF;
    for (i = 0; i < iStackCnt; i++) {
        line_from = StackTokenInfo[i][0];
        if (iFuncCandinate == FLAG_ON && iCPPMetrics == FLAG_ON) {
            if (StackTokenInfo[i][4] == SPSIF_CLASS ||
                    StackTokenInfo[i][4] == SPSIF_NAMESPACE ||
                    StackTokenInfo[i][4] == SPSIF_STRUCT ||
                    StackTokenInfo[i][4] == SPSIF_UNION
               ) {
                iFindBraceNest = FLAG_ON;
                continue;
            }
            if (StackTokenInfo[i][4] == SPSIF_PUBLIC ||
                    StackTokenInfo[i][4] == SPSIF_PROTECTED ||
                    StackTokenInfo[i][4] == SPSIF_PRIVATE
               ) {
                iFindColon = FLAG_ON;
            }
            if (iFindBraceNest == FLAG_ON && line_from < iFuncNameLine) {
                if (StackTokenInfo[i][4] == SPSIF_BRACEST) {
                    iFindBraceNest = FLAG_OFF;
                    continue;
                } else {
                    continue;
                }
            }
            if (iFindColon == FLAG_ON && line_from < iFuncNameLine) {
                if (StackTokenInfo[i][4] == SPSIF_COLON) {
                    iFindColon = FLAG_OFF;
                    continue;
                } else {
                    continue;
                }
            }
            if (iflgSkipToken == FLAG_OFF) {
                sps_info[sps_data_info.iSpsInfo_Cnt].sps_line_from = line_from ;
                iflgSkipToken = FLAG_ON ;
            }
        }
        if (iLineCntEnd  < line_from ) {
            ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc;
            if (iSpsDebug == FLAG_ON) {
                fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000709 //* MSG_LOG_ID_C_00000709
                        , iLineCntEnd, line_to, sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc);
            }
            iLineCntEnd = line_from;
        }
        if (iLineCntEnd == line_to) {
            break;
        }
    }
    if (iLineCntEnd != line_to) {
        ++sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc;
        if (iSpsDebug == FLAG_ON) {
            fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000710 //* MSG_LOG_ID_C_00000710
                    , iLineCntEnd, line_to, sps_info[sps_data_info.iSpsInfo_Cnt].sps_eloc);
        }
    }
    sps_data_info.ELOC_Cnt_LineNo = line_to;
}



/*!
    @brief      ֐ǂԂB
*/
int GetFunctionStartFlag(void)
{
    return sps_data_info.Func_start_Flag;
}

/*!
    @brief      ʂԂB
*/
int GetTargetLangType(void)
{
    return sps_fileinfo.iLangType;
}
