/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <stdlib.h>

#include <fcntl.h>
#include <io.h>
#include <Share.h>
#include <tchar.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "CDMtrStmtFile.h"
#include "CDMtrStmtLog.h"
#include "CDMtrMessage.h"


#if CDMTR_LOG_WHITE_BOX
#define CDMtrStmtBufferSize    (11)
#else
#define CDMtrStmtBufferSize    (1024 * 20)
#endif

int CDMtrStmt_flg_isEOF = CDMtrStmt_FILE_OFF;
int CDMtrStmt_flg_isTOL = CDMtrStmt_FILE_OFF;
int CDMtrStmt_flg_isEOL = CDMtrStmt_FILE_OFF;
int CDMtrStmt_flg_isTOF = CDMtrStmt_FILE_OFF;


long CDMtrStmt_lineNumber = 0L;


static int InputFileFD = -1;

static char LocalBuffer[CDMtrStmtBufferSize + 1];
static int iLocalBufferPos;


/*!
    @brief      t@CI[v
*/
int CDMtrStmtOpenInputFile(const _TCHAR *pcFileName)
{
    int iFD = 0;
    errno_t erno = 0;
    if (InputFileFD >= 0) {
        PASS_CHECK;
        return -1;
    }
#if defined(UNICODE) || defined(_UNICODE)
    erno = _tsopen_s(&iFD, pcFileName, O_RDONLY | _O_TEXT, _SH_DENYWR, S_IREAD);
    if ( erno != 0 ) {
        PASS_CHECK;
        return -1;
    }
#else
    iFD = open(pcFileName, O_RDONLY);
#endif
    if (iFD < 0) {
        PASS_CHECK;
        return -1;
    }
    InputFileFD = iFD;
    CDMtrStmt_flg_isEOF = CDMtrStmt_FILE_OFF;
    CDMtrStmt_flg_isTOL = CDMtrStmt_FILE_OFF;
    CDMtrStmt_flg_isEOL = CDMtrStmt_FILE_ON;
    CDMtrStmt_flg_isTOF = CDMtrStmt_FILE_ON;
    CDMtrStmt_lineNumber = 0L;
    memset((void *)LocalBuffer, 0, CDMtrStmtBufferSize + 1);
    iLocalBufferPos = 0;
    return 0;
}


/*!
    @brief      t@CN[Y
*/
void CDMtrStmtCloseInputFile()
{
    if (InputFileFD >= 0) {
        PASS_CHECK;
        _close(InputFileFD);
        InputFileFD = -1;
    }
    return;
}


/*!
    @brief          sǂݍ
    */
int CDMtrStmtReadOneLine(char *pcReadBuffer, int iBufSize)
{
    int iReadSize;
    int iPos = 0;
    iReadSize = 0;
    if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_ON) {
        PASS_CHECK;
        CDMtrStmt_lineNumber++;
        CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000714, CDMtrStmt_lineNumber);); //* MSG_LOG_ID_C_00000714
        CDMtrStmt_flg_isTOL = CDMtrStmt_FILE_ON;
    } else {
        PASS_CHECK;
        CDMtrStmt_flg_isTOL = CDMtrStmt_FILE_OFF;
    }
    CDMtrStmt_flg_isEOL = CDMtrStmt_FILE_OFF;
    iPos = 0;
    for ( ; ; ) {
        while (LocalBuffer[iLocalBufferPos] != '\0') {
            char  cNow;
            cNow = pcReadBuffer[iPos] = LocalBuffer[iLocalBufferPos];
            if (cNow == '\n') {
                PASS_CHECK;
                pcReadBuffer[iPos] = '\0';
                CDMtrStmt_flg_isEOL = CDMtrStmt_FILE_ON;
                break;
            }
            iPos++;
            iLocalBufferPos++;
            if (cNow == '\r') {
                iPos--;
            }
            if (iPos >= iBufSize) {
                PASS_CHECK;
                pcReadBuffer[iBufSize] = '\0';
                return iBufSize;
            }
        }
        if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_ON) {
            PASS_CHECK;
            iLocalBufferPos++;
            return iPos;
        } else {
            PASS_CHECK;
            memset((void *)LocalBuffer, 0, CDMtrStmtBufferSize + 1);
            iReadSize = _read(InputFileFD, (void *)LocalBuffer, CDMtrStmtBufferSize);
            if (iReadSize < 0) {
                PASS_CHECK;
                return -1;
            }
            iLocalBufferPos = 0;
            if (CDMtrStmt_flg_isTOF == CDMtrStmt_FILE_ON) {
                if (iReadSize >= 3) {
                    if ((unsigned char)LocalBuffer[0] == 239 &&
                            (unsigned char)LocalBuffer[1] == 187 &&
                            (unsigned char)LocalBuffer[2] == 191) {
                        iLocalBufferPos = 3;
                    }
                }
                CDMtrStmt_flg_isTOF = CDMtrStmt_FILE_OFF ;
            }
            if (iReadSize == 0 || (iLocalBufferPos == 3 && ((iReadSize - 3) == 0))) {
                PASS_CHECK;
                pcReadBuffer[iPos] = '\0';
                if (iPos == 0) {
                    CDMtrStmt_flg_isEOF = CDMtrStmt_FILE_ON;
                }
                CDMtrStmt_flg_isEOL = CDMtrStmt_FILE_ON;
                iLocalBufferPos = 0;
                return iPos;
            } else {
                PASS_CHECK;
                LocalBuffer[iReadSize] = '\0';
            }
        }
    }
}
