/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <tchar.h>

#include "CDMtrStmt.h"
#include "CDMtrStmtFile.h"
#include "CDMtrStmtLex.h"
#include "CDMtrStmtParse.h"
#include "CDMtrStmtLog.h"
#include "CDMtrMessage.h"

char CDMtrStmtWorkBuf[CDMtrStmtBufSize + 1];
int iPosWorkBuf = 0;

#if CDMTR_LOG_WHITE_BOX
#define TOKEN_BUF_LEN (17)
#else
#define TOKEN_BUF_LEN (1024)
#endif
static char *CDMtrStmtTokenBuff      = NULL;
static int   CDMtrStmtMaxTokenBufLen = 0;
static int   CDMtrStmtTokenLen       = 0;

struct cdmtrStmt_token_tag {
    char *pcTokenName;
    char *pcTokenDisp;
    char *pcTokenStr;
};

struct cdmtrStmt_prepro_tag {
    int iPreproType;
    char *pcPreproStr;
};

typedef enum {
    prepro_unknown = 0,
    prepro_include,

    prepro_end
} type_prepro;

struct cdmtrStmt_token_tag sCdmtrStmtTokenArray[] = {

    { "token_unknown",      "",         NULL },



    { "token_colon",        ":",        NULL },
    { "token_colon2",       "::",       NULL },
    { "token_eq",           "=",        NULL },
    { "token_eq2",          "==",       NULL },
    { "token_generic",      "<?",       NULL },


    { "token_atmark",       "@",        NULL },
    { "token_lbracket",     "[",        NULL },
    { "token_rbracket",     "]",        NULL },
    { "token_lparent",      "(",        NULL },
    { "token_rparent",      ")",        NULL },
    { "token_lcbrace",      "{",        NULL },
    { "token_rcbrace",      "}",        NULL },
    { "token_semicolon",    ";",        NULL },
    { "token_comma",        ",",        NULL },
    { "token_question",     "?",        NULL },
    { "token_operator",     NULL,       NULL },

    { "token_other_mark",   NULL,       NULL },


    { "token_string",       NULL,       NULL },
    { "token_character",    NULL,       NULL },


    { "token_struct",       NULL,       "struct" },
    { "token_class",        NULL,       "class" },
    { "token_union",        NULL,       "union" },
    { "token_interface",    NULL,       "interface" },
    { "token_enum",         NULL,       "enum" },
    { "token_for",          NULL,       "for" },
    { "token_while",        NULL,       "while" },
    { "token_foreach",      NULL,       "foreach"},
    { "token_switch",       NULL,       "switch" },
    { "token_try",          NULL,       "try" },
    { "token_catch",        NULL,       "catch" },
    { "token_finally",      NULL,       "finally" },
    { "token_case",         NULL,       "case" },
    { "token_default",      NULL,       "default" },
    { "token_public",       NULL,       "public" },
    { "token_private",      NULL,       "private" },
    { "token_protected",    NULL,       "protected" },
    { "token_if",           NULL,       "if" },
    { "token_else",         NULL,       "else" },
    { "token_do",           NULL,       "do" },
    { "token_lock",         NULL,       "lock"},
    { "token_delegate",     NULL,       "delegate"},
    { "token_using",        NULL,       "using"},
    { "token_fixed",        NULL,       "fixed"},
    { "token_this",         NULL,       "this"},
    { "token_goto",         NULL,       "goto"},



    { "token_is",           NULL,       "is" },
    { "token_as",           NULL,       "as" },


    { "token_other",        NULL,       NULL },


    { "token_end",          "",         NULL }
};

struct cdmtrStmt_prepro_tag sCdmtrStmtPreproArray[] = {
    { prepro_unknown,     NULL},
    { prepro_include,     "include" },


    { prepro_end,         NULL }
};


#define START_OF_MARKS  "-!#%&()*,./:;?[]^{}|~+<=@>"

#define NUMERIC_OF_CHAR "1234567890"

#define IDENTIFIER_OF_CHAR "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

#define EscapeFlg_ON    1
#define EscapeFlg_OFF   0


static int cdmtrGetAvalableIdentifier(void);

/*!
    @brief      g[Nobt@Ɉꕶǉ
*/
int CDMtrStmtTokenBuffADD(char ch)
{
    if (CDMtrStmtTokenLen == CDMtrStmtMaxTokenBufLen) {
        PASS_CHECK;
        if (CDMtrStmtTokenBuffReSize() != 0) {
            PASS_CHECK;
            return -1;
        }
    }
    CDMtrStmtTokenBuff[CDMtrStmtTokenLen] = ch;
    CDMtrStmtTokenLen++;
    return 0;
}


/*!
    @brief      g[Nobt@̃TCYύX
*/
int CDMtrStmtTokenBuffReSize()
{
    char *pOldBuf;
    int   iOldBufSize;
    char *pNewBuf;
    int   iNewBufSize;
    pNewBuf = pOldBuf = NULL;
    iNewBufSize = iOldBufSize = 0;
    pOldBuf = CDMtrStmtTokenBuff;
    iOldBufSize = CDMtrStmtMaxTokenBufLen;
    iNewBufSize = CDMtrStmtMaxTokenBufLen + TOKEN_BUF_LEN;
    pNewBuf     = (char *)malloc(sizeof(char) * (iNewBufSize + 1));
    if (NULL == pNewBuf) {
        PASS_CHECK;
        return -1;
    }
    memset((void *)pNewBuf, 0, iNewBufSize + 1);
    if (NULL != pOldBuf) {
        PASS_CHECK;
        memcpy((void *)pNewBuf, (void *)pOldBuf, iOldBufSize);
        free(pOldBuf);
        pOldBuf = NULL;
    }
    CDMtrStmtTokenBuff = pNewBuf;
    CDMtrStmtMaxTokenBufLen = iNewBufSize;
    return 0;
}


/*!
    @brief  g[Nobt@NA
*/
void CDMtrStmtTokenBuffClear()
{
    if (NULL != CDMtrStmtTokenBuff) {
        PASS_CHECK;
        memset((void *)CDMtrStmtTokenBuff, 0, CDMtrStmtMaxTokenBufLen + 1);
        CDMtrStmtTokenLen = 0;
    }
    return;
}


/*!
    @brief  g[Nobt@
*/
void CDMtrStmtTokenBuffFree()
{
    if (NULL != CDMtrStmtTokenBuff) {
        PASS_CHECK;
        free(CDMtrStmtTokenBuff);
        CDMtrStmtTokenBuff      = NULL;
        CDMtrStmtMaxTokenBufLen = 0;
        CDMtrStmtTokenLen       = 0;
    }
    return;
}


void CDMtrStmtTokenBuffInit()
{
    CDMtrStmtTokenBuffFree();
    CDMtrStmtTokenBuff      = NULL;
    CDMtrStmtMaxTokenBufLen = 0;
    CDMtrStmtTokenLen       = 0;
    PASS_CHECK;
    return;
}


/*!
    @brief  g[No͂
*/
void CDMtrStmtPrintToken(int iTokenType)
{
    struct cdmtrStmt_token_tag *pToken = NULL;
    if (token_unknown <= iTokenType && iTokenType <= token_end) {
        pToken = sCdmtrStmtTokenArray + iTokenType;
        CDMTR_STMT_LOG_DEBUG(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000715, pToken->pcTokenName);); //* MSG_LOG_ID_C_00000715
        if (NULL == pToken->pcTokenDisp) {
            CDMTR_STMT_LOG_DEBUG(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000716, CDMtrStmtTokenBuff);); //* MSG_LOG_ID_C_00000716
        } else {
            CDMTR_STMT_LOG_DEBUG(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000717, pToken->pcTokenDisp);); //* MSG_LOG_ID_C_00000717
        }
    }
    return;
}


/*!
    @brief  g[NIĎ
*/
static int cdmtrStmtSerachToken()
{
    int iTokenLen = 0;
    int iTokenID = 0;
    char *pcToken = NULL;
    int iRet;
    for (iTokenID = token_unknown + 1; iTokenID < token_end; iTokenID++) {
        pcToken = sCdmtrStmtTokenArray[iTokenID].pcTokenStr;
        if (NULL != pcToken) {
            PASS_CHECK;
            iTokenLen = strlen(pcToken);
            if (iTokenLen == CDMtrStmtTokenLen) {
                PASS_CHECK;
                iRet = strncmp(CDMtrStmtTokenBuff, pcToken, iTokenLen);
                if (iRet == 0) {
                    int iRetToken = iTokenID;
                    PASS_CHECK;
                    switch (CDMtrStmtLangType) {
                    case CDMtrStmtLangModeC:
                        PASS_CHECK;
                        switch (iTokenID) {
                        case token_class :
                        case token_interface :
                        case token_try :
                        case token_catch :
                        case token_finally :
                        case token_public :
                        case token_private :
                        case token_protected :
                        case token_foreach :
                        case token_delegate :
                        case token_lock :
                        case token_using :
                        case token_fixed :
                        case token_this :
                        case token_goto :
                        case token_is :
                        case token_as :
                            PASS_CHECK;
                            iRetToken = token_other;
                            break;
                        default:
                            PASS_CHECK;
                            iRetToken = iTokenID;
                            break;
                        }
                        break;
                    case CDMtrStmtLangModeCPP:
                        PASS_CHECK;
                        switch (iTokenID) {
                        case token_interface :
                        case token_public :
                        case token_private :
                        case token_protected :
                        case token_foreach :
                        case token_delegate :
                        case token_lock :
                        case token_using :
                        case token_fixed :
                        case token_goto :
                        case token_is :
                        case token_as :
                            PASS_CHECK;
                            iRetToken = token_other;
                            break;
                        default:
                            PASS_CHECK;
                            iRetToken = iTokenID;
                            break;
                        }
                        break;
                    case CDMtrStmtLangModeJAVA:
                        PASS_CHECK;
                        switch (iTokenID) {
                        case token_struct :
                        case token_union :
                        case token_foreach :
                        case token_delegate :
                        case token_lock :
                        case token_using :
                        case token_fixed :
                        case token_goto :
                        case token_is :
                        case token_as :
                            PASS_CHECK;
                            iRetToken = token_other;
                            break;
                        default:
                            PASS_CHECK;
                            iRetToken = iTokenID;
                            break;
                        }
                        break;
                    case CDMtrStmtLangModeCS:
                        PASS_CHECK;
                        switch (iTokenID) {
                        case token_union :
                        case token_public :
                        case token_private :
                        case token_protected :
                            PASS_CHECK;
                            iRetToken = token_other;
                            break;
                        case token_is :
                        case token_as :
                            PASS_CHECK;
                            iRetToken = token_operator;
                            break;
                        default:
                            PASS_CHECK;
                            iRetToken = iTokenID;
                            break;
                        }
                        break;
                    default:
                        PASS_CHECK;
                        CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000718, CDMtrStmtLangType);); //* MSG_LOG_ID_C_00000718
                        return -1;
                    }
                    return iRetToken;
                }
            }
        }
    }
    return token_other;
}


/*!
    @brief  vvZbTIĎ
*/
static int cdmtrStmtSerachPrepro()
{
    int iPreproLen = 0;
    int i = 0;
    char *pcPrepro = NULL;
    int iRet;
    for (i = 0; i < (sizeof(sCdmtrStmtPreproArray) / sizeof(struct cdmtrStmt_prepro_tag)); i++) {
        pcPrepro = sCdmtrStmtPreproArray[i].pcPreproStr;
        if (NULL != pcPrepro) {
            PASS_CHECK;
            iPreproLen = strlen(pcPrepro);
            if (iPreproLen == CDMtrStmtTokenLen) {
                iRet = strncmp(CDMtrStmtTokenBuff, pcPrepro, iPreproLen);
                if (iRet == 0) {
                    return sCdmtrStmtPreproArray[i].iPreproType;
                }
            }
        }
    }
    return prepro_unknown;
}


static int cdmtrStmtSkipToNextLine()
{
    int iReadSize = 0;
    if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF) {
        PASS_CHECK;
        do {
            iReadSize = CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize);
            iPosWorkBuf = 0;
            if (iReadSize < 0) {
                PASS_CHECK;
                return -1;
            }
        } while (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF);
    }
    for ( ; CDMtrStmtWorkBuf[iPosWorkBuf] != '\0'; iPosWorkBuf++) ;
    if ( (iPosWorkBuf > 0) &&
            (CDMtrStmtWorkBuf[iPosWorkBuf - 1] == '\\') ) {
        PASS_CHECK;
        if (CDMtrStmt_flg_isEOF == CDMtrStmt_FILE_OFF) {
            PASS_CHECK;
            memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
            iReadSize = CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize);
            if (iReadSize < 0) {
                PASS_CHECK;
                return -1;
            }
            iPosWorkBuf = 0;
            return cdmtrStmtSkipToNextLine();
        }
    }
    memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
    iReadSize = CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize);
    if (iReadSize < 0) {
        PASS_CHECK;
        return -1;
    }
    iPosWorkBuf = 0;
    return iReadSize;
}


/*!
    @brief  [Nobt@̓ǂݏo
*/
static int cdmtrStmtGetStrings(char chEnd, int iEscapeFlg)
{
    char cCur;
    char cOld;
    cOld = cCur = 0;
    cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
    if (CDMtrStmtTokenBuffADD(cCur) < 0) {
        PASS_CHECK;
        return -1;
    }
    for ( ; ; ) {
        if (iEscapeFlg == EscapeFlg_OFF) {
            cOld = cCur;
        } else {
            if (cCur == '\\' && cOld == '\\') {
                cOld = -1;
            } else {
                cOld = cCur;
            }
        }
        iPosWorkBuf++;
        cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
        if (cCur == '\0') {
            PASS_CHECK;
            memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
            if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize) < 0) {
                PASS_CHECK;
                return -1;
            }
            if (CDMtrStmt_flg_isEOF == CDMtrStmt_FILE_ON) {
                PASS_CHECK;
                return 0;
            }
            iPosWorkBuf = 0;
            cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
        }
        if (CDMtrStmtTokenBuffADD(cCur) < 0) {
            PASS_CHECK;
            return -1;
        }
        if (iEscapeFlg == EscapeFlg_OFF) {
            if (cCur == chEnd) {
                break;
            }
        } else {
            if ( cCur == chEnd && cOld != '\\' ) {
                break;
            }
        }
    }
    iPosWorkBuf++;
    return 0;
}


/*!
    @brief  RgI܂ł̓ǂݔ΂
*/
static int cdmtrStmtSkipToCommentEnd()
{
    int iReadSize = 0;
    char cCur, cNext;
    cCur = cNext = 0;
    for ( ; ; ) {
        do {
            if (CDMtrStmtWorkBuf[iPosWorkBuf] == '\0') {
                PASS_CHECK;
                if (CDMtrStmt_flg_isEOF == CDMtrStmt_FILE_OFF) {
                    PASS_CHECK;
                    memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                    iReadSize = CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize);
                    if (iReadSize < 0) {
                        PASS_CHECK;
                        return -1;
                    }
                    iPosWorkBuf = 0;
                } else {
                    PASS_CHECK;
                    memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                    iPosWorkBuf = 0;
                    return 0;
                }
            } else {
                PASS_CHECK;
                iPosWorkBuf++;
            }
        } while (CDMtrStmtWorkBuf[iPosWorkBuf] != '*');
        cCur  = CDMtrStmtWorkBuf[iPosWorkBuf];
        cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
        if ( cNext == '\0') {
            PASS_CHECK;
            if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF) {
                PASS_CHECK;
                memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                CDMtrStmtWorkBuf[0] = cCur;
                iReadSize = CDMtrStmtReadOneLine(CDMtrStmtWorkBuf + 1, CDMtrStmtBufSize - 1);
                if (iReadSize < 0) {
                    PASS_CHECK;
                    return -1;
                }
                iPosWorkBuf = 0;
                cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
            } else {
                PASS_CHECK;
                ;
            }
        }
        if ( cNext == '/' ) {
            PASS_CHECK;
            iPosWorkBuf += 2;
            break;
        } else {
            PASS_CHECK;
        }
    }
    return 0;
}


/*!
    @brief  vvZbTw̌ǂݔ΂
*/
static int cdmtrStmtSkipToProcEnd(int iPreproID)
{
    int iRetFunc = 0;
    char cCur, cNext;
    cCur = cNext = 0;
    if ( CDMtrStmtLangType == CDMtrStmtLangModeCS ) {
        iRetFunc = cdmtrStmtSkipToNextLine();
        if (iRetFunc < 0) {
            PASS_CHECK;
            return -1;
        }
    } else {
        for ( ; ; ) {
            cCur  = CDMtrStmtWorkBuf[iPosWorkBuf];
            cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
            if (cCur == '\0') {
                if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF) {
                    memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                    CDMtrStmtWorkBuf[0] = cCur;
                    if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf + 1, CDMtrStmtBufSize - 1) < 0) {
                        PASS_CHECK;
                        return -1;
                    }
                    iPosWorkBuf = 0;
                    cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
                    cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
                } else {
                    break;
                }
            }
            if (cNext == '\0') {
                if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF) {
                    memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                    CDMtrStmtWorkBuf[0] = cCur;
                    if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf + 1, CDMtrStmtBufSize - 1) < 0) {
                        PASS_CHECK;
                        return -1;
                    }
                    iPosWorkBuf = 0;
                    cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
                    cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
                }
            }
            if ( cCur == '\0' ) {
                break;
            } else if ( (cCur == '/') && (cNext == '*') ) {
                PASS_CHECK;
                iPosWorkBuf += 2;
                iRetFunc = cdmtrStmtSkipToCommentEnd();
                if (iRetFunc < 0) {
                    PASS_CHECK;
                    return -1;
                }
            } else if ( (cCur == '/') && (cNext == '/') ) {
                PASS_CHECK;
                iPosWorkBuf += 2;
                iRetFunc = cdmtrStmtSkipToNextLine();
                if (iRetFunc < 0) {
                    PASS_CHECK;
                    return -1;
                }
                break;
            } else if ( (cCur == '\\') && (cNext == '\0') ) {
                memset( (void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                if ( CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize) < 0) {
                    PASS_CHECK;
                    return -1;
                }
                iPosWorkBuf = 0;
            } else if ( cCur == '"' ) {
                iRetFunc = cdmtrStmtGetStrings('"', EscapeFlg_ON);
                if ( iRetFunc < 0 ) {
                    return -1;
                }
                CDMtrStmtTokenBuffClear();
            } else if ( cCur == '\'' ) {
                iRetFunc = cdmtrStmtGetStrings('\'', EscapeFlg_ON);
                if ( iRetFunc < 0 ) {
                    return -1;
                }
                CDMtrStmtTokenBuffClear();
            } else if ( (cCur == '<') && (iPreproID == prepro_include)) {
                iRetFunc = cdmtrStmtGetStrings('>', EscapeFlg_ON);
                if ( iRetFunc < 0 ) {
                    return -1;
                }
                CDMtrStmtTokenBuffClear();
            } else {
                iPosWorkBuf++;
            }
        }
    }
    return 0;
}


/*!
    @brief  Lg[N̎ʂ𔻕ʂ
*/

static int cdmtrStmtGet2CharsMarks()
{
    int iRetToken;
    char cCur, cNext;
    iRetToken = token_unknown;
    cCur  = CDMtrStmtWorkBuf[iPosWorkBuf];
    cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
    if (cNext == '\0') {
        if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF) {
            memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
            CDMtrStmtWorkBuf[0] = cCur;
            if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf + 1, CDMtrStmtBufSize - 1) < 0) {
                PASS_CHECK;
                return -1;
            }
            iPosWorkBuf = 0;
            cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
            cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
        }
    }
    switch (cCur) {
    case ':':
        if (CDMtrStmtLangType == CDMtrStmtLangModeC || CDMtrStmtLangType == CDMtrStmtLangModeJAVA) {
            iRetToken = token_colon;
            iPosWorkBuf += 1;
        } else {
            if (cNext == ':') {
                iRetToken = token_colon2;
                iPosWorkBuf += 2;
            } else {
                iRetToken = token_colon;
                iPosWorkBuf += 1;
            }
        }
        break;
    case '=':
        if (cNext == '=') {
            iRetToken = token_eq2;
            iPosWorkBuf += 2;
        } else {
            iRetToken = token_eq;
            iPosWorkBuf += 1;
        }
        break;
    case '<':
        if (CDMtrStmtLangType == CDMtrStmtLangModeJAVA) {
            if (cNext == '?') {
                iRetToken = token_generic;
                iPosWorkBuf += 2;
            }
        } else {
            iRetToken = token_operator;
            iPosWorkBuf += 1;
        }
        break;
    default:
        iRetToken = token_unknown;
        break;
    }
    return iRetToken;
}


/*!
    @brief  ʎqo
*/
static int cdmtrStmtGetIdentifier()
{
    char cCur;
    cCur     = 0;
    cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
    if (CDMtrStmtTokenBuffADD(cCur) < 0) {
        PASS_CHECK;
        return -1;
    }
    for ( ; ; ) {
        iPosWorkBuf++;
        cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
        if (cCur == '\0') {
            PASS_CHECK;
            if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_ON) {
                PASS_CHECK;
                break;
            } else {
                PASS_CHECK;
                memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize) < 0) {
                    PASS_CHECK;
                    return -1;
                }
                if (CDMtrStmt_flg_isEOF == CDMtrStmt_FILE_ON) {
                    PASS_CHECK;
                    break;
                }
                iPosWorkBuf = 0;
                cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
            }
        }
        if (cCur == ' ' || cCur == '\t') {
            PASS_CHECK;
            break;
        } else {
            char *pIsMark = strrchr(START_OF_MARKS, cCur);
            PASS_CHECK;
            if (pIsMark != NULL) {
                PASS_CHECK;
                break;
            }
        }
        if (CDMtrStmtTokenBuffADD(cCur) < 0) {
            PASS_CHECK;
            return -1;
        }
    }
    return 0;
}

/*!
    @brief  vvZbTʎqo
*/
static int cdmtrStmtGetPPIdentifier()
{
    int iRetFunc = 0;
    char cCur, cNext;
    int iStartIdentifier = 0;
    char *pIsIdentifier, *pIsNum;
    cCur     = 0;
    cNext    = 0;
    pIsIdentifier = NULL;
    pIsNum = NULL;
    iPosWorkBuf++;
    for ( ; ; ) {
        cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
        if (cCur == '\0') {
            PASS_CHECK;
            if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_ON) {
                PASS_CHECK;
                break;
            } else {
                PASS_CHECK;
                memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize) < 0) {
                    PASS_CHECK;
                    return -1;
                }
                if (CDMtrStmt_flg_isEOF == CDMtrStmt_FILE_ON) {
                    PASS_CHECK;
                    break;
                }
                iPosWorkBuf = 0;
                cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
            }
        }
        if (cCur == ' ' || cCur == '\t') {
            PASS_CHECK;
            if (iStartIdentifier == 0) {
                iPosWorkBuf++;
                continue;
            } else {
                break;
            }
        }
        if (iStartIdentifier == 0) {
            cCur  = CDMtrStmtWorkBuf[iPosWorkBuf];
            cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
            if ( cCur == '/' && cNext == '\0') {
                PASS_CHECK;
                if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF) {
                    PASS_CHECK;
                    memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                    CDMtrStmtWorkBuf[0] = cCur;
                    if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf + 1, CDMtrStmtBufSize - 1) < 0) {
                        PASS_CHECK;
                        return -1;
                    }
                    iPosWorkBuf = 0;
                    cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
                } else {
                    PASS_CHECK;
                    ;
                }
            }
            if ( (cCur == '/') && (cNext == '*')) {
                PASS_CHECK;
                iPosWorkBuf += 2;
                iRetFunc = cdmtrStmtSkipToCommentEnd();
                if (iRetFunc < 0) {
                    PASS_CHECK;
                    return -1;
                }
                continue;
            } else if ( (cCur == '/') && (cNext == '/') ) {
                PASS_CHECK;
                break;
            } else if ( (cCur == '\\') && (cNext == '\0') ) {
                PASS_CHECK;
                memset( (void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                if ( CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize) < 0) {
                    PASS_CHECK;
                    return -1;
                }
                iPosWorkBuf = 0;
                continue;
            }
        }
        pIsIdentifier = strrchr(IDENTIFIER_OF_CHAR, cCur);
        pIsNum  = strrchr(NUMERIC_OF_CHAR, cCur);
        if ( (pIsIdentifier != NULL) ||
                ((pIsNum != NULL) && (iStartIdentifier != 0))) {
            if (CDMtrStmtTokenBuffADD(cCur) < 0) {
                PASS_CHECK;
                return -1;
            }
            iStartIdentifier = 1;
        } else {
            break;
        }
        iPosWorkBuf++;
    }
    return 0;
}


/*!
    @brief  g[N擾
*/
int CDMtrStmtGetToken()
{
    int iReadSize = 0;
    int iTokenType;
    int iPreproType;
    int iRetFunc = 0;
    char *pIsMark = NULL;
    char cCur  = '\0';
    char cNext = '\0';
    int iFlgIsStartLineTop = 0;
    iTokenType = token_unknown;
    iPreproType = prepro_unknown;
    CDMtrStmtTokenBuffClear();
    if (CDMtrStmt_flg_isTOL == CDMtrStmt_FILE_ON &&
            iPosWorkBuf == 0) {
        iFlgIsStartLineTop = 1;
    } else {
        iFlgIsStartLineTop = 0;
    }
    for ( ; ; ) {
        while ( (CDMtrStmtWorkBuf[iPosWorkBuf] == ' ')  ||
                (CDMtrStmtWorkBuf[iPosWorkBuf] == '\t') ||
                (CDMtrStmtWorkBuf[iPosWorkBuf] == '\0') ) {
            if (CDMtrStmtWorkBuf[iPosWorkBuf] == '\0') {
                PASS_CHECK;
                if (CDMtrStmt_flg_isEOF == CDMtrStmt_FILE_ON) {
                    PASS_CHECK;
                    return token_end;
                }
                memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                iReadSize = CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize);
                if (iReadSize < 0) {
                    PASS_CHECK;
                    return -1;
                }
                iPosWorkBuf = 0;
                if (CDMtrStmt_flg_isTOL == CDMtrStmt_FILE_ON &&
                        iPosWorkBuf == 0) {
                    iFlgIsStartLineTop = 1;
                }
            } else {
                PASS_CHECK;
                iPosWorkBuf++;
            }
        }
        cCur  = CDMtrStmtWorkBuf[iPosWorkBuf];
        cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
        if ( cCur == '/' && cNext == '\0') {
            PASS_CHECK;
            if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_OFF) {
                PASS_CHECK;
                memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
                CDMtrStmtWorkBuf[0] = cCur;
                iReadSize = CDMtrStmtReadOneLine(CDMtrStmtWorkBuf + 1, CDMtrStmtBufSize - 1);
                if (iReadSize < 0) {
                    PASS_CHECK;
                    return -1;
                }
                iPosWorkBuf = 0;
                cNext = CDMtrStmtWorkBuf[iPosWorkBuf + 1];
            } else {
                PASS_CHECK;
                ;
            }
        }
        if ( (cCur == '/') && (cNext == '*') ) {
            PASS_CHECK;
            iPosWorkBuf += 2;
            iRetFunc = cdmtrStmtSkipToCommentEnd();
            if (iRetFunc < 0) {
                PASS_CHECK;
                return -1;
            }
            iFlgIsStartLineTop = 1;
            continue;
        } else if ( (cCur == '/') && (cNext == '/') ) {
            PASS_CHECK;
            iPosWorkBuf += 2;
            iRetFunc = cdmtrStmtSkipToNextLine();
            if (iRetFunc < 0) {
                PASS_CHECK;
                return -1;
            }
            continue;
        } else if ( (cCur == '#') && (iFlgIsStartLineTop == 1) ) {
            if (cdmtrStmtGetPPIdentifier() < 0) {
                PASS_CHECK;
                return -1;
            }
            iPreproType = cdmtrStmtSerachPrepro();
            if (cdmtrStmtSkipToProcEnd(iPreproType) < 0) {
                return -1;
            }
            CDMtrStmtTokenBuffClear();
            iPreproType = prepro_unknown;
            continue;
        } else if ( (cCur == '#') && iPosWorkBuf == 0 ) {
            PASS_CHECK;
            if (cdmtrStmtGetPPIdentifier() < 0) {
                PASS_CHECK;
                return -1;
            }
            iPreproType = cdmtrStmtSerachPrepro();
            if (cdmtrStmtSkipToProcEnd(iPreproType) < 0) {
                return -1;
            }
            CDMtrStmtTokenBuffClear();
            iPreproType = prepro_unknown;
            continue;
        }
        iTokenType = token_unknown;
        cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
        pIsMark = strrchr(START_OF_MARKS, cCur);
        if (pIsMark != NULL) {
            PASS_CHECK;
            switch (cCur) {
            case '@' :
                if (CDMtrStmtLangType == CDMtrStmtLangModeJAVA) {
                    PASS_CHECK;
                    iTokenType = token_atmark;
                }
                break;
            case '[' :
                PASS_CHECK;
                iTokenType = token_lbracket;
                break;
            case ']' :
                PASS_CHECK;
                iTokenType = token_rbracket;
                break;
            case '(' :
                PASS_CHECK;
                iTokenType = token_lparent;
                break;
            case ')' :
                PASS_CHECK;
                iTokenType = token_rparent;
                break;
            case '{' :
                PASS_CHECK;
                iTokenType = token_lcbrace;
                break;
            case '}' :
                PASS_CHECK;
                iTokenType = token_rcbrace;
                break;
            case ';' :
                PASS_CHECK;
                iTokenType = token_semicolon;
                break;
            case ',' :
                PASS_CHECK;
                iTokenType = token_comma;
                break;
            case '?' :
                PASS_CHECK;
                iTokenType = token_question;
                break;
            case '+' :
            case '-' :
            case '*' :
            case '/' :
            case '%' :
            case '&' :
            case '|' :
            case '^' :
            case '!' :
            case '~' :
            case '>' :
                PASS_CHECK;
                iTokenType = token_operator;
                break;
            default  :
                break;
            }
            if (iTokenType != token_unknown) {
                PASS_CHECK;
                iPosWorkBuf++;
                break;
            }
            if ((CDMtrStmtLangType == CDMtrStmtLangModeCS) && (cCur == '@')) {
                iTokenType = cdmtrGetAvalableIdentifier();
                if ( iTokenType < 0 ) {
                    return -1;
                }
                break;
            }
            iTokenType = cdmtrStmtGet2CharsMarks();
            if (iTokenType != token_unknown) {
                PASS_CHECK;
                break;
            }
            iTokenType = token_other_mark;
            if (CDMtrStmtTokenBuffADD(cCur) < 0) {
                PASS_CHECK;
                return -1;
            }
            iPosWorkBuf++;
            break;
        } else {
            PASS_CHECK;
            if (cCur == '"' || cCur == '\'') {
                PASS_CHECK;
                if (cdmtrStmtGetStrings(cCur, EscapeFlg_ON) < 0) {
                    PASS_CHECK;
                    return -1;
                }
                if (cCur == '"') {
                    PASS_CHECK;
                    iTokenType = token_string;
                } else {
                    PASS_CHECK;
                    iTokenType = token_character;
                }
                break;
            } else {
                PASS_CHECK;
                if (cdmtrStmtGetIdentifier() < 0) {
                    PASS_CHECK;
                    return -1;
                }
                iTokenType = cdmtrStmtSerachToken();
                break;
            }
        }
    }
    CDMTR_STMT_LOG_FUNC(CDMtrStmtPrintToken(iTokenType););
    return iTokenType;
}

/*!
    @brief  vtBNX̎ʎqo
*/
static int cdmtrGetAvalableIdentifier()
{
    char cCur;
    int iTokenType;
    cCur     = 0;
    iTokenType = token_other;
    cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
    if (CDMtrStmtTokenBuffADD(cCur) < 0) {
        PASS_CHECK;
        return -1;
    }
    iPosWorkBuf++;
    cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
    if (cCur == '\0') {
        PASS_CHECK;
        if (CDMtrStmt_flg_isEOL == CDMtrStmt_FILE_ON) {
            PASS_CHECK;
            goto FUNC_END;
        } else {
            PASS_CHECK;
            memset((void *)CDMtrStmtWorkBuf, 0, CDMtrStmtBufSize + 1);
            if (CDMtrStmtReadOneLine(CDMtrStmtWorkBuf, CDMtrStmtBufSize) < 0) {
                PASS_CHECK;
                return -1;
            }
            if (CDMtrStmt_flg_isEOF == CDMtrStmt_FILE_ON) {
                PASS_CHECK;
                goto FUNC_END;
            }
            iPosWorkBuf = 0;
            cCur = CDMtrStmtWorkBuf[iPosWorkBuf];
        }
    }
    if (cCur == ' ' || cCur == '\t') {
        PASS_CHECK;
    } else if (cCur == '"' || cCur == '\'') {
        PASS_CHECK;
        if (cdmtrStmtGetStrings(cCur, EscapeFlg_OFF) < 0) {
            PASS_CHECK;
            return -1;
        }
        if (cCur == '"') {
            PASS_CHECK;
            iTokenType = token_string;
        } else {
            PASS_CHECK;
            iTokenType = token_character;
        }
    } else {
        char *pIsMark = strrchr(START_OF_MARKS, cCur);
        char *pInNum  = strrchr(NUMERIC_OF_CHAR, cCur);
        PASS_CHECK;
        if ((pIsMark != NULL) ||
                (pInNum != NULL)) {
            PASS_CHECK;
        } else {
            PASS_CHECK;
            if (cdmtrStmtGetIdentifier() < 0) {
                PASS_CHECK;
                return -1;
            }
            iTokenType = token_other;
        }
    }
FUNC_END:
    return iTokenType;
}
