%{
    /*----------------------------------------------------------------------------*/
    /* include                                                                    */
    /*----------------------------------------------------------------------------*/
    #include <stdio.h>
    #include <stdlib.h>
    #include <string.h>
    #include "CDMtrVBBison.h"
    #include "CDMtrVBCmn.h"
    #include "CDMtrVBIface.h"
    #include "CDMtrVBtypedef.h"

    /*----------------------------------------------------------------------------*/
    /* define declaration                                                         */
    /*----------------------------------------------------------------------------*/
    /* BISONfobOp */
//  #define YYDEBUG 1 //gp

    #define VBB_TRUE  (1)
    #define VBB_FALSE (0)

    #define VBB_NAME_OPERATOR          "Operator "
    #define VBB_NAME_GET               "Get"
    #define VBB_NAME_SET               "Set"
    #define VBB_NAME_ADDHANDLER        "AddHandler"
    #define VBB_NAME_REMOVEHANDLER     "RemoveHandler"
    #define VBB_NAME_RAISEEVENT        "RaiseEvent"

    /*----------------------------------------------------------------------------*/
    /* Function declaration                                                       */
    /*----------------------------------------------------------------------------*/
    void yyerror(char* s);

    extern int yylineno;

    static int chk_function(int itype);
    static void copy2data(char *pdst, char *psrc);
    static NAME_T *add_name(int itype, char *pcname);

    static void notify_function();
    static void notify_block();
    static void notify_prepro();

    static void set_control_statements(int itype, METRIC_TABLE_T *pmtrtbl);
    static void set_nests_level(int itype, METRIC_TABLE_T *pmtrtbl);

    
    
    

    static int  ibase_phase = 0;

%}


%token VBB_PREPRO VBB_FUNCTION VBB_BLOCK

%start sourcecode
%%

sourcecode
    :
    | sourcecode postfix_expression
    ;
postfix_expression
    : FunctionDeclaration
    | BlockDelaration
    | PreproDelaration
    ;
FunctionDeclaration
    : VBB_FUNCTION             {notify_function();}    
    ;
PreproDelaration
    : VBB_PREPRO               {notify_prepro();}      
    ;
BlockDelaration
    : VBB_BLOCK                {notify_block();}       
    ;

%%

/*----------------------------------------------------------------------------*/
/* Global function                                                            */
/*----------------------------------------------------------------------------*/

/*!
    @brief BisonW[֐
    W[ƃp[XsOɎs邱
*/
void cdmtrVBParsesInit(int iphase)
{
#if YYDEBUG
    yydebug = 1;
#endif

    ibase_phase = iphase;
}

/*!
    @brief BisonG[nh֐
*/
void yyerror(char* s)
{
    printf("ERROR:%s",s);
}

/*----------------------------------------------------------------------------*/
/* Local function                                                             */
/*----------------------------------------------------------------------------*/

/*!
    @brief ֐֐
    ɏꂽ񂩂Flexʒmꂽ֐ʂ֐Ƃ
    F邩`FbNs
*/
static int chk_function(int itype)
{

    switch (itype) {
    case VBIF_TYPE_INTERFACE:
        pParseInfo->iNotBlock |= BIT_CUSTOMEVENT;
        break;
    case VBIF_TYPE_END_INTERFACE:
        pParseInfo->iNotBlock &= ~BIT_CUSTOMEVENT;
        break;
    default:
        if (pParseInfo->iNotBlock) {
           return VBF_FALSE;
        }
    }

    switch (itype) {
    case VBIF_TYPE_CUSTOMEVENT:
        pParseInfo->iCustomEvent = BIT_CUSTOMEVENT;
        break;

    case VBIF_TYPE_END_CUSTOMEVENT:
        pParseInfo->iCustomEvent = BIT_CUSTOMEVENT_NONE;
        break;

    case VBIF_TYPE_ADDHANDLER:
        if (!(pParseInfo->iCustomEvent == BIT_CUSTOMEVENT)) {
            return VBF_FALSE;
        }
        pParseInfo->iCustomEvent |= BIT_ADDHANDLER;
        break;

    case VBIF_TYPE_RAISEEVENT:
        if (!(pParseInfo->iCustomEvent == BIT_CUSTOMEVENT)) {
            return VBF_FALSE;
        }
        pParseInfo->iCustomEvent |= BIT_RAISEEVENT;
        break;

    case VBIF_TYPE_REMOVEHANDLER:
        if (!(pParseInfo->iCustomEvent == BIT_CUSTOMEVENT)) {
            return VBF_FALSE;
        }
        pParseInfo->iCustomEvent |= BIT_REMOVEHANDLER;
        break;

    case VBIF_TYPE_END_ADDHANDLER:
        pParseInfo->iCustomEvent &= ~BIT_ADDHANDLER;
        break;

    case VBIF_TYPE_END_RAISEEVENT:
        pParseInfo->iCustomEvent &= ~BIT_RAISEEVENT;
        break;

    case VBIF_TYPE_END_REMOVEHANDLER:
        pParseInfo->iCustomEvent &= ~BIT_REMOVEHANDLER;
        break;

    default:
        break;
    }

    return VBF_TRUE;
}

/*!
    @brief ֐֐
    Flexʒmꂽ֐Metric/Functione[uXVB
*/
static void notify_function()
{
    int               itype = pFuncInfo->itype;
    METRIC_TABLE_T   *pmtrtbl  = &pMetricTable[pParseInfo->iCurIndex];
    NAME_T           *pname = NULL;
    int               ifuncid = 0;
    FUNCTION_TABLE_T *pfunctbl = NULL;
    int               ilen = 0;
    char              chwk[VBF_STRSIZE_MAX];

    memset(chwk, 0x00, VBF_STRSIZE_MAX);

    if (pmtrtbl->eloc_kind == LINEDTL_ELOC_KIND_SKIP) {
        pmtrtbl->type = itype;
        set_control_statements(itype, pmtrtbl);
        return;
    }

    if (!chk_function(itype)) {
        (*(p_method->pefDebugLog))(VBIF_DEBUGMODE_1, "[L=%d,TYPE=%d]not function\n", pmtrtbl->line_number, itype);
        return;
    }

    (*(p_method->pefDebugLog))(VBIF_DEBUGMODE_1, "[L=%d,TYPE=%d]function\n", pmtrtbl->line_number, itype);

    pmtrtbl->type = itype;

    switch (itype) {
    case VBIF_TYPE_NAMESPACE:
        pmtrtbl->namespace_delim = LINEDTL_NAMESPACE_DELIM_START;
        break;
    case VBIF_TYPE_CLASS:
    case VBIF_TYPE_MODULE:
    case VBIF_TYPE_STRUCTURE:
        pmtrtbl->class_delim = LINEDTL_CLASS_DELIM_START;
        break;
    case VBIF_TYPE_FUNCTION:
    case VBIF_TYPE_SUB:
    case VBIF_TYPE_OPERATOR:
    case VBIF_TYPE_GET:
    case VBIF_TYPE_SET:
    case VBIF_TYPE_ADDHANDLER:
    case VBIF_TYPE_RAISEEVENT:
    case VBIF_TYPE_REMOVEHANDLER:
        pmtrtbl->func_delim = LINEDTL_FUNC_DELIM_START;
        break;

    default:
        break;
    }

    switch (itype) {
    case VBIF_TYPE_NAMESPACE:
    case VBIF_TYPE_CLASS:
    case VBIF_TYPE_MODULE:
    case VBIF_TYPE_STRUCTURE:
    case VBIF_TYPE_CUSTOMEVENT:
    case VBIF_TYPE_PROPERTY:
    case VBIF_TYPE_FUNCTION:
    case VBIF_TYPE_SUB:
        pname = add_name(itype, pFuncInfo->str_name);
        break;
    case VBIF_TYPE_OPERATOR:
        ilen = strlen(pFuncInfo->str_name);
        pFuncInfo->str_name[VBF_STRSIZE_MAX - strlen(VBB_NAME_OPERATOR) - 1] = '\0';
        strcpy(chwk, VBB_NAME_OPERATOR);
        copy2data(&chwk[strlen(VBB_NAME_OPERATOR)], pFuncInfo->str_name);
        pname = add_name(itype, chwk);
        break;
    case VBIF_TYPE_GET:
        pname = add_name(itype, VBB_NAME_GET);
        break;
    case VBIF_TYPE_SET:
        pname = add_name(itype, VBB_NAME_SET);
        break;
    case VBIF_TYPE_ADDHANDLER:
        pname = add_name(itype, VBB_NAME_ADDHANDLER);
        break;
    case VBIF_TYPE_RAISEEVENT:
        pname = add_name(itype, VBB_NAME_RAISEEVENT);
        break;
    case VBIF_TYPE_REMOVEHANDLER:
        pname = add_name(itype, VBB_NAME_REMOVEHANDLER);
        break;
    default:
        (*(p_method->pefDebugLog))(VBIF_DEBUGMODE_NONE, "ERROR : Unknown Function.\n");
        pmtrtbl->status = LINEDTL_STATUS_ERROR_PARSE;
        return;
    }

    if (pname != NULL) {
        pmtrtbl->nameid = pname->iId;
    }
    else {
        (*(p_method->pefDebugLog))(VBIF_DEBUGMODE_NONE, "ERROR : Name is no memory.\n");
        pmtrtbl->status = LINEDTL_STATUS_ERROR_PARSE;
        return;
    }

    switch (itype) {
    case VBIF_TYPE_FUNCTION:
    case VBIF_TYPE_SUB:
    case VBIF_TYPE_OPERATOR:
    case VBIF_TYPE_GET:
    case VBIF_TYPE_SET:
    case VBIF_TYPE_ADDHANDLER:
    case VBIF_TYPE_RAISEEVENT:
    case VBIF_TYPE_REMOVEHANDLER:

        if(p_method->pefGetFuncID != NULL){
            ifuncid = (*(p_method->pefGetFuncID))();
            if (ifuncid == 0) {
                (*(p_method->pefDebugLog))(VBIF_DEBUGMODE_NONE, "ERROR : Func ID is no memory.\n");
                pmtrtbl->status = LINEDTL_STATUS_ERROR_PARSE;
                return;
            }
        }
        pfunctbl = &pFunctionTable[ifuncid - 1];
        pfunctbl->type = itype;
        pfunctbl->func_startline = pmtrtbl->line_number;
        pfunctbl->startindex = pParseInfo->iCurIndex;

        pfunctbl->func_name = pname;

        if (pFuncInfo->idetail & VBF_FUNCDTL_PARAMONLY) {
            if (pFuncInfo->str_param[0] != '\0') {
                int len = strlen(pFuncInfo->str_param);
                pfunctbl->param_name = malloc(sizeof(char) * (len + 1));
                copy2data(pfunctbl->param_name, pFuncInfo->str_param);
            }
        }
        pmtrtbl->func_id = ifuncid;

        pParseInfo->iCurFuncid = ifuncid;
        break;

    default:
        break;
    }

    set_control_statements(itype, pmtrtbl);
    set_nests_level(itype, pmtrtbl);

}

/*!
    @brief ֐̑Ҕ֐
    w肳ꂽÖmۂăRs[B
*/

static NAME_T *add_name(int itype, char *pcname)
{
    NAME_T *pname = NULL;

    pname = (*(p_method->pefGetNameInfo))(itype);

    if (pname != NULL) {
        if (pcname[0] != '\0') {
            copy2data(pname->chName, pcname);
        }
    }

    return pname;
}

/*!
    @brief Rs[֐
    ̃Rs[sB
*/
static void copy2data(char *pdst, char *psrc)
{
    char cbefore  = ' ';
    int  iscpy    = VBB_FALSE;

    if ((pdst == NULL) || (psrc == NULL)) {
        return;
    }

    while (*psrc != '\0') {
        if ((cbefore != ' ') || (*psrc != ' ')) {
            *pdst = *psrc;
            cbefore = *psrc;
            pdst++;
            iscpy = VBB_TRUE;
        }
        psrc++;
    }

    if ((iscpy) && (*(pdst-1) == ' ')) {
            *(pdst-1) = '\0';
    }
    else {
        *pdst = '\0';
    }
}

/*!
    @brief BLOCK֐
    FlexʒmꂽXe[ggMetric/Functione[uXVB
*/
static void notify_block()
{
    int               itype    = pStatInfo->itype;
    int               ifuncid  = 0;
    METRIC_TABLE_T   *pmtrtbl  = &pMetricTable[pParseInfo->iCurIndex];

    (*(p_method->pefDebugLog))(VBIF_DEBUGMODE_1, "[L=%d,TYPE=%d]block\n", pmtrtbl->line_number, itype);

    if (pmtrtbl->eloc_kind == LINEDTL_ELOC_KIND_SKIP) {
        pmtrtbl->type       = itype;
        pmtrtbl->statements = pStatInfo->istatements;
        set_control_statements(itype, pmtrtbl);
        return;
    }

    chk_function(itype);

    pmtrtbl->type       = pStatInfo->itype;
    pmtrtbl->statements = pStatInfo->istatements;

    switch (itype) {

    case VBIF_TYPE_END_NAMESPACE:
        pmtrtbl->namespace_delim = LINEDTL_NAMESPACE_DELIM_END;
        (*(p_method->pefEndNameInfo))();
        break;

    case VBIF_TYPE_END_CLASS:
    case VBIF_TYPE_END_MODULE:
    case VBIF_TYPE_END_STRUCTURE:
        pmtrtbl->class_delim = LINEDTL_CLASS_DELIM_END;
        (*(p_method->pefEndNameInfo))();
        break;

    case VBIF_TYPE_END_CUSTOMEVENT:
    case VBIF_TYPE_END_PROPERTY:
        (*(p_method->pefEndNameInfo))();
        break;

    case VBIF_TYPE_END_FUNCTION:
    case VBIF_TYPE_END_SUB:
    case VBIF_TYPE_END_OPERATOR:
    case VBIF_TYPE_END_GET:
    case VBIF_TYPE_END_SET:
    case VBIF_TYPE_END_ADDHANDLER:
    case VBIF_TYPE_END_RAISEEVENT:
    case VBIF_TYPE_END_REMOVEHANDLER:

        ifuncid = pParseInfo->iCurFuncid;
        pParseInfo->iCurFuncid = 0;      
        if (ifuncid > 0) {

            pmtrtbl->func_id = ifuncid; 
            pmtrtbl->func_delim = LINEDTL_FUNC_DELIM_END;
            (*(p_method->pefEndNameInfo))();

            pFunctionTable[ifuncid - 1].effective = LINEDTL_EFFECTIVE_LINE;
            pFunctionTable[ifuncid - 1].func_endline = pmtrtbl->line_number;
            pFunctionTable[ifuncid - 1].endindex = pmtrtbl->index;
        }
        break;

    default:
        break;
    }

    set_control_statements(itype, pmtrtbl);
    set_nests_level(itype, pmtrtbl);

}

/*!
    @brief PREPRO֐
    FlexʒmꂽPREPROMetric/Functione[uXVB
*/
static void notify_prepro()
{
    int             i = 0;
    METRIC_TABLE_T *pmtrtbl = &pMetricTable[pParseInfo->iCurIndex];

    (*(p_method->pefDebugLog))(VBIF_DEBUGMODE_1, "[L=%d,TYPE=%d]block\n", pmtrtbl->line_number, pStatInfo->itype);

    pmtrtbl->type       = pStatInfo->itype;
    pmtrtbl->eloc_kind  = LINEDTL_ELOC_KIND_PREP;
    pmtrtbl->prep_kind  = pStatInfo->iprepkind;
    pmtrtbl->statements = 0;

    for (i = pParseInfo->iCurIndex+1; i <= pSourceInfo->iMaxIndex; i++) {
        if ((pMetricTable[i-1].line_continuation == LINEDTL_CONTINUATION_LINE) &&
            (pMetricTable[i].eloc_kind == LINEDTL_ELOC_KIND_CODE)) {
            pMetricTable[i].eloc_kind = LINEDTL_ELOC_KIND_PREP;
            pMetricTable[i].prep_kind = pStatInfo->iprepkind;
            pMetricTable[i].statements = 0;
        }
        else {
            break;
        }
    }

}

/*!
    @brief Xe[ggݒ菈֐
    Metric/Functione[ucontrol_statements, 
    cyclowc, branches, cndops̐ݒsB
*/
static void set_control_statements(int itype, METRIC_TABLE_T *pmtrtbl)
{
    switch (itype) {

    case VBIF_TYPE_FUNCTION:
    case VBIF_TYPE_SUB:
    case VBIF_TYPE_OPERATOR:
    case VBIF_TYPE_GET:
    case VBIF_TYPE_SET:
    case VBIF_TYPE_ADDHANDLER:
    case VBIF_TYPE_RAISEEVENT:
    case VBIF_TYPE_REMOVEHANDLER:
        pmtrtbl->control_statements = 1;
        pmtrtbl->cyclowc = 1;
        pmtrtbl->branches = 1;
        break;


    case VBIF_TYPE_DO:     
    case VBIF_TYPE_FOR:    
    case VBIF_TYPE_IF:     
    case VBIF_TYPE_ELSEIF: 
    case VBIF_TYPE_WHILE:  
        pmtrtbl->control_statements = 1;
        pmtrtbl->cyclowc = 1;
        pmtrtbl->branches = pStatInfo->ibranches;
        break;

    case VBIF_TYPE_LOOP:
        pmtrtbl->branches = pStatInfo->ibranches;
        break;

    case VBIF_TYPE_SELECT:
        pmtrtbl->branches = pStatInfo->ibranches;
        pmtrtbl->control_statements = 1;
        break;

    case VBIF_TYPE_CASE:
        pmtrtbl->cyclowc = 1;
        pmtrtbl->branches = 1;
        break;

    case VBIF_TYPE_CASEELSE:
        break;

    case VBIF_TYPE_IF2:
        pmtrtbl->control_statements = 1 + pStatInfo->icontrols;
        pmtrtbl->cyclowc = 1 + pStatInfo->icyclowc;
        pmtrtbl->branches = pStatInfo->ibranches;
        break;

    case VBIF_TYPE_IFOP:
    case VBIF_TYPE_IIF: 
        break;

    default:
        break;
    }

    pmtrtbl->cndops = pStatInfo->icndops;

}


/*!
    @brief lXgݒ菈֐
    Metric/Functione[unests_level̐ݒsB
*/
static void set_nests_level(int itype, METRIC_TABLE_T *pmtrtbl)
{
    switch (itype) {

    case VBIF_TYPE_SUB:
    case VBIF_TYPE_FUNCTION:
    case VBIF_TYPE_OPERATOR:
    case VBIF_TYPE_GET:
    case VBIF_TYPE_SET:
    case VBIF_TYPE_ADDHANDLER:
    case VBIF_TYPE_REMOVEHANDLER:
    case VBIF_TYPE_RAISEEVENT:

    case VBIF_TYPE_WITH:
    case VBIF_TYPE_USING:
    case VBIF_TYPE_TRY:
    case VBIF_TYPE_SYNCLOCK:
    case VBIF_TYPE_ENUM:
    case VBIF_TYPE_INTERFACE:
    case VBIF_TYPE_NAMESPACE:

    case VBIF_TYPE_IF:
    case VBIF_TYPE_FOR:
    case VBIF_TYPE_WHILE:
    case VBIF_TYPE_DO:

    case VBIF_TYPE_SELECT:

        pmtrtbl->nest_level = 1;
        break;

    case VBIF_TYPE_IF2:
        pmtrtbl->nest_level = 1 + pStatInfo->inests;
        break;


    case VBIF_TYPE_END_SUB:
    case VBIF_TYPE_END_FUNCTION:
    case VBIF_TYPE_END_OPERATOR:
    case VBIF_TYPE_END_GET:
    case VBIF_TYPE_END_SET:
    case VBIF_TYPE_END_ADDHANDLER:
    case VBIF_TYPE_END_REMOVEHANDLER:
    case VBIF_TYPE_END_RAISEEVENT:

    case VBIF_TYPE_END_WITH:
    case VBIF_TYPE_END_USING:
    case VBIF_TYPE_END_TRY:
    case VBIF_TYPE_END_SYNCLOCK:
    case VBIF_TYPE_END_ENUM:
    case VBIF_TYPE_END_INTERFACE:
    case VBIF_TYPE_END_NAMESPACE:

    case VBIF_TYPE_END_IF:
    case VBIF_TYPE_NEXT:
    case VBIF_TYPE_END_WHILE:
    case VBIF_TYPE_LOOP:

    case VBIF_TYPE_END_SELECT:

        pmtrtbl->nest_level = -1;
        break;

    default:
        break;
    }

}
