/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "CDMtrVBIface.h"
#include "CDMtrLibOpt.h"
#include "CDMtrLibCmn.h"
#include "CDMtrLibUtil.h"
#include<windows.h>
#include "CDMtrVBMessage.h"


#define OPTION_TARGET_FILE             (1)
#define OPTION_METRIC_FILE_PATH        (2)
#define OPTION_SOURCEID                (3)
#define OPTION_ERROR_OUTPUT            (4)
#define OPTION_DEBUG                   (5)
#define OPTION_DEBUG2                  (6)
#define OPTION_STATEMENT               (7)

#define OPTION_STR_TARGET_FILE         "-t"
#define OPTION_STR_METRIC_FILE_PATH    "-m"
#define OPTION_STR_SOURCEID            "-n"
#define OPTION_STR_ERROR_OUTPUT        "-e"
#define OPTION_STR_DEBUG               "-debug"
#define OPTION_STR_DEBUG2              "-debug2"
#define OPTION_STR_STATEMENT           "-stmt"


static OptionInfo stOptionArray[] = {
    { OPTION_TARGET_FILE,      OPTION_STR_TARGET_FILE,      OPT_HAS_PARAM },
    { OPTION_METRIC_FILE_PATH, OPTION_STR_METRIC_FILE_PATH, OPT_HAS_PARAM },
    { OPTION_SOURCEID,         OPTION_STR_SOURCEID,         OPT_HAS_PARAM },
    { OPTION_ERROR_OUTPUT,     OPTION_STR_ERROR_OUTPUT,     OPT_HAS_PARAM },
    { OPTION_DEBUG2,           OPTION_STR_DEBUG2,           OPT_HAS_NOPARAM },
    { OPTION_DEBUG,            OPTION_STR_DEBUG,            OPT_HAS_NOPARAM },
    { OPTION_STATEMENT,        OPTION_STR_STATEMENT,        OPT_HAS_NOPARAM }
};


PARAM_INFO_T    strParamInfo;


/*!
    VBp[TC֐
*/
int wmain(int argc, wchar_t *argv[])
{
    int iRet = VBIF_RET_OK;
    char *pcParamStr = NULL;
    int iChkRes = 0;
    FILE *errfile = NULL;
    int i = 0;
    char **cArgv = NULL;
    int i_result = 0;
    char *codepage = ENCODING_SYSTEM;
    if (argc > 0) {
        cArgv = (char **)malloc(sizeof(char *) * argc);
        if (cArgv == NULL) {
            iRet = VBIF_RET_NG;
            goto Func_End;
        }
        memset(cArgv, 0x00, sizeof(char *) * argc);
    } else {
        iRet = VBIF_RET_NG;
        goto Func_End;
    }
    for (i = 0; i < argc; i++) {
        i_result = CDMtrLibUtilUNICODEtoMBCS(&cArgv[i], &(argv[i]), codepage);
        if (i_result != VBIF_RET_OK) {
            iRet = VBIF_RET_NG;
            goto Func_End;
        }
    }
    memset(&strParamInfo, 0x00, sizeof(strParamInfo));
    CDMtrLibOptInitGetParameter(stOptionArray, sizeof(stOptionArray) / sizeof(OptionInfo));
    while (OPTION_END != (iChkRes = CDMtrLibOptGetParameter(argc, cArgv, &pcParamStr))) {
        switch (iChkRes) {
        case OPTION_ERROR:
        case OPTION_NOINIT:
            iRet = VBIF_RET_NG;
            goto Func_End;
        case OPTION_TARGET_FILE:
            strParamInfo.pcTargetFile = pcParamStr;
            break;
        case OPTION_METRIC_FILE_PATH:
            strParamInfo.pcMetricFilePath = pcParamStr;
            break;
        case OPTION_SOURCEID:
            strParamInfo.pcSourceID = pcParamStr;
            break;
        case OPTION_ERROR_OUTPUT:
            strParamInfo.pcErrorOutput = pcParamStr;
            break;
        case OPTION_DEBUG:
            strParamInfo.iDebugMode |= VBIF_DEBUGMODE_1;
            break;
        case OPTION_DEBUG2:
            strParamInfo.iDebugMode |= VBIF_DEBUGMODE_2;
            break;
        case OPTION_STATEMENT:
            strParamInfo.iStmtFlg = VBIF_STAT_ON;
            break;
        default:
            break;
        }
    }
    CDMtrLibOptFinishGetParameter();
    if ((strParamInfo.pcErrorOutput == NULL) ||
            (strlen(strParamInfo.pcErrorOutput) == 0)) {
        strParamInfo.errfile = stdout;
    } else {
        errfile = fopen(strParamInfo.pcErrorOutput, "w");
        if (errfile != NULL) {
            strParamInfo.errfile = errfile;
        } else {
            strParamInfo.errfile = stdout;
        }
    }
    if ((strParamInfo.pcTargetFile == NULL)
            || (strParamInfo.pcMetricFilePath == NULL)
            || (strParamInfo.pcSourceID == NULL)
            || (strlen(strParamInfo.pcTargetFile) == 0)
            || (strlen(strParamInfo.pcMetricFilePath) == 0)
            || (strlen(strParamInfo.pcSourceID) == 0)
       ) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000805); //* MSG_LOG_ID_C_00000805
        iRet = VBIF_RET_NG;
        goto Func_End;
    }
    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000806); //* MSG_LOG_ID_C_00000806
    if (cdmtrVBIfaceInit(&strParamInfo) != VBIF_RET_OK) {
        fprintf(pParamInfo->errfile, MSG_LOG_ID_C_00000807); //* MSG_LOG_ID_C_00000807
        iRet = VBIF_RET_NG;
        goto Func_End;
    }
    for (i = VBIF_PHASE_1; i <= VBIF_PHASE_4; i++) {
        fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000808, i); //* MSG_LOG_ID_C_00000808
        if (cdmtrVBIfaceParses(i) != VBIF_RET_OK) {
            fprintf(pParamInfo->errfile, MSG_LOG_ID_C_00000809, i); //* MSG_LOG_ID_C_00000809
            iRet = VBIF_RET_NG;
            goto Func_End;
        }
    }
    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000810); //* MSG_LOG_ID_C_00000810
    if (cdmtrVBIfaceOutput() != VBIF_RET_OK) {
        fprintf(pParamInfo->errfile, MSG_LOG_ID_C_00000811); //* MSG_LOG_ID_C_00000811
        iRet = VBIF_RET_NG;
        goto Func_End;
    }
    fprintf(strParamInfo.errfile, MSG_LOG_ID_C_00000812); //* MSG_LOG_ID_C_00000812
Func_End:
    cdmtrVBIfaceFinal();
    if (errfile != NULL) {
        fclose(errfile);
    }
    if (cArgv != NULL) {
        for (i = 0; i < argc; i++) {
            if (cArgv[i] != NULL) {
                free(cArgv[i]);
                cArgv[i] = NULL;
            }
        }
        free(cArgv);
        cArgv = NULL;
    }
    return iRet;
}
